@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/DOH/Doh/file.c,v 1.21.4.3 2001/12/10 18:39:56 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/DOH/Doh/file.c,v 1.21.4.1 2001/07/30 08:17:21 mkoeppe Exp $";
 
 #include "dohint.h"
 
@@ -169,28 +169,6 @@ File_ungetc(DOH *fo, int ch) {
   return -1;
 }
 
-/* -----------------------------------------------------------------------------
- * File_close()
- *
- * Close the file
- * ----------------------------------------------------------------------------- */
-
-static int
-File_close(DOH *fo) {
-  int ret = 0;
-  DohFile *f = (DohFile *) ObjData(fo);
-  if (f->filep) {
-    ret = fclose(f->filep);
-    f->filep = 0;
-  } else if (f->fd) {
-#ifdef DOH_INTFILE
-    ret = close(f->fd);
-    f->fd = 0;
-#endif
-  }
-  return ret;
-}
-
 static DohFileMethods FileFileMethods = {
   File_read, 
   File_write,
@@ -199,7 +177,7 @@ static DohFileMethods FileFileMethods = {
   File_ungetc,
   File_seek,
   File_tell,
-  File_close,           /* close */
+  0,              /* close */
 };
 
 static DohObjInfo DohFileType = {
@@ -234,7 +212,7 @@ static DohObjInfo DohFileType = {
 static int init = 0;
 
 DOH *
-NewFile(DOH *fn, const char *mode)
+NewFile(DOH *fn, char *mode)
 {
   DohFile *f;
   FILE *file;
