@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/DOH/Doh/fio.c,v 1.22 2000/08/28 20:15:07 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/DOH/Doh/fio.c,v 1.22.4.1 2001/07/30 08:17:21 mkoeppe Exp $";
 
 #include "dohint.h"
 
@@ -265,7 +265,7 @@ DohvPrintf(DOH *so, const char *format, va_list ap)
 	doh = va_arg(ap, DOH *);
 	if (DohCheck(doh)) {
 	  /* Is a DOH object. */
- 	  if (DohIsString(doh)) {
+ 	  if (DohIsString(doh) && (ObjType(doh) == DOHTYPE_STRING)) {
 	    Sval = doh;
 	  } else {
 	    Sval = Str(doh);
@@ -470,10 +470,10 @@ DohSplit(DOH *in, char *chs, int nsplits) {
   int c;
   
   list = NewList();
-  /*  if (String_check(in)) {
+
+  if (DohIsString(in)) {
     Seek(in,0,SEEK_SET);
   }
-  */
 
   while (1) {
     str = NewString("");
