@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/DOH/Doh/string.c,v 1.45.4.2 2001/11/11 03:47:25 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/DOH/Doh/string.c,v 1.45 2000/09/01 15:50:54 mkoeppe Exp $";
 
 #include "dohint.h"
 
@@ -513,9 +513,7 @@ int replace_simple(String *str, char *token, char *rep, int flags, int count, ch
   register char *base;
   int i;
 
-
   /* Figure out if anything gets replaced */
-  if (!strlen(token)) return 0;
 
   base = str->str;
   tokenlen = strlen(token);
@@ -713,7 +711,6 @@ String_replace(DOH *stro, DOH *token, DOH *rep, int flags)
 {
     int count = -1;
     String *str = (String *) ObjData(stro);
-
     if (flags & DOH_REPLACE_FIRST) count = 1;
 
     if (flags & DOH_REPLACE_ID) {
@@ -883,28 +880,3 @@ NewStringf(const DOH *fmt, ...)
   va_end(ap);
   return (DOHString *) r;
 }
-
-/* -----------------------------------------------------------------------------
- * Strcmp()
- * Strncmp()
- * Strstr()
- * Strchr()
- *
- * Some utility functions.
- * ----------------------------------------------------------------------------- */
-
-int Strcmp(const DOHString_or_char *s1, const DOHString_or_char *s2) {
-  return strcmp(Char(s1),Char(s2));
-}
-
-int Strncmp(const DOHString_or_char *s1, const DOHString_or_char *s2, int n) {
-  return strncmp(Char(s1),Char(s2),n);
-}
-
-char *Strstr(const DOHString_or_char *s1, const DOHString_or_char *s2) {
-  return strstr(Char(s1),Char(s2));
-}
-
-char *Strchr(const DOHString_or_char *s1, int ch) {
-  return strchr(Char(s1),ch);
-}
