@@ -8,7 +8,7 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvs/projects/SWIG/Source/DOH/Include/doh.h,v 1.45.4.7 2001/12/10 18:39:56 beazley Exp $
+ * $Header: /cvs/projects/SWIG/Source/DOH/Include/doh.h,v 1.45.4.1 2001/07/30 08:17:21 mkoeppe Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef _DOH_H
@@ -73,7 +73,7 @@ extern void          DohIncref(DOH *obj);
 
 extern DOH    *DohGetattr(DOH *obj, const DOHString_or_char *name);
 extern int     DohSetattr(DOH *obj, const DOHString_or_char *name, const DOHObj_or_char *value);
-extern int     DohDelattr(DOH *obj, const DOHString_or_char *name);
+extern void    DohDelattr(DOH *obj, const DOHString_or_char *name);
 extern DOH    *DohFirstkey(DOH *obj);
 extern DOH    *DohNextkey(DOH *obj);
 extern int     DohGetInt(DOH *obj, const DOHString_or_char *name);
@@ -168,7 +168,6 @@ extern int     DohIsFile(const DOH *obj);
 #define Getc               DohGetc
 #define Putc               DohPutc
 #define Ungetc             DohUngetc
-#define Close              DohClose
 #define vPrintf            DohvPrintf
 #define GetInt             DohGetInt
 #define GetDouble          DohGetDouble
@@ -183,7 +182,6 @@ extern int     DohIsFile(const DOH *obj);
 #define Readline           DohReadline
 #define Replace            DohReplace
 #define Chop               DohChop
-
 #endif
 
 /* -----------------------------------------------------------------------------
@@ -193,11 +191,6 @@ extern int     DohIsFile(const DOH *obj);
 extern DOHString   *NewString(const DOH *c);
 extern DOHString   *NewStringf(const DOH *fmt, ...);
 
-extern int Strcmp(const DOHString_or_char *s1, const DOHString_or_char *s2);
-extern int Strncmp(const DOHString_or_char *s1, const DOHString_or_char *s2, int n);
-extern char *Strstr(const DOHString_or_char *s1, const DOHString_or_char *s2);
-extern char *Strchr(const DOHString_or_char *s1, int ch);
-
 /* String replacement flags */
 
 #define   DOH_REPLACE_ANY         0x01
@@ -205,21 +198,17 @@ extern char *Strchr(const DOHString_or_char *s1, int ch);
 #define   DOH_REPLACE_ID          0x04
 #define   DOH_REPLACE_FIRST       0x08
 
-#define Replaceall(s,t,r)  DohReplace(s,t,r,DOH_REPLACE_ANY)
-#define Replaceid(s,t,r)   DohReplace(s,t,r,DOH_REPLACE_ID)
-
 /* -----------------------------------------------------------------------------
  * Files
  * ----------------------------------------------------------------------------- */
 
-extern DOHFile *NewFile(DOH *file, const char *mode);
+extern DOHFile *NewFile(DOH *file, char *mode);
 extern DOHFile *NewFileFromFile(FILE *f);
 extern DOHFile *NewFileFromFd(int fd);
-extern int      DohClose(DOH *file);
 
-extern int      DohCopyto(DOHFile *input, DOHFile *output);
+extern int  DohCopyto(DOHFile *input, DOHFile *output);
 
-#define Copyto  DohCopyto
+#define Copyto DohCopyto
 
 /* -----------------------------------------------------------------------------
  * List
@@ -240,7 +229,7 @@ extern DOHList   *Hash_keys(DOHHash *);
 
 extern DOHVoid  *NewVoid(void *ptr, void (*del)(void *));
 
-extern DOHList *DohSplit(DOHFile *input, const char *chs, int nsplits);
+extern DOHList *DohSplit(DOHFile *input, char *chs, int nsplits);
 #define Split DohSplit
 
 extern DOH *DohNone;
