@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17 2001/06/19 13:41:49 mkoeppe Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17.2.11 2001/08/28 15:36:41 mkoeppe Exp $";
 
 /***********************************************************************
- * $Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17 2001/06/19 13:41:49 mkoeppe Exp $
+ * $Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx,v 1.68.2.17.2.11 2001/08/28 15:36:41 mkoeppe Exp $
  *
  * guile.cxx
  *
@@ -25,7 +25,9 @@ static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/guile.cxx
 
 #include "mod11.h"
 #include "guile.h"
+#ifndef MACSWIG
 #include "swigconfig.h"
+#endif
 
 static char *guile_usage = (char*)"\
 Guile Options (available with -guile)\n\
@@ -35,18 +37,16 @@ Guile Options (available with -guile)\n\
      -package name   - Set the path of the module [default NULL]\n\
      -linkage lstyle - Use linkage protocol LSTYLE [default `module']\n\
      -procdoc file   - Output procedure documentation to FILE\n\
-\n\
      -procdocformat format - Output procedure documentation in FORMAT;\n\
                              one of `guile-1.4', `plain', `texinfo'\n\
-  The module option does not create a guile module with a separate name\n\
-  space.  It specifies the name of the initialization function and is\n\
-  called a module here so that it is compadible with the rest of SWIG.\n\
+     -scmstub file   - Output Scheme FILE with module declaration and\n\
+                       exports; only with `passive' and `simple' linkage\n\
 \n\
   When unspecified, the default LSTYLE is `simple'.  For native Guile\n\
   module linking (for Guile versions >=1.5.0), use `module'.  Other\n\
-  LSTYLE values are: `passive' for passive linking (no module-handling\n\
-  code), `ltdlmod' for Guile's old dynamic module convention\n\
-  (versions <= 1.4), or `hobbit' for hobbit modules.\n\
+  LSTYLE values are: `passive' for passive linking (no C-level\n\
+  module-handling  code), `ltdlmod' for Guile's old dynamic module\n\
+  convention (versions <= 1.4), or `hobbit' for hobbit modules.\n\
 \n";
 
 // ---------------------------------------------------------------------
@@ -66,11 +66,14 @@ GUILE::GUILE ()
   package = NULL;
   linkage = GUILE_LSTYLE_SIMPLE;
   procdoc = NULL;
+  scmstub = NULL;
   docformat = GUILE_1_4;
   emit_setters = 0;
   struct_member = 0;
   before_return = NULL;
   exported_symbols = NewString("");
+  scmtext = NewString("");
+  Swig_register_filebyname("scheme", scmtext);
 }
 
 // ---------------------------------------------------------------------
@@ -129,7 +132,6 @@ GUILE::parse_args (int argc, char *argv[])
 	printf("%s\n", SWIG_GUILE_RUNTIME);
 	SWIG_exit (EXIT_SUCCESS);
       }
-      /* The upcase variant is a historic artefact. */
       else if (strcmp (argv[i], "-Linkage") == 0
 	       || strcmp (argv[i], "-linkage") == 0) {
         if (argv[i + 1]) {
@@ -154,7 +156,7 @@ GUILE::parse_args (int argc, char *argv[])
       }
       else if (strcmp (argv[i], "-procdoc") == 0) {
 	if (argv[i + 1]) {
-	  procdoc = NewFile(argv[i + 1], "w");
+	  procdoc = NewFile(argv[i + 1], (char *) "w");
 	  Swig_mark_arg (i);
           Swig_mark_arg (i + 1);
 	  i++;
@@ -178,6 +180,16 @@ GUILE::parse_args (int argc, char *argv[])
 	emit_setters = 1;
 	Swig_mark_arg (i);
       }
+      else if (strcmp (argv[i], "-scmstub") == 0) {
+	if (argv[i + 1]) {
+	  scmstub = NewFile(argv[i + 1], (char *) "w");
+	  Swig_mark_arg (i);
+          Swig_mark_arg (i + 1);
+	  i++;
+	} else {
+	  Swig_arg_error();
+        }
+      }
     }
   }
 
@@ -261,11 +273,12 @@ GUILE::headers (void)
 
   Printf (f_runtime, "/* Implementation : GUILE */\n\n");
 
-  // Write out directives and declarations
+  /* Write out directives and declarations */
 
   if (NoInclude) {
     Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
   }
+
 }
 
 // --------------------------------------------------------------------
@@ -363,6 +376,7 @@ GUILE::emit_linkage (char *module_name)
       Printf(f_init, "    SCM module = scm_c_define_module(\"%s\",\n", mod);
       Printf(f_init, "      SWIG_init_helper, NULL);\n");
       Printf(f_init, "    return SCM_UNSPECIFIED;\n");
+      Delete(mod);
     }
     Printf (f_init, "}\n");
     break;
@@ -384,6 +398,30 @@ GUILE::emit_linkage (char *module_name)
   default:
     abort();                            // for now
   }
+
+  if (scmstub) {
+    /* Emit Scheme stub if requested */
+    DOHString *mod = NewString(module_name);
+    Replace(mod, "/", " ", DOH_REPLACE_ANY);
+    Printf (scmstub, ";;; -*- buffer-read-only: t -*- vi: set ro: */\n");
+    Printf (scmstub, ";;; Automatically generated by SWIG; do not edit.\n\n");
+    if (linkage == GUILE_LSTYLE_SIMPLE
+	|| linkage == GUILE_LSTYLE_PASSIVE)
+      Printf (scmstub, "(define-module (%s))\n\n", mod);
+    Delete(mod);
+    Printf (scmstub, "%s", scmtext);
+    if ((linkage == GUILE_LSTYLE_SIMPLE
+	|| linkage == GUILE_LSTYLE_PASSIVE)
+	&& Len(exported_symbols) > 0) {
+      DOHString *ex = NewString(exported_symbols);
+      Replace(ex, ", ", "\n        ", DOH_REPLACE_ANY);
+      Replace(ex, "\"", "", DOH_REPLACE_ANY);
+      Chop(ex);
+      Printf(scmstub, "\n(export %s)\n", ex);
+      Delete(ex);
+    }
+  }
+  
   Delete(module_func);
   if (CPlusPlus) {
     Printf(f_init, "\n}\n");
@@ -419,6 +457,10 @@ GUILE::close (void)
     Delete(procdoc);
     procdoc = NULL;
   }
+  if (scmstub) {
+    Delete(scmstub);
+    scmstub = NULL;
+  }
 }
 
 /* Return true iff T is a pointer type */
@@ -442,7 +484,7 @@ guile_typemap_lookup(const char *op, SwigType *type, const String_or_char *pname
   if (!tm) {
     SwigType *base = SwigType_typedef_resolve_all(type);
     if (strncmp(Char(base), "enum ", 5)==0)
-      tm = Swig_typemap_lookup((char*) op, (char*) "int", (char*)pname, source, target, f);
+      tm = Swig_typemap_lookup((char*) op, NewSwigType(T_INT), (char*)pname, source, target, f);
   }
   return tm;
 }
@@ -461,42 +503,11 @@ guile_do_typemap(DOHFile *file, const char *op,
   if ((tm = guile_typemap_lookup(op, type, arg,
 				 source, target, f))) {
     String *s = NewString(tm);
-    String *descriptor = NewString("");
-    String *basedescriptor = NewString("");
-    String *stardescriptor = NewString("");
     char argnum_s[10];
-    SwigType *startype = NULL;
-
-    if (SwigType_ispointer(type)) {
-      startype = Copy(type);
-      SwigType_del_pointer(startype);
-      Printf(stardescriptor, "SWIGTYPE%s",
-	     SwigType_manglestr(startype));
-    }
-    else Printf(stardescriptor, "SWIGTYPE_BAD");
-    Printf(descriptor, "SWIGTYPE%s",
-	   SwigType_manglestr(type));
-    Printf(basedescriptor, "SWIGTYPE%s",
-	   SwigType_manglestr(SwigType_base(type)));
     sprintf(argnum_s, "%d", argnum);
     Replace(s,"$argnum", argnum_s, DOH_REPLACE_ANY);
     Replace(s,"$arg",    arg,      DOH_REPLACE_ANY);
     Replace(s,"$name",   name,     DOH_REPLACE_ANY);
-    if (Replace(s, "$descriptor",
-		descriptor, DOH_REPLACE_ANY))
-      SwigType_remember(type);
-    if (Replace(s, "$basedescriptor",
-		basedescriptor, DOH_REPLACE_ANY))
-      SwigType_remember(SwigType_base(type));
-    if (Replace(s, "$*descriptor", stardescriptor,
-		DOH_REPLACE_ANY)) {
-      if (!startype) {
-	Printf (stderr, "%s : Line %d. $*descriptor is meaningless for non-pointer types.\n",
-		input_file, line_number);
-	error_count++;
-      }
-      else SwigType_remember(startype);
-    }
     if (nonewline_p)
       Printv(file, s, 0);
     else Printv(file, s, "\n", 0);
@@ -589,8 +600,10 @@ GUILE::create_function (char *name, char *iname, SwigType *d, ParmList *l)
   int numopt = 0;
 
   // Make a wrapper name for this
-  strcpy(wname, Char(Swig_name_wrapper(name)));
-
+  {
+    String *n = Swig_name_mangle(iname);
+    strcpy(wname, Char(Swig_name_wrapper(n)));
+  }
   // Build the name for scheme.
   proc_name = NewString(iname);
   Replace(proc_name,"_", "-", DOH_REPLACE_ANY);
@@ -690,7 +703,7 @@ GUILE::create_function (char *name, char *iname, SwigType *d, ParmList *l)
 
   /* Define the scheme name in C. This define is used by several Guile
      macros. */
-  Printv(f->def, "#define FUNC_NAME \"", proc_name, "\"\n", 0);
+  Printv(f->def, "#define FUNC_NAME \"", proc_name, "\"", 0);
 
   // Now write code to make the function call
   Printv(f->code, tab4, "gh_defer_ints();\n", 0);
@@ -755,7 +768,7 @@ GUILE::create_function (char *name, char *iname, SwigType *d, ParmList *l)
     Printv(f_wrappers, ");\n", 0);
     Printv(f_wrappers, "}\n", 0);
     /* Register it */
-    Printf (f_init, "gh_new_procedure(\"%s\", (SCM (*) ()) %s_rest, 0, 0, 1);\n",
+    Printf (f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s_rest, 0, 0, 1);\n",
              proc_name, wname, numargs-numopt, numopt);
   }
   else if (emit_setters && struct_member && strlen(Char(proc_name))>3) {
@@ -768,7 +781,7 @@ GUILE::create_function (char *name, char *iname, SwigType *d, ParmList *l)
       struct_member = 2; /* have a setter */
     }
     else Printf(f_init, "SCM getter = ");
-    Printf (f_init, "gh_new_procedure(\"%s\", (SCM (*) ()) %s, %d, %d, 0);\n",
+    Printf (f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s, %d, %d, 0);\n",
 	    proc_name, wname, numargs-numopt, numopt);
     if (!is_setter) {
       /* Strip off "-get" */
@@ -792,7 +805,7 @@ GUILE::create_function (char *name, char *iname, SwigType *d, ParmList *l)
   }
   else {
     /* Register the function */
-    Printf (f_init, "gh_new_procedure(\"%s\", (SCM (*) ()) %s, %d, %d, 0);\n",
+    Printf (f_init, "gh_new_procedure(\"%s\", (swig_guile_proc) %s, %d, %d, 0);\n",
 	    proc_name, wname, numargs-numopt, numopt);
   }
   Printf (exported_symbols, "\"%s\", ", proc_name);
@@ -845,42 +858,31 @@ GUILE::link_variable (char *name, char *iname, SwigType *t)
 
   if ((SwigType_type(t) != T_USER) || (is_a_pointer(t))) {
 
-    Printf (f_wrappers, "static SCM %s(SCM s_0)\n{\n", var_name);
+    Printf (f->def, "static SCM\n%s(SCM s_0)\n{\n", var_name);
 
     /* Define the scheme name in C. This define is used by several Guile
        macros. */
-    Printv(f_wrappers, "#define FUNC_NAME \"", proc_name, "\"\n", 0);
+    Printv(f->def, "#define FUNC_NAME \"", proc_name, "\"", 0);
 
-    if (!(Status & STAT_READONLY) && SwigType_type(t) == T_STRING) {
-      Printf (f_wrappers, "\t char *_temp;\n");
-      Printf (f_wrappers, "\t int  _len;\n");
+    Wrapper_add_local (f, "gswig_result", "SCM gswig_result");
+
+    if (!(Status & STAT_READONLY)) {
+      /* Check for a setting of the variable value */
+      Printf (f->code, "if (s_0 != GH_NOT_PASSED) {\n");
+      if (guile_do_typemap(f->code, "varin",
+			   t, name, (char*) "s_0", name, 1, name, f, 0)) {
+	/* nothing */
+      }
+      else {
+	throw_unhandled_guile_type_error (t);
+      }
+      Printf (f->code, "}\n");
     }
-    Printf (f_wrappers, "\t SCM gswig_result;\n");
-
-    // Check for a setting of the variable value
-
-    Printf (f_wrappers, "\t if (s_0 != GH_NOT_PASSED) {\n");
-
-    // Yup. Extract the type from s_0 and set variable value
-
-    if (Status & STAT_READONLY) {
-      Printf (f_wrappers, "\t\t scm_misc_error(\"%s\", "
-	       "\"Unable to set %s. Variable is read only.\", SCM_EOL);\n",
-	       proc_name, proc_name);
-    }
-    else if (guile_do_typemap(f_wrappers, "varin",
-			      t, name, (char*) "s_0", name, 1, name, f, 0)) {
-      /* nothing */
-    }
-    else {
-      throw_unhandled_guile_type_error (t);
-    }
-    Printf (f_wrappers, "\t}\n");
 
     // Now return the value of the variable (regardless
     // of evaluating or setting)
 
-    if (guile_do_typemap (f_wrappers, "varout",
+    if (guile_do_typemap (f->code, "varout",
 			  t, name, name, (char*)"gswig_result",
 			  0, name, f, 1)) {
       /* nothing */
@@ -888,22 +890,25 @@ GUILE::link_variable (char *name, char *iname, SwigType *t)
     else {
       throw_unhandled_guile_type_error (t);
     }
-    Printf (f_wrappers, "\t return gswig_result;\n");
-    Printf (f_wrappers, "#undef FUNC_NAME\n");
-    Printf (f_wrappers, "}\n\n");
+    Printf (f->code, "\nreturn gswig_result;\n");
+    Printf (f->code, "#undef FUNC_NAME\n");
+    Printf (f->code, "}\n");
+
+    Wrapper_print (f, f_wrappers);
 
     // Now add symbol to the Guile interpreter
 
     if (!emit_setters
 	|| Status & STAT_READONLY) {
       /* Read-only variables become a simple procedure returning the
-	 value. */
-      Printf (f_init, "\t gh_new_procedure(\"%s\", (SCM (*) ()) %s, 0, 1, 0);\n",
-	      proc_name, var_name);
+	 value; read-write variables become a simple procedure with
+	 an optional argument. */
+      Printf (f_init, "\t gh_new_procedure(\"%s\", (swig_guile_proc) %s, 0, %d, 0);\n",
+	      proc_name, var_name, (Status & STAT_READONLY) ? 0 : 1);
     }
     else {
       /* Read/write variables become a procedure with setter. */
-      Printf (f_init, "\t{ SCM p = gh_new_procedure(\"%s\", (SCM (*) ()) %s, 0, 1, 0);\n",
+      Printf (f_init, "\t{ SCM p = gh_new_procedure(\"%s\", (swig_guile_proc) %s, 0, 1, 0);\n",
 	      proc_name, var_name);
       Printf (f_init, "\t  gh_define(\"%s\", "
 	      "scm_make_procedure_with_setter(p, p)); }\n",
@@ -962,6 +967,7 @@ GUILE::declare_const (char *name, char *iname, SwigType *type, char *value)
   DOHString *rvalue;
   char   *tm;
   Wrapper *f;
+  SwigType *nctype;
 
   f = NewWrapper();
   Status = STAT_READONLY;      // Enable readonly mode.
@@ -970,11 +976,18 @@ GUILE::declare_const (char *name, char *iname, SwigType *type, char *value)
 
   sprintf (var_name, "%sconst_%s", prefix, name);
 
+  // Strip const qualifier from type if present
+
+  nctype = NewString(type);
+  if (SwigType_isconst(nctype)) {
+    Delete(SwigType_pop(nctype));
+  }
+  
   // Build the name for scheme.
   proc_name = NewString(iname);
   Replace(proc_name,"_", "-", DOH_REPLACE_ANY);
 
-  if ((SwigType_type(type) == T_USER) && (!is_a_pointer(type))) {
+  if ((SwigType_type(nctype) == T_USER) && (!is_a_pointer(nctype))) {
     Printf (stderr, "%s : Line %d.  Unsupported constant value.\n",
              input_file, line_number);
     return;
@@ -982,24 +995,25 @@ GUILE::declare_const (char *name, char *iname, SwigType *type, char *value)
 
   // See if there's a typemap
 
-  if (SwigType_type(type) == T_STRING) {
+  if (SwigType_type(nctype) == T_STRING) {
     rvalue = NewStringf("\"%s\"", value);
-  } else if (SwigType_type(type) == T_CHAR) {
+  } else if (SwigType_type(nctype) == T_CHAR) {
     rvalue = NewStringf("\'%s\'", value);
   } else {
     rvalue = NewString(value);
   }
-  if (guile_do_typemap(f_header, "const", type, name,
+  if (guile_do_typemap(f_header, "const", nctype, name,
 		       Char(rvalue), name, 0, name, f, 0)) {
     /* nothing */
   } else {
     // Create variable and assign it a value
-    Printf (f_header, "static %s %s = %s;\n", SwigType_lstr(type,0),
+    Printf (f_header, "static %s %s = %s;\n", SwigType_lstr(nctype,0),
 	    var_name, rvalue);
   }
   // Now create a variable declaration
-  link_variable (var_name, iname, type);
+  link_variable (var_name, iname, nctype);
   Status = OldStatus;
+  Delete(nctype);
   Delete(proc_name);
   Delete(rvalue);
   DelWrapper(f);
@@ -1026,7 +1040,15 @@ void GUILE::pragma(char *lang, char *cmd, char *value)
     if (strcmp(cmd, (char*)"beforereturn")==0) {
       if (before_return)
 	Delete(before_return);
-      before_return = NewString(value);
+      before_return = value ? NewString(value) : NULL;
     }
   }
 }
+
+void
+GUILE::import_start(char *modname) {
+}
+
+void 
+GUILE::import_end() {
+}
