@@ -14,7 +14,7 @@
  *****************************************************************************/
 
 /**************************************************************************
- * $Header: /cvs/projects/SWIG/Source/Modules1.1/guile.h,v 1.15.2.5 2001/06/18 11:00:04 mkoeppe Exp $
+ * $Header: /cvs/projects/SWIG/Source/Modules1.1/guile.h,v 1.15.2.5.2.2 2001/08/14 17:28:58 mkoeppe Exp $
  *
  * class GUILE
  *
@@ -38,6 +38,8 @@ private:
     GUILE_LSTYLE_HOBBIT                 // use (hobbit4d link)
   } linkage;
   File  *procdoc;
+  File  *scmstub;
+  String *scmtext;
   enum {
     GUILE_1_4,
     PLAIN,
@@ -67,6 +69,8 @@ public :
   void create_command (char *, char *) { };
   void cpp_variable(char *name, char *iname, SwigType *t);
   void pragma(char *lang, char *cmd, char *value);
+  void import_start(char *modname);
+  void import_end();
 };
 
 /* guile.h ends here */
