@@ -9,6 +9,23 @@
 
 #include <ctype.h>
 
+#ifdef NO_STRDUP
+/* not standard ANSI - CodeWarrior does not implement it */
+char* strdup(const char* str);
+
+#include <string.h>
+#include <stdlib.h>
+
+char* strdup(const char* str) {
+  char* out = 0;
+  /* it will cause a leak, I know. */
+  out = (char*) calloc(strlen(str)+1,sizeof(char));
+  if (out)
+    strcpy(out,str);
+  return out;
+}
+#endif
+
 #include "mod11.h"
 #include "java.h"
 
@@ -35,15 +52,16 @@ static FILE   *f_shadow = 0;
 static int    shadow = 0;
 static Hash   *shadow_classes;
 static String *shadow_classdef;
+static String *shadow_code;
 static char   *shadow_variable_name = 0; //Name of a c struct variable or c++ public member variable (may or may not be const)
 static int    classdef_emitted = 0;
-static int    shadow_classdef_emitted = 0;
 static int    have_default_constructor = 0;
 static int    native_func = 0;     // Set to 1 when wrapping a native function
 static int    enum_flag = 0; // Set to 1 when wrapping an enum
 static int    static_flag = 0; // Set to 1 when wrapping a static functions or member variables
-static int    variable_wrapper_flag = 0; // Set to 1 when wrapping a member variable
+static int    variable_wrapper_flag = 0; // Set to 1 when wrapping a nonstatic member variable
 static int    wrapping_member = 0; // Set to 1 when wrapping a member variable/enum/const
+static int    abstract_class_flag = 0;
 static int    jnic = -1;          // 1: use c syntax jni; 0: use c++ syntax jni
 static int    nofinalize = 0;          // for generating finalize methods
 static int    useRegisterNatives = 0;        // Set to 1 when doing stuff with register natives
@@ -102,6 +120,7 @@ void JAVA::SwigToJavaType(SwigType *t, String_or_char *pname, String* java_type,
 
   if(jtype) {
     Printf(java_type, jtype);
+    free(jtype);
   }
   else {
     /* Map type here */
@@ -155,6 +174,7 @@ void JAVA::SwigToJNIType(SwigType *t, String_or_char *pname, String* jni_type) {
 
   if(jtype) {
     Printf(jni_type, jtype);
+    free(jtype);
   }
   else {
     /* Map type here */
@@ -283,8 +303,8 @@ char *JAVA::JavaTypeFromTypemap(char *op, SwigType *t, String_or_char *pname) {
   return strdup(bigbuf);
 }
 
-char *JAVA::makeValidJniName(char *name) {
-  char *c = name;
+char *JAVA::makeValidJniName(const char *name) {
+  const char *c = name;
   char *b = bigbuf;
 
   while(*c) {
@@ -419,6 +439,8 @@ void JAVA::parse_args(int argc, char *argv[]) {
 
   // Add typemap definitions
   typemap_lang = (char*)"java";
+
+  SWIG_config_file("java.i");
 }
 
 // ---------------------------------------------------------------------
@@ -430,6 +452,7 @@ void JAVA::parse_args(int argc, char *argv[]) {
 void JAVA::parse() {
   shadow_classes = NewHash();
   shadow_classdef = NewString("");
+  shadow_code = NewString("");
   registerNativesList = NewString("");
   java_enum_code = NewString("");
   module_extra_code = NewString("");
@@ -474,7 +497,6 @@ void JAVA::headers(void)
   Printf(f_header,"/* Implementation : Java */\n\n");
 
   // Include header file code fragment into the output
-  // if (file::include("java.swg",f_header) == -1) {
   if (Swig_insert_file("java.swg",f_header) == -1) {
     Printf(stderr,"Fatal Error. Unable to locate 'java.swg'.\n");
     SWIG_exit (EXIT_FAILURE);
@@ -495,10 +517,16 @@ void JAVA::initialize()
     SWIG_exit(1);
   }
 
+  String* wrapper_name = NewString("");
+
   if(package) {
     String *s = NewString(package);
+    char *jniname = makeValidJniName(Char(s));
+    Clear(s);
+    Printv(s, jniname, 0);
+    free(jniname);
     Replace(s,".","_", DOH_REPLACE_ANY);
-    Append(s,"_");
+    Append(s, "_");
     c_pkgstr = Swig_copy_string(Char(s));
     Delete(s);
 
@@ -511,19 +539,17 @@ void JAVA::initialize()
     package = c_pkgstr = jni_pkgstr = (char*)"";
   }
 
-  sprintf(bigbuf, "Java_%s%s", c_pkgstr, module);
-  c_pkgstr = Swig_copy_string(bigbuf);
+  char *jniname = makeValidJniName(module);
+  Printf(wrapper_name, "Java_%s%s_%%f", c_pkgstr, jniname);
+  free(jniname);
 
-  if (strchr(module + strlen(module)-1, '_')) //if module has a '_' as last character
-    sprintf(bigbuf, "%s1_%%f", c_pkgstr); //separate double underscore with 1
-  else
-    sprintf(bigbuf, "%s_%%f", c_pkgstr);
-
-  Swig_name_register((char*)"wrapper", Swig_copy_string(bigbuf));
+  Swig_name_register((char*)"wrapper", Char(wrapper_name));
   Swig_name_register((char*)"set", (char*)"set_%v");
   Swig_name_register((char*)"get", (char*)"get_%v");
   Swig_name_register((char*)"member", (char*)"%c_%m");
 
+  Delete(wrapper_name);
+
   // Generate the java class
   sprintf(bigbuf, "%s.java", module);
   if((f_java = fopen(bigbuf, "w")) == 0) {
@@ -562,8 +588,8 @@ void JAVA::emit_classdef() {
       Printf(f_java, "public");
     Printf(f_java, " class %s ", module);
 
-    if (module_baseclass)
-      Printv(f_java, module_baseclass, 0);
+    if (module_baseclass && *Char(module_baseclass))
+      Printf(f_java, "extends %s ", module_baseclass);
     if (module_interfaces)
       Printv(f_java, module_interfaces, " ", 0);
     Printf(f_java, "{\n", module);
@@ -597,6 +623,7 @@ void JAVA::close(void)
 
   Delete(shadow_classes); shadow_classes = NULL;
   Delete(shadow_classdef); shadow_classdef = NULL;
+  Delete(shadow_code); shadow_code = NULL;
   Delete(registerNativesList); registerNativesList = NULL;
   Delete(java_enum_code); java_enum_code = NULL;
   Delete(module_extra_code); module_extra_code = NULL;
@@ -658,7 +685,7 @@ void JAVA::create_function(char *name, char *iname, SwigType *t, ParmList *l)
     Putc(toupper((int) *shadow_variable_name), member_function_name);
     Printf(member_function_name, "%s", shadow_variable_name+1);
 
-    cpp_func(Char(member_function_name), t, l, java_function_name);
+    cpp_func(Char(member_function_name), t, l, java_function_name, NOT_VIRTUAL);
 
     Delete(java_function_name);
     Delete(member_function_name);
@@ -678,7 +705,6 @@ void JAVA::create_function(char *name, char *iname, SwigType *t, ParmList *l)
   // Make a wrapper name for this function
   char *jniname = makeValidJniName(iname);
   String *wname = Swig_name_wrapper(jniname);
-
   free(jniname);
 
   /* Get the java and jni types of the return */
@@ -1064,26 +1090,25 @@ void JAVA::link_variable(char *name, char *iname, SwigType *t)
 void JAVA::declare_const(char *name, char *iname, SwigType *type, char *value) {
   int OldStatus = Status;
   char *tm;
-  FILE *jfile;
   char *jname;
+  DOH *jout;
   Status = STAT_READONLY;
   String *java_type = NewString("");
 
   if(!classdef_emitted) emit_classdef();
 
   if(shadow && wrapping_member) {
-    if(!shadow_classdef_emitted) emit_shadow_classdef();
-    jfile = f_shadow;
+    jout = shadow_code;
     jname = shadow_variable_name;
   } else {
-    jfile = f_java;
+    jout = f_java;
     jname = name;
   }
 
   if ((tm = Swig_typemap_lookup((char*)"const",type,name,name,iname,NULL))) {
     String *str = NewString(tm);
     Replace(str,"$value",value, DOH_REPLACE_ANY);
-    Printf(jfile,"  %s\n\n", str);
+    Printf(jout,"  %s\n\n", str);
     Delete(str);
   } else {
     SwigToJavaType(type, iname, java_type, shadow && wrapping_member);
@@ -1093,7 +1118,7 @@ void JAVA::declare_const(char *name, char *iname, SwigType *type, char *value) {
       The enum implementation is done using a public final static int
       in Java. They are then initialised through a JNI call to c in a Java static block.
       */
-      Printf(jfile, "  public final static %s %s;\n", java_type, jname, value);
+      Printf(jout, "  public final static %s %s;\n", java_type, jname, value);
 
       if(shadow && wrapping_member) {
         Printv(shadow_enum_code, tab4, jname, " = ", module, ".", Swig_name_get(iname), "();\n", 0);
@@ -1112,9 +1137,11 @@ void JAVA::declare_const(char *name, char *iname, SwigType *type, char *value) {
       enum_flag = 0;
     } else {
       if(SwigType_type(type) == T_STRING)
-        Printf(jfile, "  public final static %s %s = \"%s\";\n", java_type, jname, value);
+        Printf(jout, "  public final static %s %s = \"%s\";\n", java_type, jname, value);
+      else if(SwigType_type(type) == T_CHAR)
+        Printf(jout, "  public final static String %s = \"%s\";\n", jname, value);
       else
-        Printf(jfile, "  public final static %s %s = %s;\n", java_type, jname, value);
+        Printf(jout, "  public final static %s %s = %s;\n", java_type, jname, value);
     }
   }
   Delete(java_type);
@@ -1135,6 +1162,18 @@ These pragmas start with 'allshadow' or 'module'
  allshadowinterface - interface (implements) for all shadow classes
  modulemethodmodifiers    - replaces the generated native calls' default modifiers
 */
+
+/* 
+C++ pragmas: pragmas declared within a class or c struct for the shadow class. 
+These pragmas start with 'shadow'
+Valid pragmas:
+ shadowbase      - base (extends) for all java shadow classes
+ shadowcode      - text (java code) is copied verbatim to the shadow class
+ shadowclassmodifiers  - class modifiers for the shadow class
+ shadowimport    - import statement generation for the shadow class
+ shadowinterface - interfaces (extends) for the shadow class
+*/
+
 void JAVA::pragma(char *lang, char *code, char *value) {
   if(strcmp(lang, "java") != 0) return;
 
@@ -1165,11 +1204,11 @@ void JAVA::pragma(char *lang, char *code, char *value) {
   } 
   else if(strcmp(code, "modulebase") == 0) {
     if(shadow && module_baseclass)
-      Printf(module_baseclass, "extends %s ", strvalue);
+      Printf(module_baseclass, "%s", strvalue);
   } 
   else if(strcmp(code, "allshadowbase") == 0) {
     if(shadow && all_shadow_baseclass)
-      Printf(all_shadow_baseclass, "extends %s ", strvalue);
+      Printf(all_shadow_baseclass, "%s", strvalue);
   } 
   else if(strcmp(code, "moduleinterface") == 0) {
     if(shadow && module_interfaces)
@@ -1199,70 +1238,49 @@ void JAVA::pragma(char *lang, char *code, char *value) {
       Printf(stderr,"%s : Line %d. Soon to be deprecated pragma. Please replace with modulemethodmodifiers pragma.\n", input_file, line_number);
     Clear(module_method_modifiers);
     Printv(module_method_modifiers, strvalue, 0);
-  } 
-  else {
+  } else if (shadow) {
+    if (strcmp(code,"shadowcode") == 0) {
+      if (f_shadow && shadow_code)
+        Printf(shadow_code, "%s\n", strvalue);
+      else
+        Printf(stderr,"%s : Line %d. Out of scope pragma.\n", input_file, line_number);
+    } 
+    else if (strcmp(code,"shadowimport") == 0) {
+      if (this_shadow_import)
+        Printf(this_shadow_import, "import %s;\n", strvalue);
+      else
+        Printf(stderr,"%s : Line %d. Out of scope pragma.\n", input_file, line_number);
+    } 
+    else if (strcmp(code,"shadowbase") == 0) {
+      if (this_shadow_baseclass)
+        Printf(this_shadow_baseclass, "%s", strvalue);
+      else
+        Printf(stderr,"%s : Line %d. Out of scope pragma.\n", input_file, line_number);
+    } 
+    else if (strcmp(code,"shadowinterface") == 0) {
+      if (this_shadow_interfaces) {
+        if (!*Char(this_shadow_interfaces))
+          Printf(this_shadow_interfaces, "implements %s", strvalue);
+        else
+          Printf(this_shadow_interfaces, ", %s", strvalue);
+      }
+      else
+        Printf(stderr,"%s : Line %d. Out of scope pragma.\n", input_file, line_number);
+    } 
+    else if (strcmp(code,"shadowclassmodifiers") == 0) {
+      if (this_shadow_class_modifiers)
+        Printv(this_shadow_class_modifiers, strvalue, 0);
+      else
+        Printf(stderr,"%s : Line %d. Out of scope pragma.\n", input_file, line_number);
+    }  else {
+      Printf(stderr,"%s : Line %d. Unrecognized pragma.\n", input_file, line_number);
+    }
+  } else {
     Printf(stderr,"%s : Line %d. Unrecognized pragma.\n", input_file, line_number);
   }
   Delete(strvalue);
 }
 
-// ---------------------------------------------------------------------
-// C++ Handling
-//
-// The following functions provide some support for C++ classes and
-// C structs.
-// ---------------------------------------------------------------------
-
-/* 
-C++ pragmas: pragmas declared within a class or c struct for the shadow class. 
-These pragmas start with 'shadow'
-Valid pragmas:
- shadowbase      - base (extends) for all java shadow classes
- shadowcode      - text (java code) is copied verbatim to the shadow class
- shadowclassmodifiers  - class modifiers for the shadow class
- shadowimport    - import statement generation for the shadow class
- shadowinterface - interfaces (extends) for the shadow class
-*/
-void JAVA::cpp_pragma(Pragma* plist) {
-  while (plist) {
-    if ( (strcmp(Char(plist->lang),(char*)"java") == 0) && shadow) {
-      String* strvalue = NewString(plist->value);
-      Replace(strvalue,"\\\"", "\"", DOH_REPLACE_ANY);
-
-      if (strcmp(Char(plist->name),"shadowcode") == 0) {
-        if (this_shadow_extra_code)
-          Printf(this_shadow_extra_code, "%s\n", strvalue);
-      } 
-      else if (strcmp(Char(plist->name),"shadowimport") == 0) {
-        if (this_shadow_import)
-          Printf(this_shadow_import, "import %s;\n", strvalue);
-      } 
-      else if (strcmp(Char(plist->name),"shadowbase") == 0) {
-        if (this_shadow_baseclass)
-          Printf(this_shadow_baseclass, "extends %s ", strvalue);
-      } 
-      else if (strcmp(Char(plist->name),"shadowinterface") == 0) {
-        if (this_shadow_interfaces) {
-          if (!*Char(this_shadow_interfaces))
-            Printf(this_shadow_interfaces, "implements %s", strvalue);
-          else
-            Printf(this_shadow_interfaces, ", %s", strvalue);
-        }
-      } 
-      else if (strcmp(Char(plist->name),"shadowclassmodifiers") == 0) {
-        if (this_shadow_class_modifiers)
-          Printv(this_shadow_class_modifiers, strvalue, 0);
-      } 
-      else {
-        Printf(stderr,"%s : Line %d. Unrecognized pragma for shadow class.\n", plist->filename, plist->lineno);
-      }
-
-      Delete(strvalue);
-    }
-    plist = plist->next;
-  }
-}
-
 void JAVA::add_typedef(SwigType *t, char *name) {
   if(!shadow) return;
   if (is_shadow(t)) {
@@ -1302,13 +1320,11 @@ void JAVA::cpp_open_class(char *classname, char *rename, char *ctype, int strip)
     Printf(f_shadow, "package %s;\n\n", package);
   else 
     Printf(f_shadow, "import %s;\n", module);
-  if(all_shadow_import)
-    Printf(f_shadow, "%s", all_shadow_import);
 
   Clear(shadow_classdef);
-  Printv(shadow_classdef, "$classmodifiers class ", shadow_classname, " $bases", "{\n", 0);
+  Clear(shadow_code);
 
-  shadow_classdef_emitted = 0;
+  abstract_class_flag = 0;
   have_default_constructor = 0;
   shadow_enum_code = NewString("");
   this_shadow_baseclass =  NewString("");
@@ -1322,17 +1338,43 @@ void JAVA::cpp_open_class(char *classname, char *rename, char *ctype, int strip)
 }
 
 void JAVA::emit_shadow_classdef() {
-  String* temp_str = NewString("");
-  if(this_shadow_baseclass)
-    Printv(temp_str, this_shadow_baseclass, 0);
-  if(all_shadow_baseclass)
-    Printv(temp_str, all_shadow_baseclass, 0);
+  String* baseclass = NULL;
+
+  // Import statements
+  if(all_shadow_import)
+    Printf(shadow_classdef, "%s", all_shadow_import);
+  if(this_shadow_import)
+    Printf(shadow_classdef, "%s", this_shadow_import);
+  Printf(shadow_classdef, "\n");
+
+  // Class modifiers
+  if (this_shadow_class_modifiers && *Char(this_shadow_class_modifiers))
+    Printv(shadow_classdef, this_shadow_class_modifiers, 0);
+  else if (all_shadow_class_modifiers && *Char(all_shadow_class_modifiers))
+    Printv(shadow_classdef, all_shadow_class_modifiers, 0);
+  else
+    Printv(shadow_classdef, "public", 0);
+  if (abstract_class_flag)
+    Printv(shadow_classdef, " abstract", 0);
+  Printf(shadow_classdef, " class $class ");
+
+  // Inherited classes
+  if(this_shadow_baseclass && *Char(this_shadow_baseclass)) {
+    Printf(shadow_classdef, "extends %s ", this_shadow_baseclass);
+    baseclass = this_shadow_baseclass;
+  }
+  if(all_shadow_baseclass && *Char(all_shadow_baseclass)) {
+    Printf(shadow_classdef, "extends %s ", all_shadow_baseclass);
+    baseclass = all_shadow_baseclass;
+  }
+
+  // Implemented interfaces
   if(this_shadow_interfaces && *Char(this_shadow_interfaces))
-    Printv(temp_str, this_shadow_interfaces, " ", 0);
-  Replace(shadow_classdef,"$bases", temp_str, DOH_REPLACE_ANY);
+    Printv(shadow_classdef, this_shadow_interfaces, " ", 0);
+  Printf(shadow_classdef, "{\n");
 
   //Display warning on attempt to use multiple inheritance
-  char* search_str = Char(temp_str);
+  char* search_str = Char(shadow_classdef);
   int count = 0;
   while(search_str = strstr(search_str, "extends")) {
     search_str += strlen("extends");
@@ -1341,26 +1383,50 @@ void JAVA::emit_shadow_classdef() {
   if (count > 1)
     Printf(stderr, "Warning for shadow class %s: Multiple inheritance is not supported in Java.\n", shadow_classname);
 
-  if (this_shadow_class_modifiers && *Char(this_shadow_class_modifiers))
-    Replace(shadow_classdef, "$classmodifiers", this_shadow_class_modifiers, DOH_REPLACE_ANY);
-  else if (all_shadow_class_modifiers && *Char(all_shadow_class_modifiers))
-    Replace(shadow_classdef, "$classmodifiers", all_shadow_class_modifiers, DOH_REPLACE_ANY);
-  else
-    Replace(shadow_classdef, "$classmodifiers", "public", DOH_REPLACE_ANY);
+  // Different code depending on whether or not the base class is a SWIG shadow class
+  if (baseclass && is_shadow(baseclass)) {
+    Printv(shadow_classdef,
+      "  public $class(long cPointer, boolean cMemoryOwn) {\n",
+      "    super(cPointer, cMemoryOwn);\n",
+      "  }\n",
+      "\n", 0);
+
+    if(!have_default_constructor) {
+      Printv(shadow_classdef, 
+        "  protected $class() {\n",
+        "  }\n",
+        "\n", 0);
+    }
+    // Control which super constructor is called - we don't want 2 malloc/new c/c++ calls
+    Replace(shadow_code, "$superconstructorcall", "    super(0, false);\n", DOH_REPLACE_ANY);
+  }
+  else {
+    Printv(shadow_classdef,
+      "  protected long _cPtr;\n",
+      "  protected boolean _cMemOwn;\n",
+      "\n",
+      "  public $class(long cPointer, boolean cMemoryOwn) {\n",
+      "    _cPtr = cPointer;\n",
+      "    _cMemOwn = cMemoryOwn;\n",
+      "  }\n",
+      "\n",
+      "  public long getCPtr() {\n",
+      "    return _cPtr;\n",
+      "  }\n",
+      "\n", 0);
+
+    if(!have_default_constructor) {
+      Printv(shadow_classdef, 
+        "  protected $class() {\n",
+        "    _cPtr = 0;\n",
+        "    _cMemOwn = false;\n",
+        "  }\n",
+        "\n", 0);
+    }
+    // No explicit super constructor call as this class does not have a SWIG base class.
+    Replace(shadow_code, "$superconstructorcall", "", DOH_REPLACE_ANY);
+  }
 
-  Printv(shadow_classdef,
-    "  private long _cPtr = 0;\n",
-    "  private boolean _cMemOwn = false;\n",
-    "\n",
-    "  public $class(long cPointer, boolean cMemoryOwn) {\n",
-    "    _cPtr = cPointer;\n",
-    "    _cMemOwn = cMemoryOwn;\n",
-    "  }\n",
-    "\n",
-    "  public long getCPtr() {\n",
-    "    return _cPtr;\n",
-    "  };\n",
-    "\n", 0);
   Replace(shadow_classdef, "$class", shadow_classname, DOH_REPLACE_ANY);
 
   if (all_shadow_extra_code)
@@ -1368,28 +1434,15 @@ void JAVA::emit_shadow_classdef() {
 
   if (this_shadow_extra_code)
     Printv(shadow_classdef, this_shadow_extra_code, 0);
-
-  if(this_shadow_import)
-    Printf(f_shadow, "%s", this_shadow_import);
-  Printf(f_shadow, "\n");
-
-  Printv(f_shadow, shadow_classdef,0);
-  shadow_classdef_emitted = 1;
-  Delete(temp_str);
 }
 
 void JAVA::cpp_close_class() {
   this->Language::cpp_close_class();
   if(!shadow) return;
 
-  if(!shadow_classdef_emitted) emit_shadow_classdef();
+  emit_shadow_classdef();
 
-  // No default constructor implies class is abstract.
-  // Note that abstract in cplus.cxx has this information - it ought to be passed to each
-  // module when the class is opened.
-  if(have_default_constructor == 0) {
-    Printf(f_shadow, "  protected %s() {}\n\n", shadow_classname);
-  }
+  Printv(f_shadow, shadow_classdef, shadow_code, 0);
 
   // Write the enum initialisation code in a static block.
   // These are all the enums defined within the c++ class.
@@ -1414,31 +1467,23 @@ void JAVA::cpp_close_class() {
 void JAVA::cpp_member_func(char *name, char *iname, SwigType *t, ParmList *l) {
   this->Language::cpp_member_func(name,iname,t,l);
 
-/*
-// TODO: modify output so that pure virtual functions are declared abstract and 
-// have no body. Also make the class declaration abstract. Make other constructor protected.
-extern int IsVirtual;
-  printf("IsVirtual: %d [%s] %s\n", IsVirtual, name, shadow_classname);
-  if (IsVirtual == PURE_VIRTUAL) {
-  }
-*/
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     char* realname = iname ? iname : name;
     String* java_function_name = Swig_name_member(shadow_classname, realname);
 
-    cpp_func(iname, t, l, java_function_name);
+    cpp_func(iname, t, l, java_function_name, IsVirtual);
   }
 }
 
 void JAVA::cpp_static_func(char *name, char *iname, SwigType *t, ParmList *l) {
   this->Language::cpp_static_func(name,iname,t,l);
 
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     char* realname = iname ? iname : name;
     String* java_function_name = Swig_name_member(shadow_classname, realname);
 
     static_flag = 1;
-    cpp_func(iname, t, l, java_function_name);
+    cpp_func(iname, t, l, java_function_name, NOT_VIRTUAL);
     static_flag = 0;
   }
 }
@@ -1450,19 +1495,23 @@ C++ static functions map to java static functions.
 iname is the name of the java class wrapper function, which in turn will call 
 java_function_name, the java function which wraps the c++ function.
 */
-void JAVA::cpp_func(char *iname, SwigType *t, ParmList *l, String* java_function_name) {
+void JAVA::cpp_func(char *iname, SwigType *t, ParmList *l, String* java_function_name, int is_virtual) {
   char     arg[256];
   String   *nativecall = NewString("");
   String   *shadowrettype = NewString("");
   String   *user_arrays = NewString("");
 
   if(!shadow) return;
-  if(!shadow_classdef_emitted) emit_shadow_classdef();
 
   /* Get the java return type */
   SwigToJavaType(t, iname, shadowrettype, shadow);
 
-  Printf(f_shadow, "  public %s %s %s(", (static_flag ? "static":""), shadowrettype, iname);
+  Printf(shadow_code, "  public ");
+  if (static_flag)
+    Printf(shadow_code, "static ");
+  if (is_virtual == PURE_VIRTUAL)
+    Printf(shadow_code, "abstract ");
+  Printf(shadow_code, "%s %s(", shadowrettype, iname);
 
   if(SwigType_type(t) == T_ARRAY && is_shadow(get_array_type(t))) {
     Printf(nativecall, "long[] cArray = ");
@@ -1478,7 +1527,7 @@ void JAVA::cpp_func(char *iname, SwigType *t, ParmList *l, String* java_function
 
   Printv(nativecall, module, ".", java_function_name, "(", 0);
   if (!static_flag)
-    Printv(nativecall, "getCPtr()", 0);
+    Printv(nativecall, "_cPtr", 0);
 
   int gencomma = !static_flag;
   int pcount = ParmList_len(l);
@@ -1529,9 +1578,9 @@ DelWrapper(f);
   
       /* Add to java shadow function header */
       if (gencomma >= 2)
-        Printf(f_shadow, ", ");
+        Printf(shadow_code, ", ");
       gencomma = 2;
-      Printf(f_shadow, "%s %s", javaparamtype, arg);
+      Printf(shadow_code, "%s %s", javaparamtype, arg);
 
       Delete(javaparamtype);
     }
@@ -1561,7 +1610,10 @@ DelWrapper(f);
         break;
       case T_REFERENCE:
       case T_POINTER:
-        Printf(nativecall, "), false");
+        if (NewObject) // %new indicating Java must take responsibility for memory ownership
+          Printf(nativecall, "), true");
+        else
+          Printf(nativecall, "), false");
         break;
       default:
         Printf(stderr, "Internal Error: unknown shadow type: %s\n", SwigType_str(t,0));
@@ -1571,10 +1623,17 @@ DelWrapper(f);
     Printf(nativecall,");\n");
   }
 
-  Printf(f_shadow, ") {\n");
-  Printv(f_shadow, user_arrays, 0);
-  Printf(f_shadow, "\t%s", nativecall);
-  Printf(f_shadow, "  }\n\n");
+  if (is_virtual == PURE_VIRTUAL) {
+    Printf(shadow_code, ");\n\n");
+    abstract_class_flag = 1;
+  }
+  else {
+    Printf(shadow_code, ") {\n");
+    Printv(shadow_code, user_arrays, 0);
+    Printf(shadow_code, "    %s", nativecall);
+    Printf(shadow_code, "  }\n\n");
+  }
+
   Delete(shadowrettype);
   Delete(nativecall);
   Delete(user_arrays);
@@ -1583,19 +1642,17 @@ DelWrapper(f);
 void JAVA::cpp_constructor(char *name, char *iname, ParmList *l) {
   this->Language::cpp_constructor(name,iname,l);
 
-  if(shadow && !is_multiple_definition()) {
-    if(!shadow_classdef_emitted) emit_shadow_classdef();
-  
+  if(shadow) {
     String *nativecall = NewString("");
     char arg[256];
   
-    Printf(f_shadow, "  public %s(", shadow_classname);
+    Printf(shadow_code, "  public %s(", shadow_classname);
   
-    Printv(nativecall, "    if(getCPtr() == 0) {\n", 0);
+    Printv(nativecall, "$superconstructorcall", 0); // Super call for filling in later.
     if (iname != NULL)
-      Printv(nativecall, tab8, " _cPtr = ", module, ".", Swig_name_construct(iname), "(", 0);
+      Printv(nativecall, tab4, "_cPtr = ", module, ".", Swig_name_construct(iname), "(", 0);
     else
-      Printv(nativecall, tab8, " _cPtr = ", module, ".", Swig_name_construct(shadow_classname), "(", 0);
+      Printv(nativecall, tab4, "_cPtr = ", module, ".", Swig_name_construct(shadow_classname), "(", 0);
   
     int pcount = ParmList_len(l);
     if(pcount == 0)  // We must have a default constructor
@@ -1623,25 +1680,23 @@ void JAVA::cpp_constructor(char *name, char *iname, ParmList *l) {
       SwigToJavaType(pt, pn, javaparamtype, shadow); 
   
       /* Add to java shadow function header */
-      Printf(f_shadow, "%s %s", javaparamtype, arg);
+      Printf(shadow_code, "%s %s", javaparamtype, arg);
   
       if(i != pcount-1) {
         Printf(nativecall, ", ");
-        Printf(f_shadow, ", ");
+        Printf(shadow_code, ", ");
       }
       Delete(javaparamtype);
     }
   
-  
-    Printf(f_shadow, ") {\n");
+    Printf(shadow_code, ") {\n");
     Printv(nativecall,
-  	 ");\n",
-  	 tab8, " _cMemOwn = true;\n",
-  	 "    }\n",
-  	 0);
+  	  ");\n",
+  	  tab4, "_cMemOwn = true;\n",
+  	  0);
   
-    Printf(f_shadow, "%s", nativecall);
-    Printf(f_shadow, "  }\n\n");
+    Printf(shadow_code, "%s", nativecall);
+    Printf(shadow_code, "  }\n\n");
     Delete(nativecall);
   }
 }
@@ -1649,23 +1704,21 @@ void JAVA::cpp_constructor(char *name, char *iname, ParmList *l) {
 void JAVA::cpp_destructor(char *name, char *newname) {
   this->Language::cpp_destructor(name,newname);
 
-  if(shadow && !is_multiple_definition()) {
-    if(!shadow_classdef_emitted) emit_shadow_classdef();
-  
+  if(shadow) {
     char *realname = (newname) ? newname : name;
   
     if(!nofinalize) {
-      Printf(f_shadow, "  protected void finalize() {\n");
-      Printf(f_shadow, "    _delete();\n");
-      Printf(f_shadow, "  };\n\n");
+      Printf(shadow_code, "  protected void finalize() {\n");
+      Printf(shadow_code, "    _delete();\n");
+      Printf(shadow_code, "  }\n\n");
     }
   
-    Printf(f_shadow, "  public void _delete() {\n");
-    Printf(f_shadow, "    if(getCPtr() != 0 && _cMemOwn) {\n");
-    Printf(f_shadow, "\t%s.%s(_cPtr);\n", module, Swig_name_destroy(shadow_classname));
-    Printf(f_shadow, "\t_cPtr = 0;\n");
-    Printf(f_shadow, "    }\n");
-    Printf(f_shadow, "  }\n\n");
+    Printf(shadow_code, "  public void _delete() {\n");
+    Printf(shadow_code, "    if(_cPtr!=0 && _cMemOwn) {\n");
+    Printf(shadow_code, "      %s.%s(_cPtr);\n", module, Swig_name_destroy(shadow_classname));
+    Printf(shadow_code, "      _cPtr = 0;\n");
+    Printf(shadow_code, "    }\n");
+    Printf(shadow_code, "  }\n\n");
   }
 }
 
@@ -1688,7 +1741,6 @@ void JAVA::cpp_class_decl(char *name, char *rename, char *type) {
 
 void JAVA::cpp_inherit(char **baseclass, int) {
   this->Language::cpp_inherit(baseclass, 0);
-
   if(!shadow) return;
 
   int i = 0;
@@ -1704,7 +1756,7 @@ void JAVA::cpp_inherit(char **baseclass, int) {
     String *b = NewString(baseclass[i]);
     bc = Char(is_shadow(b));
     if (bc && this_shadow_baseclass)
-      Printf(this_shadow_baseclass, "extends %s ", bc);
+      Printf(this_shadow_baseclass, "%s", bc);
     Delete(b);
   }
 }
@@ -1722,16 +1774,11 @@ void JAVA::cpp_variable(char *name, char *iname, SwigType *t) {
 void JAVA::cpp_static_var(char *name, char *iname, SwigType *t) {
   shadow_variable_name = Swig_copy_string((iname) ? iname : name);
 
-/*
   wrapping_member = 1;
   static_flag = 1;
-  variable_wrapper_flag = 1;
   this->Language::cpp_static_var(name, iname, t);
   wrapping_member = 0;
   static_flag = 0;
-  variable_wrapper_flag = 0;
-*/
-  Printf(stderr, "Ignoring %s::%s. Static member variables are broken in SWIG!\n", shadow_classname, shadow_variable_name);
 }
 
 void JAVA::cpp_declare_const(char *name, char *iname, SwigType *type, char *value) {
@@ -1743,4 +1790,9 @@ void JAVA::cpp_declare_const(char *name, char *iname, SwigType *type, char *valu
 }
 
 
+void JAVA::import_start(char *modulename) {
+}
+
+void JAVA::import_end() {
+}
 
