@@ -14,7 +14,7 @@
  *******************************************************************************/
 
 /**************************************************************************
- * $Header: /cvs/projects/SWIG/Source/Modules1.1/mzscheme.h,v 1.6.4.3 2001/11/12 02:50:54 beazley Exp $
+ * $Header: /cvs/projects/SWIG/Source/Modules1.1/mzscheme.h,v 1.6.4.1 2001/08/23 12:47:12 mkoeppe Exp $
  *
  * class MZSCHEME
  *
@@ -28,14 +28,17 @@ class MZSCHEME : public Language
 private:
   void   get_pointer(DOHString_or_char *name, int parm, SwigType *t, Wrapper *f);
 public :
-  virtual void main (int, char *argv[]);
-  virtual int top(Node *); 
-  virtual int functionWrapper(Node *);
-  virtual int constantWrapper(Node *);
-  virtual int variableWrapper(Node *);
-
+  void parse_args (int, char *argv[]);
+  void parse ();
+  void create_function (char *, char *, SwigType *, ParmList *);
+  void link_variable (char *, char *, SwigType *);
+  void declare_const (char *, char *, SwigType *, char *);
+  void initialize ();
+  void headers (void);
+  void close (void);
   void set_module (char *);
+  void set_init (char *);
+  void create_command (char *, char *) { };
   void import_start(char *modname);
   void import_end();
 };
-
