@@ -15,11 +15,13 @@
 /* DB: I had to take some features related to package naming out of this to
    get the new type system to work.   These need to be put back in at some point. */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/perl5.cxx,v 1.45.2.3 2001/06/29 16:43:48 mkoeppe Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/perl5.cxx,v 1.45.2.3.2.16 2001/09/20 21:47:50 beazley Exp $";
 
 #include "mod11.h"
 #include "perl5.h"
+#ifndef MACSWIG
 #include "swigconfig.h"
+#endif
 
 static char *usage = (char*)"\
 Perl5 Options (available with -perl5)\n\
@@ -29,9 +31,12 @@ Perl5 Options (available with -perl5)\n\
      -package name   - Set package prefix\n\
      -static         - Omit code related to dynamic loading.\n\
      -shadow         - Create shadow classes.\n\
+     -const          - Wrap constants as constants and not variables (implies -shadow).\n\
      -compat         - Compatibility mode.\n\n";
 
 static String *import_file = 0;
+static List   *import_stack = 0;
+
 static String *smodule = 0;
 static int     compat = 0;
 
@@ -50,6 +55,7 @@ static int          is_static = 0;
 /* The following variables are used to manage Perl5 classes */
 
 static  int      blessed = 0;                /* Enable object oriented features */
+static  int      do_constants = 0;           /* Constant wrapping */
 static  Hash     *classes = 0;               /* A hash table for storing the classes we've seen so far */
 static  int      have_constructor = 0;
 static  int      have_destructor= 0;
@@ -66,6 +72,8 @@ static  String   *blessedmembers = 0;        /* Member data associated with each
 static  int      member_func = 0;            /* Set to 1 when wrapping a member function */
 static  String   *realpackage = 0;           /* Name of real module  */
 static  String   *func_stubs = 0;         /* Function stubs */
+static  String   *const_stubs = 0;        /* Constant stubs */
+static  int       num_consts = 0;        /* Number of constants */
 static  String   *var_stubs = 0;          /* Variable stubs */
 static  String   *member_keys = 0;        /* Keys for all member data */
 static  String   *exported = 0;           /* Exported symbols */
@@ -130,6 +138,10 @@ PERL5::parse_args(int argc, char *argv[]) {
 	  } else if (strcmp(argv[i],"-shadow") == 0) {
 	    blessed = 1;
 	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-const") == 0) {
+	    do_constants = 1;
+	    blessed = 1;
+	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-compat") == 0) {
 	    compat = 1;
 	    Swig_mark_arg(i);
@@ -159,14 +171,16 @@ PERL5::parse() {
   pm             = NewString("");
   func_stubs     = NewString("");
   var_stubs      = NewString("");
+  const_stubs    = NewString("");
   exported       = NewString("");
   magic          = NewString("");
   pragma_include = NewString("");
+  import_stack   = NewList();
 
   Swig_banner(f_runtime);
-
+  
   if (NoInclude) {
-    Printf(f_header,"#define SWIG_NOINCLUDE\n");
+    Printf(f_runtime,"#define SWIG_NOINCLUDE\n");
   }
 
   if (Swig_insert_file("common.swg", f_runtime) == -1) {
@@ -182,23 +196,40 @@ PERL5::parse() {
 }
 
 
+
+/* -----------------------------------------------------------------------------
+ * PERL5::import_start(char *modname)
+ * ----------------------------------------------------------------------------- */
+
+void
+PERL5::import_start(char *modname) {
+  if (blessed) {
+    Printf(f_pm,"require %s;\n", modname);
+  }
+  /* Save the old module */
+  if (import_file) {
+    Append(import_stack,import_file);
+  }
+  import_file = NewString(modname);
+}
+
+void 
+PERL5::import_end() {
+  Delete(import_file);
+  if (Len(import_stack)) {
+    import_file = Copy(Getitem(import_stack,Len(import_stack)-1));
+    Delitem(import_stack,Len(import_stack)-1);
+  } else {
+    import_file = 0;
+  }
+}
+
 /* -----------------------------------------------------------------------------
  * PERL5::set_module()
  * ----------------------------------------------------------------------------- */
 void
 PERL5::set_module(char *mod_name) {
-  if (import_file) {
-    if (!(Cmp(import_file,input_file+strlen(input_file)-Len(import_file)))) {
-      if (blessed) {
-	Printf(f_pm,"require %s;\n", mod_name);
-      }
-      Delete(import_file);
-      import_file = 0;
-    }
-  }
-
   if (module) return;
-
   module = NewString(mod_name);
 
   /* Create a C module name and put it in 'cmodule' */
@@ -275,7 +306,11 @@ PERL5::initialize()
   Printf(f_header,"extern \"C\"\n");
   Printf(f_header,"#endif\n");
   Printf(f_header,"#ifndef PERL_OBJECT\n");
-  Printf(f_header,"SWIGEXPORT(void) SWIG_init (CV* cv);\n");
+  Printf(f_header,"#ifndef MULTIPLICITY\n"); 
+  Printf(f_header,"SWIGEXPORT(void) SWIG_init (CV* cv);\n"); 
+  Printf(f_header,"#else\n"); 
+  Printf(f_header,"SWIGEXPORT(void) SWIG_init (pTHXo_ CV* cv);\n"); 
+  Printf(f_header,"#endif\n"); 
   Printf(f_header,"#else\n");
   Printf(f_header,"SWIGEXPORT(void) SWIG_init (CV *cv, CPerlObj *);\n");
   Printf(f_header,"#endif\n");
@@ -327,15 +362,6 @@ PERL5::initialize()
 	 0);
 }
 
-/* -----------------------------------------------------------------------------
- * PERL5::import()
- * ----------------------------------------------------------------------------- */
-void
-PERL5::import(char *filename) {
-  if (import_file) Delete(import_file);
-  import_file = NewString(filename);
-}
-
 /* -----------------------------------------------------------------------------
  * PERL5::close()
  * ----------------------------------------------------------------------------- */
@@ -393,7 +419,7 @@ PERL5::close(void) {
   Printf(f_pm,"var_%s_init();\n", cmodule);
   Printf(f_pm,"%s",pragma_include);
   Printf(f_pm,"package %s;\n", realpackage);
-  Printf(f_pm,"@EXPORT = qw(%s );\n",exported);
+  Printf(f_pm,"@EXPORT = qw( %s);\n",exported);
 
   if (blessed) {
 
@@ -445,6 +471,13 @@ PERL5::close(void) {
 
     Printf(f_pm,"%s",pm);
 
+    if (num_consts > 0) {
+      /* Emit constant stubs */
+      Printf(f_pm,"\n# ------- CONSTANT STUBS -------\n\n");
+      Printf(f_pm,"package %s;\n\n",realpackage);
+      Printf(f_pm,"%s",const_stubs);
+    }
+
     /* Emit variable stubs */
 
     Printf(f_pm,"\n# ------- VARIABLE STUBS --------\n\n");
@@ -807,7 +840,7 @@ PERL5::create_function(char *name, char *iname, SwigType *d, ParmList *l)
       /* If we're returning an object by value, put it's reference
          into our local hash table */
 
-      if ((!SwigType_ispointer(d)) || NewObject) {
+      if ((!SwigType_ispointer(d) && !SwigType_isreference(d)) || NewObject) {
 	Printv(func, tab4, "$", is_shadow(d), "::OWNER{$result} = 1;\n", 0);
       }
 
@@ -1068,9 +1101,10 @@ void PERL5::link_variable(char *name, char *iname, SwigType *t)
     } else {
       Printv(var_stubs, "*", iname, " = *", package, "::", iname, ";\n", 0);
     }
-    if (export_all)
-      Printf(exported,"$%s ", name);
   }
+  if (export_all)
+    Printf(exported,"$%s ", name);
+
   DelWrapper(setf);
   DelWrapper(getf);
 }
@@ -1134,7 +1168,7 @@ static const char *setrv = "#ifndef PERL_OBJECT\
 \n}\n";
 
 void
-PERL5::declare_const(char *name, char *, SwigType *type, char *value)
+PERL5::declare_const(char *name, char *iname, SwigType *type, char *value)
   {
 
   char   *tm;
@@ -1155,7 +1189,7 @@ PERL5::declare_const(char *name, char *, SwigType *type, char *value)
 	Printf(f_header,"%s\n",setiv);
 	have_int_func = 1;
       }
-      Printv(vinit, tab4, "swig_setiv(\"", package, "::", name, "\", (long) ", value, ");\n",0);
+      Printv(vinit, tab4, "swig_setiv(\"", package, "::", iname, "\", (long) ", value, ");\n",0);
       break;
     case T_DOUBLE:
     case T_FLOAT:
@@ -1163,14 +1197,14 @@ PERL5::declare_const(char *name, char *, SwigType *type, char *value)
 	Printf(f_header,"%s\n",setnv);
 	have_double_func = 1;
       }
-      Printv(vinit, tab4, "swig_setnv(\"", package, "::", name, "\", (double) (", value, "));\n",0);
+      Printv(vinit, tab4, "swig_setnv(\"", package, "::", iname, "\", (double) (", value, "));\n",0);
       break;
     case T_CHAR :
       if (!have_char_func) {
 	Printf(f_header,"%s\n",setpv);
 	have_char_func = 1;
       }
-      Printf(vinit,"    swig_setpv(\"%s::%s\",\"%s\");\n", package, name, value);
+      Printf(vinit,"    swig_setpv(\"%s::%s\",\"%s\");\n", package, iname, value);
 
       break;
     case T_STRING:
@@ -1178,7 +1212,7 @@ PERL5::declare_const(char *name, char *, SwigType *type, char *value)
 	Printf(f_header,"%s\n",setpv);
 	have_char_func = 1;
       }
-      Printf(vinit,"    swig_setpv(\"%s::%s\",\"%s\");\n", package, name, value);
+      Printf(vinit,"    swig_setpv(\"%s::%s\",\"%s\");\n", package, iname, value);
       break;
 
     case T_POINTER: case T_ARRAY: case T_REFERENCE:
@@ -1186,7 +1220,7 @@ PERL5::declare_const(char *name, char *, SwigType *type, char *value)
 	Printf(f_header,"%s\n",setrv);
 	have_ref_func = 1;
       }
-      Printv(vinit, tab4, "swig_setrv(\"", package, "::", name, "\", (void *) ", value, ", \"",
+      Printv(vinit, tab4, "swig_setrv(\"", package, "::", iname, "\", (void *) ", value, ", \"",
 	     SwigType_manglestr(type), "\");\n", 0);
       break;
 
@@ -1199,18 +1233,27 @@ PERL5::declare_const(char *name, char *, SwigType *type, char *value)
   if (blessed) {
     if (is_shadow(type)) {
       Printv(var_stubs,
-	     "\nmy %__", name, "_hash;\n",
-	     "tie %__", name, "_hash,\"", is_shadow(type), "\", $",
-	     package, "::", name, ";\n",
-	     "$", name, "= \\%__", name, "_hash;\n",
-	     "bless $", name, ", ", is_shadow(type), ";\n",
+	     "\nmy %__", iname, "_hash;\n",
+	     "tie %__", iname, "_hash,\"", is_shadow(type), "\", $",
+	     package, "::", iname, ";\n",
+	     "$", iname, "= \\%__", iname, "_hash;\n",
+	     "bless $", iname, ", ", is_shadow(type), ";\n",
 	     0);
+    } else if (do_constants) {
+      Printv(const_stubs,"sub ", name, " { $",
+	     package, "::", name, " }\n", 0);
+      num_consts++;
     } else {
-      Printv(var_stubs, "*",name," = *", package, "::", name, ";\n", 0);
+      Printv(var_stubs, "*",iname," = *", package, "::", iname, ";\n", 0);
+    }
+  }
+  if (export_all) {
+    if (do_constants && !is_shadow(type)) {
+      Printf(exported,"%s ",name);
+    } else {
+      Printf(exported,"$%s ",iname);
     }
   }
-  if (export_all)
-    Printf(exported,"$%s ",name);
 }
 
 /* -----------------------------------------------------------------------------
@@ -1350,7 +1393,7 @@ PERL5::cpp_open_class(char *classname, char *rname, char *ctype, int strip) {
     member_keys = NewString("");
 
     /* Add some symbols to the hash tables */
-    cpp_class_decl(Char(classname),Char(fullclassname),Char(ctype));
+    cpp_class_decl(Char(classname),Char(class_name),Char(ctype));
   }
 }
 
@@ -1368,10 +1411,13 @@ PERL5::cpp_close_class() {
 
     /* If we are inheriting from a base class, set that up */
 
+    Printv(pm, "@ISA = qw( ",realpackage, 0);
+    /* Removed to fix SF Bug #449398
     if (Cmp(class_name,realpackage))
       Printv(pm, "@ISA = qw( ",realpackage, 0);
     else
       Printv(pm, "@ISA = qw( ", 0);
+    */
 
     if (base_class) {
       Printv(pm, " ", base_class, 0);
@@ -1418,8 +1464,8 @@ PERL5::cpp_close_class() {
       Printv(pm,
 	     "sub FETCH {\n",
 	     tab4, "my ($self,$field) = @_;\n",
-	     tab4, "my $member_func = \"", package, "::", Swig_name_get(Swig_name_member(class_name,(char*)"${field}")), "\";\n",
-	     tab4, "my $val = &$member_func($self);\n",
+	     tab4, "my $member_func = \"swig_${field}_get\";\n",
+	     tab4, "my $val = $self->$member_func();\n",
 	     tab4, "if (exists $BLESSEDMEMBERS{$field}) {\n",
 	     tab8, "return undef if (!defined($val));\n",
 	     tab8, "my %retval;\n",
@@ -1435,15 +1481,17 @@ PERL5::cpp_close_class() {
       Printv(pm,
 	     "sub STORE {\n",
 	     tab4, "my ($self,$field,$newval) = @_;\n",
-	     tab4, "my $member_func = \"", package, "::", Swig_name_set(Swig_name_member(class_name,(char*)"${field}")), "\";\n",
+	     tab4, "my $member_func = \"swig_${field}_set\";\n",
 	     tab4, "if (exists $BLESSEDMEMBERS{$field}) {\n",
-	     tab8, "&$member_func($self,tied(%{$newval}));\n",
+	     tab8, "$self->$member_func(tied(%{$newval}));\n",
 	     tab4, "} else {\n",
-	     tab8, "&$member_func($self,$newval);\n",
+	     tab8, "$self->$member_func($newval);\n",
 	     tab4, "}\n",
 	     "}\n\n",
 	     0);
 
+
+#ifdef OLD
       /* Output a FIRSTKEY method.   This is to allow iteration over a structure's keys. */
 
       Printv(pm,
@@ -1470,6 +1518,8 @@ PERL5::cpp_close_class() {
 	     tab4, "}\n",
 	     "}\n\n",
 	     0);
+#endif
+
     }
   }
 }
@@ -1492,7 +1542,7 @@ PERL5::cpp_member_func(char *name, char *iname, SwigType *t, ParmList *l) {
   this->Language::cpp_member_func(name,iname,t,l);
   member_func = 0;
 
-  if (blessed && !is_multiple_definition()) {
+  if (blessed) {
     func = NewString("");
     cname = NewString("perl5:");
   
@@ -1560,7 +1610,7 @@ PERL5::cpp_member_func(char *name, char *iname, SwigType *t, ParmList *l) {
       /* If we're returning an object by value, put it's reference
          into our local hash table */
   
-      if (!SwigType_ispointer(t) || NewObject) {
+      if ((!SwigType_ispointer(t) && !SwigType_isreference(t)) || NewObject) {
         Printv(func, tab4, "$", is_shadow(t), "::OWNER{$result} = 1; \n", 0);
       }
   
@@ -1620,13 +1670,16 @@ void PERL5::cpp_variable(char *name, char *iname, SwigType *t) {
   this->Language::cpp_variable(name, iname, t);
   member_func = 0;
 
-  if (blessed && !is_multiple_definition()) {
+  if (blessed) {
     if (iname) realname = iname;
     else realname = name;
 
     /* Store name of key for future reference */
     Printf(member_keys,"'%s', ", realname);
 
+    Printv(pcode,"*swig_", realname, "_get = *", package, "::", Swig_name_get(Swig_name_member(class_name,realname)), ";\n", 0);
+    Printv(pcode,"*swig_", realname, "_set = *", package, "::", Swig_name_set(Swig_name_member(class_name,realname)), ";\n", 0);
+
     /* Now we need to generate a little Perl code for this */
 
     if (is_shadow(t)) {
@@ -1661,7 +1714,7 @@ PERL5::cpp_constructor(char *name, char *iname, ParmList *l) {
   member_func = 1;
   this->Language::cpp_constructor(name, iname, l);
 
-  if (blessed && !is_multiple_definition()) {
+  if (blessed) {
 
     if (iname)
       realname = iname;
@@ -1670,7 +1723,7 @@ PERL5::cpp_constructor(char *name, char *iname, ParmList *l) {
       else realname = class_name;
     }
 
-    if ((Cmp(realname,class_name) == 0) || ((!iname) && (ObjCClass)) ){
+    if ((Cmp(realname,class_name) == 0)) {
 
       /* Emit a blessed constructor  */
 
@@ -1684,7 +1737,7 @@ PERL5::cpp_constructor(char *name, char *iname, ParmList *l) {
 
     }
 
-    Printv(pcode, tab4, "my $self = shift;\n",
+    Printv(pcode, tab4, "my $pkg = shift;\n",
 	   tab4, "my @args = @_;\n", 0);
 
     /* We are going to need to patch up arguments here if necessary
@@ -1706,13 +1759,13 @@ PERL5::cpp_constructor(char *name, char *iname, ParmList *l) {
     }
 
     Printv(pcode,
-	   tab4, "$self = ", package, "::", Swig_name_construct(realname), "(@args);\n",
+	   tab4, "my $self = ", package, "::", Swig_name_construct(realname), "(@args);\n",
 	   tab4, "return undef if (!defined($self));\n",
 	   tab4, "bless $self, \"", fullclassname, "\";\n",
 	   tab4, "$OWNER{$self} = 1;\n",
 	   tab4, "my %retval;\n",
 	   tab4, "tie %retval, \"", fullclassname, "\", $self;\n",
-	   tab4, "return bless \\%retval,\"", fullclassname, "\";\n",
+	   tab4, "return bless \\%retval, $pkg;\n",
 	   "}\n\n",
 	   0);
 
@@ -1731,7 +1784,7 @@ PERL5::cpp_destructor(char *name, char *newname) {
   member_func = 1;
   this->Language::cpp_destructor(name, newname);
 
-  if (blessed && !is_multiple_definition()) {
+  if (blessed) {
     if (newname) realname = newname;
     else {
       if (class_renamed) realname = class_name;
@@ -1764,12 +1817,28 @@ void
 PERL5::cpp_static_func(char *name, char *iname, SwigType *t, ParmList *l) {
   this->Language::cpp_static_func(name,iname,t,l);
 
-  if (blessed && !is_multiple_definition()) {
+  if (blessed) {
     char *realname;
     if (iname) realname = name;
     else realname = iname;
 
-    Printv(pcode, "*", realname, " = *", realpackage, "::", Swig_name_member(class_name,realname), ";\n", 0);
+    Printv(pcode, "*", realname, " = *", package, "::", Swig_name_member(class_name,realname), ";\n", 0);
+  }
+}
+
+
+/* -----------------------------------------------------------------------------
+ * PERL5::cpp_static_var()
+ * ----------------------------------------------------------------------------- */
+void
+PERL5::cpp_static_var(char *name, char *iname, SwigType *t) {
+  this->Language::cpp_static_var(name,iname,t);
+
+  if (blessed) {
+    char *realname;
+    if (iname) realname = name;
+    else realname = iname;
+    Printv(pcode, "*", realname, " = *", package, "::", Swig_name_member(class_name,realname), ";\n", 0);
   }
 }
 
@@ -1824,7 +1893,7 @@ PERL5::cpp_declare_const(char *name, char *iname, SwigType *type, char *value) {
   this->Language::cpp_declare_const(name, iname, type, value);
   blessed = oldblessed;
 
-  if (blessed && !is_multiple_definition()) {
+  if (blessed) {
     if (!iname)
       realname = name;
     else
@@ -1841,15 +1910,25 @@ PERL5::cpp_declare_const(char *name, char *iname, SwigType *type, char *value) {
 void
 PERL5::cpp_class_decl(char *name, char *rename, char *type) {
   String *stype;
+  String *fullname;
+  String *actualpackage;
+  
+  actualpackage = import_file ? import_file : realpackage;
+
   if (blessed) {
     stype = NewString(name);
     SwigType_add_pointer(stype);
-    Setattr(classes,stype,rename);
+    if ((!compat) && (!strchr(rename,':'))) {
+      fullname = NewStringf("%s::%s",actualpackage,rename);
+    } else {
+      fullname = NewString(rename);
+    }
+    Setattr(classes,stype,fullname);
     Delete(stype);
     if (strlen(type) > 0) {
       stype = NewStringf("%s %s",type,name);
       SwigType_add_pointer(stype);
-      Setattr(classes,stype,rename);
+      Setattr(classes,stype,fullname);
       Delete(stype);
     }
   }
@@ -1863,7 +1942,7 @@ PERL5::add_typedef(SwigType *t, char *name) {
 
   if (!blessed) return;
   if (is_shadow(t)) {
-    cpp_class_decl(name,Char(is_shadow(t)),"");
+    cpp_class_decl(name,Char(is_shadow(t)), (char *) "");
   }
 }
 
