@@ -9,11 +9,13 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/python.cxx,v 1.57.2.4 2001/07/03 19:44:10 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/python.cxx,v 1.57.2.4.2.14 2001/09/22 01:54:33 beazley Exp $";
 
 #include "mod11.h"
 #include "python.h"
+#ifndef MACSWIG
 #include "swigconfig.h"
+#endif
 
 static  String       *const_code = 0;
 static  String       *shadow_methods = 0;
@@ -25,13 +27,14 @@ static  int           have_defarg = 0;
 static  int           have_output;
 static  int           use_kw = 0;
 static  int           noopt = 1;
-static  FILE         *f_shadow;
+static  FILE         *f_shadow = 0;
 static  Hash         *hash;
 static  String       *classes;
 static  String       *func;
 static  String       *vars;
 static  String       *pragma_include = 0;
 static  String       *import_file = 0;
+static  List         *import_stack = 0;
 static  String       *methods;
 static  char         *class_name;
 
@@ -134,6 +137,7 @@ PYTHON::parse() {
   vars           = NewString("");
   pragma_include = NewString("");
   methods        = NewString("");
+  import_stack   = NewList();
 
   Swig_banner(f_runtime);
 
@@ -157,30 +161,35 @@ PYTHON::parse() {
  * ----------------------------------------------------------------------------- */
 void
 PYTHON::set_module(char *mod_name) {
-
-  /* If an "import" method has been set and we're in shadow class mode,
-     output a python command to load the module */
-
-  if (import_file) {
-    if (!(strcmp(Char(import_file),input_file+strlen(input_file)-strlen(Char(import_file))))) {
-      if (shadow) {
-	Printf(f_shadow,"\nfrom %s import *\n", mod_name);
-      }
-      Delete(import_file);
-      import_file = 0;
-    }
-  }
   if (module) return;
   module = NewString(mod_name);
 }
 
 /* -----------------------------------------------------------------------------
- * PYTHON::import(char *filename)
+ * PYTHON::import_start(char *modname)
  * ----------------------------------------------------------------------------- */
+
 void
-PYTHON::import(char *filename) {
-  if (import_file) Delete(import_file);
-  import_file = NewString(filename);
+PYTHON::import_start(char *modname) {
+  if (shadow) {
+    Printf(f_shadow,"import %s\n", modname);
+  }
+  /* Save the old module */
+  if (import_file) {
+    Append(import_stack,import_file);
+  }
+  import_file = NewString(modname);
+}
+
+void 
+PYTHON::import_end() {
+  Delete(import_file);
+  if (Len(import_stack)) {
+    import_file = Copy(Getitem(import_stack,Len(import_stack)-1));
+    Delitem(import_stack,Len(import_stack)-1);
+  } else {
+    import_file = 0;
+  }
 }
 
 /* -----------------------------------------------------------------------------
@@ -189,9 +198,9 @@ PYTHON::import(char *filename) {
 void
 PYTHON::add_method(char *name, char *function, int kw) {
   if (!kw)
-    Printf(methods,"\t { \"%s\", %s, METH_VARARGS },\n", name, function);
+    Printf(methods,"\t { (char *)\"%s\", %s, METH_VARARGS },\n", name, function);
   else
-    Printf(methods,"\t { \"%s\", (PyCFunction) %s, METH_VARARGS | METH_KEYWORDS },\n", name, function);
+    Printf(methods,"\t { (char *)\"%s\", (PyCFunction) %s, METH_VARARGS | METH_KEYWORDS },\n", name, function);
 }
 
 /* -----------------------------------------------------------------------------
@@ -210,18 +219,18 @@ PYTHON::initialize(void) {
 
     sprintf(filen,"%s%s.py", output_dir, Char(module));
     // If we don't have an interface then change the module name X to Xc
-    if (! interface)
-    Append(module,"c");
+    if (interface) module = interface;
+    else Append(module,"c");
     if ((f_shadow = fopen(filen,"w")) == 0) {
       Printf(stderr,"Unable to open %s\n", filen);
       SWIG_exit (EXIT_FAILURE);
     }
     Printf(f_shadow,"# This file was created automatically by SWIG.\n");
-    Printf(f_shadow,"import %s\n", interface ? interface : module);
+    Printf(f_shadow,"import %s\n", module);
 
     // Include some information in the code
     Printf(f_header,"\n/*-----------------------------------------------\n              @(target):= %s.so\n\
-  ------------------------------------------------*/\n", interface ? interface : module);
+  ------------------------------------------------*/\n", module);
 
     if (!noopt)
       Printf(f_shadow,"import new\n");
@@ -239,7 +248,7 @@ PYTHON::initialize(void) {
   Printf(f_init,"PyObject *m, *d;\n");
   Printf(f_init,"int i;\n");
   Printf(f_init,"SWIG_globals = SWIG_newvarlink();\n");
-  Printf(f_init,"m = Py_InitModule(\"%s\", %sMethods);\n", module, module);
+  Printf(f_init,"m = Py_InitModule((char*)\"%s\", %sMethods);\n", module, module);
   Printf(f_init,"d = PyModule_GetDict(m);\n");
   Printv(f_init,
 	 "for (i = 0; swig_types_initial[i]; i++) {\n",
@@ -376,9 +385,9 @@ PYTHON::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
   /* Write code to extract function parameters. */
   pcount = emit_args(d, l, f);
   if (!use_kw) {
-    Printf(parse_args,"    if(!PyArg_ParseTuple(args,\"");
+    Printf(parse_args,"    if(!PyArg_ParseTuple(args,(char *)\"");
   } else {
-    Printf(parse_args,"    if(!PyArg_ParseTupleAndKeywords(args,kwargs,\"");
+    Printf(parse_args,"    if(!PyArg_ParseTupleAndKeywords(args,kwargs,(char *)\"");
     Printf(arglist,",kwnames");
   }
 
@@ -462,6 +471,7 @@ PYTHON::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
 
 	case T_VOID :
 	  noarg = 1;
+	  Printf(stdout,"VOID!\n");
 	  break;
 
 	case T_USER:
@@ -499,6 +509,19 @@ PYTHON::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
 	  noarg = 1;
 	  break;
 
+	case T_MPOINTER:
+	  /* Pointer to a member.  Ugh! */
+	  Putc('O',parse_args);
+	  sprintf(source,"arg0%d", i);
+	  sprintf(target,"%s",Char(Getlname(p)));
+	  Wrapper_add_localv(f,source,"PyObject *", source, "=0",0);
+	  Printf(arglist,"&%s",source);
+	  Printv(get_pointers, "if ((SWIG_ConvertPacked(", source, ", (void *) &", target, ", sizeof(", SwigType_str(pt,0),"),",
+		 "SWIGTYPE", SwigType_manglestr(pt), ",1)) == -1) return NULL;\n", 0);
+	  SwigType_remember(pt);
+	  noarg = 1;
+	  break;
+
 	default :
 	  Printf(stderr,"%s : Line %d. Unable to use type %s as a function argument.\n",input_file, line_number, SwigType_str(pt,0));
 	  break;
@@ -578,6 +601,11 @@ PYTHON::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
       SwigType_remember(d);
       Printv(f->code, tab4, "resultobj = SWIG_NewPointerObj((void *) result, SWIGTYPE", SwigType_manglestr(d), ");\n", 0);
       break;
+    case T_MPOINTER:
+      SwigType_remember(d);
+      Printv(f->code, tab4, "resultobj = SWIG_NewPackedObj((void *) &result, sizeof(", SwigType_str(d,0), "), SWIGTYPE", SwigType_manglestr(d), ");\n", 0);
+      break;
+
     case T_VOID:
       Printf(f->code,"    Py_INCREF(Py_None);\n");
       Printf(f->code,"    resultobj = Py_None;\n");
@@ -652,7 +680,7 @@ PYTHON::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
 
 	  if (!have_output) {
 	    Printv(func, tab4, "if val: val = ", is_shadow(d), "Ptr(val)", 0);
-	    if ((!SwigType_ispointer(d)) || NewObject)
+	    if ((!SwigType_ispointer(d) && !SwigType_isreference(d)) || NewObject)
 	      Printf(func, "; val.thisown = 1\n");
 	    else
 	      Printf(func,"\n");
@@ -689,7 +717,7 @@ PYTHON::link_variable(char *name, char *iname, SwigType *t) {
        Python dictionary. */
 
     if (!have_globals) {
-      Printf(f_init,"\t PyDict_SetItemString(d,\"%s\", SWIG_globals);\n",global_name);
+      Printf(f_init,"\t PyDict_SetItemString(d,(char*)\"%s\", SWIG_globals);\n",global_name);
       have_globals=1;
       if ((shadow) && (!(shadow & PYSHADOW_MEMBER))) {
 	Printv(vars, global_name, " = ", module, ".", global_name, "\n", 0);
@@ -809,11 +837,19 @@ PYTHON::link_variable(char *name, char *iname, SwigType *t) {
 	  break;
 
 	case T_POINTER: case T_REFERENCE:
-	  Wrapper_add_localv(setf,"temp", SwigType_lstr(t,0), "temp",0);
+	  Wrapper_add_localv(setf,"temp", SwigType_lstr(t,"temp"),0);
 	  get_pointer((char*)"val",(char*)"temp",t,setf->code,(char*)"1");
 	  Printv(setf->code,tab4, name, " = temp;\n", 0);
 	  break;
 
+	case T_MPOINTER:
+	  Wrapper_add_localv(setf,"temp", SwigType_lstr(t,"temp"), 0);	  
+	  Printv(setf->code, "if ((SWIG_ConvertPacked(val,(void *) &temp, sizeof(", SwigType_str(t,0),"),",
+		 "SWIGTYPE", SwigType_manglestr(t), ",1)) == -1) return 1;\n", 0);
+	  Printv(setf->code,tab4, name, " = temp;\n", 0);
+	  SwigType_remember(t);
+	  break;
+
 	default:
 	  Printf(stderr,"%s : Line %d. Unable to link with type %s.\n", input_file, line_number, SwigType_str(t,0));
 	}
@@ -876,14 +912,40 @@ PYTHON::link_variable(char *name, char *iname, SwigType *t) {
 	       0);
 	break;
 
-      case T_POINTER: case T_ARRAY: case T_REFERENCE:
+      case T_POINTER: case T_REFERENCE:
 	SwigType_remember(t);
 	Printv(getf->code,
 	       tab4, "pyobj = SWIG_NewPointerObj((void *)", name,
 	       ", SWIGTYPE", SwigType_manglestr(t), ");\n",
 	       0);
 	break;
-
+      case T_MPOINTER:
+	SwigType_remember(t);
+	Printv(getf->code,
+	       tab4, "pyobj = SWIG_NewPackedObj((void *) &",name,
+	       ", sizeof(", SwigType_str(t,0), "),",
+	       "SWIGTYPE", SwigType_manglestr(t), ");\n",
+	       0);
+	break;
+      case T_ARRAY:
+	{
+	  SwigType_remember(t);
+	  SwigType *ta = Copy(t);
+	  SwigType *aop = SwigType_pop(ta);
+	  if (SwigType_type(ta) == T_CHAR) {
+	    Printv(getf->code,
+		   tab4, "pyobj = PyString_FromString(", name, ");\n",
+		   0);
+	  } else {
+	    Printv(getf->code,
+		   tab4, "pyobj = SWIG_NewPointerObj((void *)", name,
+		   ", SWIGTYPE", SwigType_manglestr(t), ");\n",
+		   0);
+	  }
+	  Delete(ta);
+	  Delete(aop);
+	}
+	break;
       default:
 	Printf(stderr,"Unable to link with type %s\n", SwigType_str(t,0));
 	break;
@@ -895,7 +957,7 @@ PYTHON::link_variable(char *name, char *iname, SwigType *t) {
 
     /* Now add this to the variable linking mechanism */
 
-    Printf(f_init,"\t SWIG_addvarlink(SWIG_globals,\"%s\",%s_get, %s_set);\n", iname, wname, wname);
+    Printf(f_init,"\t SWIG_addvarlink(SWIG_globals,(char*)\"%s\",%s_get, %s_set);\n", iname, wname, wname);
 
     /* Output a shadow variable.  (If applicable and possible) */
     if ((shadow) && (!(shadow & PYSHADOW_MEMBER))) {
@@ -914,7 +976,7 @@ PYTHON::link_variable(char *name, char *iname, SwigType *t) {
  * PYTHON::declare_const()
  * ----------------------------------------------------------------------------- */
 void
-PYTHON::declare_const(char *name, char *, SwigType *type, char *value) {
+PYTHON::declare_const(char *name, char *iname, SwigType *type, char *value) {
   char   *tm;
 
   if ((tm = Swig_typemap_lookup((char*)"const",type,name,value,name,0))) {
@@ -925,22 +987,33 @@ PYTHON::declare_const(char *name, char *, SwigType *type, char *value) {
     case T_SHORT: case T_USHORT:
     case T_LONG: case T_ULONG:
     case T_SCHAR: case T_UCHAR:
-      Printv(const_code, tab4, "{ SWIG_PY_INT,     \"", name, "\", (long) ", value, ", 0, 0, 0},\n", 0);
+      Printv(const_code, tab4, "{ SWIG_PY_INT,   (char*)\"", iname, "\", (long) ", value, ", 0, 0, 0},\n", 0);
       break;
     case T_DOUBLE:
     case T_FLOAT:
-      Printv(const_code, tab4, "{ SWIG_PY_FLOAT,   \"", name, "\", 0, (double) ", value, ", 0,0},\n", 0);
+      Printv(const_code, tab4, "{ SWIG_PY_FLOAT,   (char*)\"", iname, "\", 0, (double) ", value, ", 0,0},\n", 0);
       break;
     case T_CHAR :
-      Printf(const_code,"    { SWIG_PY_STRING, \"%s\", 0, 0, (void *) \"%s\", 0 }, \n", name, value);
+      Printf(const_code,"    { SWIG_PY_STRING, (char*)\"%s\", 0, 0, (void *) \"%s\", 0 }, \n", iname, value);
       break;
     case T_STRING:
-      Printf(const_code,"    { SWIG_PY_STRING, \"%s\", 0, 0, (void *) \"%s\", 0 }, \n", name, value);
+      Printf(const_code,"    { SWIG_PY_STRING, (char*)\"%s\", 0, 0, (void *) \"%s\", 0 }, \n", iname, value);
       break;
     case T_POINTER: case T_ARRAY: case T_REFERENCE:
       SwigType_remember(type);
-      Printv(const_code, tab4, "{ SWIG_PY_POINTER, \"", name, "\", 0, 0, (void *) ", value, ", &SWIGTYPE", SwigType_manglestr(type), "}, \n", 0);
+      Printv(const_code, tab4, "{ SWIG_PY_POINTER, (char*)\"", iname, "\", 0, 0, (void *) ", value, ", &SWIGTYPE", SwigType_manglestr(type), "}, \n", 0);
       break;
+    case T_MPOINTER: {
+      String *wname = Copy(Swig_name_wrapper(iname));
+      SwigType_remember(type);
+      Printf(f_wrappers, "static %s = %s;\n", SwigType_str(type,wname), value);
+      Printv(f_init, tab4, "PyDict_SetItemString(d, (char*)\"", iname, "\", SWIG_NewPackedObj((void *) &", wname,
+	     ", sizeof(", SwigType_str(type,0), "), SWIGTYPE", SwigType_manglestr(type), "));\n", 0);
+    }
+      break;
+      
+
+      
     default:
       Printf(stderr,"%s : Line %d. Unsupported constant value.\n", input_file, line_number);
       return;
@@ -948,7 +1021,7 @@ PYTHON::declare_const(char *name, char *, SwigType *type, char *value) {
     }
   }
   if ((shadow) && (!(shadow & PYSHADOW_MEMBER))) {
-    Printv(vars,name, " = ", module, ".", name, "\n", 0);
+    Printv(vars,iname, " = ", module, ".", iname, "\n", 0);
   }
 }
 
@@ -1015,27 +1088,53 @@ PYTHON::add_native(char *name, char *funcname, SwigType *, ParmList *) {
 void
 PYTHON::cpp_class_decl(char *name, char *rename, char *type) {
     String *stype;
+    String *importname;
     if (shadow) {
+      if ((import_file) && (!strchr(rename,'.'))) {
+	importname = NewStringf("%s.%s", import_file, rename);
+      } else {
+	importname = NewString(rename);
+      }
       stype = NewString(name);
       SwigType_add_pointer(stype);
-      Setattr(hash,stype,rename);
+      Setattr(hash,stype,importname);
       Delete(stype);
+      /*      Printf(stdout,"cpp_class_decl: %s %s %s\n", name,importname,type); */
       /* Add full name of datatype to the hash table */
       if (strlen(type) > 0) {
 	stype = NewStringf("%s %s", type, name);
 	SwigType_add_pointer(stype);
-	Setattr(hash,stype,rename);
+	Setattr(hash,stype,importname);
 	Delete(stype);
       }
     }
 }
 
+struct PyPragma {
+  String  *m_method;
+  String  *m_text;
+  PyPragma  *next;
+  PyPragma(char *method, char *text) {
+    m_method = NewString(method);
+    m_text = NewString(text);
+    next = 0;
+  }
+  ~PyPragma() {
+    Delete(m_method);
+    Delete(m_text);
+    if (next) delete next;
+  }
+};
+
+static PyPragma *pragmas = 0;
+
 /* -----------------------------------------------------------------------------
  * PYTHON::pragma()
  * ----------------------------------------------------------------------------- */
 void
 PYTHON::pragma(char *lang, char *cmd, char *value) {
-
+  PyPragma *pyp1 = 0;
+  static PyPragma *pyp2 = 0;
     if (strcmp(lang,(char*)"python") == 0) {
 	if (strcmp(cmd,"CODE") == 0) {
 	  if (shadow) {
@@ -1059,78 +1158,42 @@ PYTHON::pragma(char *lang, char *cmd, char *value) {
 	      }
 	    }
 	  }
-	} else {
-	  Printf(stderr,"%s : Line %d. Unrecognized pragma.\n", input_file, line_number);
-	}
-    }
-}
-
-struct PyPragma {
-  String  *m_method;
-  String  *m_text;
-  PyPragma  *next;
-  PyPragma(char *method, char *text) {
-    m_method = NewString(method);
-    m_text = NewString(text);
-    next = 0;
-  }
-  ~PyPragma() {
-    Delete(m_method);
-    Delete(m_text);
-    if (next) delete next;
-  }
-};
-
-static PyPragma *pragmas = 0;
-
-/* -----------------------------------------------------------------------------
- * PYTHON::cpp_pragma() - Handle C++ pragmas
- * ----------------------------------------------------------------------------- */
-void
-PYTHON::cpp_pragma(Pragma *plist) {
-  PyPragma *pyp1 = 0, *pyp2 = 0;
-  if (pragmas) {
-    delete pragmas;
-    pragmas = 0;
-  }
-  while (plist) {
-    if (strcmp(Char(plist->lang),(char*)"python") == 0) {
-      if (strcmp(Char(plist->name),"addtomethod") == 0) {
+	} else if (strcmp(cmd,"addtomethod") == 0) {
 	/* parse value, expected to be in the form "methodName:line" */
-	String *temp = NewString(plist->value);
-	char* txtptr = strchr(Char(temp), ':');
-	if (txtptr) {
-	  /* add name and line to a list in current_class */
-	  *txtptr = 0;
-	  txtptr++;
-	  pyp1 = new PyPragma(Char(temp),txtptr);
-	  pyp1->next = 0;
-	  if (pyp2) {
+	  String *tvalue = NewString(value);
+	  char* txtptr = strchr(Char(tvalue), ':');
+	  if (txtptr) {
+	    /* add name and line to a list in current_class */
+	    *txtptr = 0;
+	    txtptr++;
+	    pyp1 = new PyPragma(Char(tvalue),txtptr);
+	    pyp1->next = 0;
+	    if (pyp2) {
 	      pyp2->next = pyp1;
 	      pyp2 = pyp1;
+	    } else {
+	      pragmas = pyp1;
+	      pyp2 = pragmas;
+	    }
+	  } else {
+	    Printf(stderr,"%s : Line %d. Malformed addtomethod pragma.  Should be \"methodName:text\"\n",
+		   input_file, line_number);
+	  }
+	  Delete(tvalue);
+	} else if (strcmp(cmd, "addtoclass") == 0) {
+	  pyp1 = new PyPragma((char*)"__class__",Char(value));
+	  pyp1->next = 0;
+	  if (pyp2) {
+	    pyp2->next = pyp1;
+	    pyp2 = pyp1;
 	  } else {
 	    pragmas = pyp1;
 	    pyp2 = pragmas;
 	  }
 	} else {
-	  Printf(stderr,"%s : Line %d. Malformed addtomethod pragma.  Should be \"methodName:text\"\n",
-		  Char(plist->filename),plist->lineno);
+	  Printf(stderr,"%s : Line %d. Unrecognized pragma.\n", input_file, line_number);
 	}
-	Delete(temp);
-      } else if (strcmp(Char(plist->name), "addtoclass") == 0) {
-	pyp1 = new PyPragma((char*)"__class__",Char(plist->value));
-	pyp1->next = 0;
-	if (pyp2) {
-	  pyp2->next = pyp1;
-	  pyp2 = pyp1;
-	} else {
-	  pragmas = pyp1;
-	  pyp2 = pragmas;
-	}
-      }
     }
-    plist = plist->next;
-  }
 }
 
 /* -----------------------------------------------------------------------------
@@ -1218,9 +1281,10 @@ PYTHON::cpp_open_class(char *classname, char *rname, char *ctype, int strip) {
 	   0);
 
     Printv(getattr, tab4, "def __getattr__(self,name):\n", 0);
-    Printv(csetattr, tab4, "__setmethods__ = {\n", 0);
-    Printv(cgetattr, tab4, "__getmethods__ = {\n", 0);
+    Printv(csetattr, tab4, "__setmethods__.update({\n", 0);
+    Printv(cgetattr, tab4, "__getmethods__.update({\n", 0);
   }
+  pragmas = 0;
 }
 
 /* -----------------------------------------------------------------------------
@@ -1237,7 +1301,7 @@ PYTHON::cpp_member_func(char *name, char *iname, SwigType *t, ParmList *l) {
   this->Language::cpp_member_func(name,iname,t,l);
   shadow = oldshadow;
 
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     realname = iname ? iname : name;
 
     if (strcmp(realname,"__repr__") == 0)
@@ -1263,7 +1327,7 @@ PYTHON::cpp_member_func(char *name, char *iname, SwigType *t, ParmList *l) {
 	if (!Swig_typemap_search((char*)"out",t,Swig_name_member(class_name,realname))) {
 	  if (!have_output) {
 	    Printv(pyclass, tab8, "if val: val = ", is_shadow(t), "Ptr(val) ", 0);
-	    if ((!SwigType_ispointer(t) || NewObject)) {
+	    if (((!SwigType_ispointer(t) && !SwigType_isreference(t)) || NewObject)) {
 	      Printf(pyclass, "; val.thisown = 1\n");
 	    } else {
 	      Printf(pyclass,"\n");
@@ -1290,7 +1354,7 @@ PYTHON::cpp_constructor(char *name, char *iname, ParmList *l) {
   this->Language::cpp_constructor(name,iname,l);
   shadow = oldshadow;
 
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     realname = iname ? iname : class_name;
 
     if (!have_constructor) {
@@ -1337,7 +1401,7 @@ PYTHON::cpp_destructor(char *name, char *newname) {
   if (shadow) shadow = shadow | PYSHADOW_MEMBER;
   this->Language::cpp_destructor(name,newname);
   shadow = oldshadow;
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     if (newname) realname = newname;
     else realname = class_renamed ? class_name : name;
 
@@ -1357,9 +1421,11 @@ void
 PYTHON::cpp_close_class() {
   String    *ptrclass;
   String    *repr;
+  String    *attrpatch;
 
   ptrclass = NewString("");
   repr =  NewString("");
+  attrpatch = NewString("");
 
   if (shadow) {
     if (!have_constructor) {
@@ -1381,14 +1447,17 @@ PYTHON::cpp_close_class() {
 	   tab8, "raise AttributeError,name\n",
 	   0);
     Printv(setattr, tab8, "self.__dict__[name] = value\n",0);
-    Printv(cgetattr, tab4, "}\n", 0);
-    Printv(csetattr, tab4, "}\n", 0);
+    Printv(cgetattr, tab4, "})\n", 0);
+    Printv(csetattr, tab4, "})\n", 0);
     Printv(ptrclass,cinit,construct,"\n",0);
     Printv(classes,ptrclass,pyclass,0);
-
+    Printv(classes,tab4,"__setmethods__ = {}\n",0);
+    Printf(classes,"%sfor _s in [%s]: __setmethods__.update(_s.__setmethods__)\n",tab4,base_class);
     if (have_setattr) {
       Printv(classes, csetattr, setattr, 0);
     }
+    Printv(classes,tab4,"__getmethods__ = {}\n",0);
+    Printf(classes,"%sfor _s in [%s]: __getmethods__.update(_s.__getmethods__)\n",tab4,base_class);
     if (have_getattr) {
       Printv(classes,cgetattr,getattr,0);
     }
@@ -1475,7 +1544,7 @@ PYTHON::cpp_variable(char *name, char *iname, SwigType *t) {
   this->Language::cpp_variable(name,iname,t);
   shadow = oldshadow;
 
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     have_getattr = 1;
     have_setattr = 1;
     realname = iname ? iname : name;
@@ -1508,7 +1577,7 @@ PYTHON::cpp_declare_const(char *name, char *iname, SwigType *type, char *value)
   this->Language::cpp_declare_const(name,iname,type,value);
   shadow = oldshadow;
 
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     realname = iname ? iname : name;
     Printv(cinit, tab4, realname, " = ", module, ".", Swig_name_member(class_name,realname), "\n", 0);
   }
@@ -1521,6 +1590,6 @@ void
 PYTHON::add_typedef(SwigType *t, char *name) {
   if (!shadow) return;
   if (is_shadow(t)) {
-    cpp_class_decl(name,Char(is_shadow(t)),"");
+    cpp_class_decl(name,Char(is_shadow(t)), (char *) "");
   }
 }
