@@ -1,7 +1,7 @@
 /********************************************************************
  * Ruby module for SWIG
  *
- * $Header: /cvs/projects/SWIG/Source/Modules1.1/ruby.cxx,v 1.30.2.2 2001/06/29 08:56:45 cheetah Exp $
+ * $Header: /cvs/projects/SWIG/Source/Modules1.1/ruby.cxx,v 1.30.2.2.2.6 2001/09/01 01:56:58 beazley Exp $
  *
  * Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
  * Copyright (C) 2000  Information-technology Promotion Agency, Japan
@@ -10,11 +10,13 @@
  *
  ********************************************************************/
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/ruby.cxx,v 1.30.2.2 2001/06/29 08:56:45 cheetah Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/ruby.cxx,v 1.30.2.2.2.6 2001/09/01 01:56:58 beazley Exp $";
 
 #include "mod11.h"
 #include "ruby.h"
+#ifndef MACSWIG
 #include "swigconfig.h"
+#endif
 
 #include <ctype.h>
 #include <string.h>
@@ -77,6 +79,7 @@ class RClass {
     Append(name,valn);
     Clear(vname);
     Printf(vname,"c%s",name);
+    Clear(prefix);
     Printv(prefix,(rn ? rn : cn), "_", 0);
   }
 
@@ -707,7 +710,6 @@ void RUBY::link_variable(char *name, char *iname, SwigType *t) {
 
   /* create getter */
   getfname = NewString(Swig_name_get(name));
-  Replace(getfname,"::", "_", DOH_REPLACE_ANY); /* FIXME: Swig_name_get bug? */
   Printv(getf->def, "static VALUE\n", getfname, "(", 0);
   Printf(getf->def, "VALUE self");
   Printf(getf->def, ") {");
@@ -743,7 +745,6 @@ void RUBY::link_variable(char *name, char *iname, SwigType *t) {
     char *target;
 
     setfname = NewString(Swig_name_set(name));
-    Replace(setfname,"::", "_", DOH_REPLACE_ANY); /* FIXME: Swig_name_get bug? */
     Printv(setf->def, "static VALUE\n", setfname, "(VALUE self, ", 0);
     Printf(setf->def, "VALUE _val) {");
 
@@ -751,7 +752,7 @@ void RUBY::link_variable(char *name, char *iname, SwigType *t) {
       SwigType_add_pointer(t);
       Wrapper_add_localv(setf,"temp",SwigType_lstr(t,0), "temp",0);
       SwigType_del_pointer(t);
-      target = "temp";
+      target = (char *) "temp";
     } else {
       target = name;
     }
@@ -1009,7 +1010,10 @@ int RUBY::to_VALUE(SwigType *type, char *value, String *str, int raw) {
     Printv(str, "rb_float_new(", value, ")", 0);
     break;
   case T_CHAR:
-    Printv(str, "rb_str_new(&", value, ", 1)", 0);
+    if (raw)
+      Printv(str, "rb_str_new(\"", value, "\", 1)", 0);
+    else
+      Printv(str, "rb_str_new(&", value, ", 1)", 0);
     break;
   case T_BOOL:
     Printv(str, "(", value, " ? Qtrue : Qfalse)", 0);
@@ -1162,6 +1166,17 @@ void RUBY::cpp_open_class(char *cname, char *rename, char *ctype, int strip) {
 
   klass = RCLASS(classes, cname);
 
+  /* !!! Added by beazley. 8/29/01 */
+  if (!klass) {
+    klass = new RClass();
+    SET_RCLASS(classes,cname,klass);
+  }
+  String *valid_name = NewString((rename ? rename : cname));
+  validate_const_name(Char(valid_name));
+  klass->set_name(cname,rename,Char(valid_name));
+
+  /* !!! */
+
   if (strip) {
     Clear(klass->type);
     Append(klass->type, klass->cname);
@@ -1278,6 +1293,7 @@ void RUBY::cpp_inherit(char **baseclass, int mode) {
 
 void RUBY::cpp_member_func(char *name, char *iname, SwigType *t, ParmList *l) {
   current = MEMBER_FUNC;
+  /*  Printf(stdout,"%s %s %s\n", name, iname, t); */
   this->Language::cpp_member_func(name, iname, t, l);
   current = NO_CPP;
 }
@@ -1314,7 +1330,6 @@ void RUBY::cpp_destructor(char *name, char *newname) {
   current = DESTRUCTOR;
   this->Language::cpp_destructor(name, newname);
 
-  if (!is_multiple_definition()) {
     String *freefunc = NewString("");
     String *freeproto = NewString("");
     String *freebody = NewString("");
@@ -1348,7 +1363,6 @@ void RUBY::cpp_destructor(char *name, char *newname) {
     Delete(freefunc);
     Delete(freeproto);
     Delete(freebody);
-  }
 }
 
 /* ---------------------------------------------------------------------
@@ -1433,6 +1447,25 @@ void RUBY::cpp_static_var(char *name, char *iname, SwigType *t) {
  * ----------------------------------------------------------------------- */
 
 void RUBY::cpp_class_decl(char *cname, char *rename, char *type) {
+  RClass *kls;
+  
+  kls = RCLASS(classes,cname);
+  if (!kls) {
+    kls = new RClass();
+    String *valid_name = NewString((rename ? rename : cname));
+    validate_const_name(Char(valid_name));
+    kls->set_name(cname, rename, Char(valid_name));
+    SET_RCLASS(classes, cname, kls);
+    Delete(valid_name);
+  } 
+  if (type && strlen(type) > 0) {
+    char temp[256];
+    sprintf(temp,"%s %s", type, cname);
+    SET_RCLASS(classes,temp,kls);
+  }
+
+  /* !!!! Removed by beazley  8/28/01 */
+#ifdef BROKEN
   String *valid_name = NewString((rename ? rename : cname));
   validate_const_name(Char(valid_name));
   klass->set_name(cname, rename, Char(valid_name));
@@ -1449,6 +1482,9 @@ void RUBY::cpp_class_decl(char *cname, char *rename, char *type) {
   */
   klass = new RClass();
   Delete(valid_name);
+#endif
+  /* !!!! */
+
 }
 
 /* --------------------------------------------------------------------
@@ -1505,20 +1541,6 @@ void RUBY::pragma(char *lang, char *cmd, char *value) {
   }
 }
 
-/* -----------------------------------------------------------------------------
- * RUBY::cpp_pragma(Pragma *plist)
- *
- * Handle C++ pragmas
- * ----------------------------------------------------------------------------- */
-
-
-void RUBY::cpp_pragma(Pragma *plist) {
-  while (plist) {
-    pragma(Char(plist->lang), Char(plist->name), Char(plist->value));
-    plist = plist->next;
-  }
-}
-
 /* ---------------------------------------------------------------------
  * RUBY::import(char *filename)
  *
