@@ -9,12 +9,14 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/tcl8.cxx,v 1.53.2.3 2001/06/29 08:56:45 cheetah Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Modules1.1/tcl8.cxx,v 1.53.2.3.2.9 2001/09/20 22:33:52 beazley Exp $";
 
 #include "mod11.h"
 #include "tcl8.h"
 #include <ctype.h>
+#ifndef MACSWIG
 #include "swigconfig.h"
+#endif
 
 static char *usage = (char*)"\
 Tcl 8.0 Options (available with -tcl)\n\
@@ -29,7 +31,7 @@ static String     *cmd_info = 0;
 static String     *var_info = 0;
 static String     *methods = 0;
 static String     *attributes = 0;
-
+static String     *cpp_bases = 0;
 static String     *prefix = 0;
 static String     *module = 0;
 static int        nspace = 0;
@@ -91,7 +93,7 @@ TCL8::parse_args(int argc, char *argv[]) {
     ns_name = Copy(prefix);
 
   }
-  if (prefix)
+  if (prefix && Len(prefix))
     Append(prefix,"_");
 
   Preprocessor_define((void *) "SWIGTCL 1",0);
@@ -231,7 +233,7 @@ TCL8::close(void) {
   Printf(f_wrappers,"%s", var_info);
 
   Printf(f_init,"for (i = 0; swig_commands[i].name; i++) {\n");
-  Printf(f_init,"Tcl_CreateObjCommand(interp, (char *) swig_commands[i].name, swig_commands[i].wrapper, swig_commands[i].clientdata, NULL);\n");
+  Printf(f_init,"Tcl_CreateObjCommand(interp, (char *) swig_commands[i].name, (swig_wrapper_func) swig_commands[i].wrapper, swig_commands[i].clientdata, NULL);\n");
   Printf(f_init,"}\n");
 
   Printf(f_init,"for (i = 0; swig_variables[i].name; i++) {\n");
@@ -256,7 +258,7 @@ void
 TCL8::create_command(char *cname, char *iname) {
 
   String *wname = Swig_name_wrapper(cname);
-  Printv(cmd_info, tab4, "{ SWIG_prefix \"", iname, "\", ", wname, ", NULL},\n", 0);
+  Printv(cmd_info, tab4, "{ SWIG_prefix \"", iname, "\", (swig_wrapper_func) ", wname, ", NULL},\n", 0);
 
   /* Add interpreter name to repeatcmd hash table.  This hash is used in C++ code
      generation to try and find repeated wrapper functions. */
@@ -414,13 +416,13 @@ TCL8::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
       Replace(incode,"$arg",source, DOH_REPLACE_ANY);
     }
     /* Check if there was any cleanup code (save it for later) */
-    if ((tm = Swig_typemap_lookup((char*)"freearg",pt,pn,target,(char*)"tcl_result",0))) {
+    if ((tm = Swig_typemap_lookup((char*)"freearg",pt,pn,target,(char*)"Tcl_GetObjResult(interp)",0))) {
        Printf(cleanup,"%s\n", tm);
       Replace(cleanup,"$argnum",argnum, DOH_REPLACE_ANY);
       Replace(cleanup,"$arg",source,DOH_REPLACE_ANY);
     }
     /* Look for output arguments */
-    if ((tm = Swig_typemap_lookup((char*)"argout",pt,pn,target,(char*)"tcl_result",0))) {
+    if ((tm = Swig_typemap_lookup((char*)"argout",pt,pn,target,(char*)"Tcl_GetObjResult(interp)",0))) {
       Printf(outarg,"%s\n", tm);
       Replace(outarg,"$argnum",argnum, DOH_REPLACE_ANY);
       Replace(outarg,"$arg",source, DOH_REPLACE_ANY);
@@ -440,7 +442,7 @@ TCL8::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
   emit_func_call(name,d,l,f);
 
   /* Return value if necessary  */
-  if ((tm = Swig_typemap_lookup((char*)"out",d,name,(char*)"result",(char*)"tcl_result",0))) {
+  if ((tm = Swig_typemap_lookup((char*)"out",d,name,(char*)"result",(char*)"Tcl_GetObjResult(interp)",0))) {
     Printf(f->code,"%s\n", tm);
   } else {
     switch(SwigType_type(d)) {
@@ -524,7 +526,7 @@ TCL8::create_function(char *name, char *iname, SwigType *d, ParmList *l) {
   Wrapper_print(f,f_wrappers);
 
   /* Register the function with Tcl */
-  Printv(cmd_info, tab4, "{ SWIG_prefix \"", iname, "\", ", Swig_name_wrapper(iname), ", NULL},\n", 0);
+  Printv(cmd_info, tab4, "{ SWIG_prefix \"", iname, "\", (swig_wrapper_func) ", Swig_name_wrapper(iname), ", NULL},\n", 0);
 
   Delete(incode);
   Delete(cleanup);
@@ -648,7 +650,7 @@ TCL8::link_variable(char *name, char *iname, SwigType *t) {
       Printv(set->code, "{\n",
 	     "void *ptr;\n",
 	     "if (SWIG_ConvertPtrFromString(interp,value,&ptr,SWIGTYPE", SwigType_manglestr(t), ") != TCL_OK) {\n",
-	     "return \"Type Error\";\n",
+	     "return (char *) \"Type Error\";\n",
 	     "}\n",
 	     "*(addr) = *((", SwigType_lstr(t,0), ") ptr);\n",
 	     "}\n",
@@ -709,7 +711,7 @@ TCL8::link_variable(char *name, char *iname, SwigType *t) {
       Printv(set->code,  "{\n",
 	     "void *ptr;\n",
 	     "if (SWIG_ConvertPtrFromString(interp,value,&ptr,SWIGTYPE", SwigType_manglestr(t), ") != TCL_OK) {\n",
-	     "return \"Type Error\";\n",
+	     "return (char *) \"Type Error\";\n",
 	     "}\n",
 	     "*(addr) = (", SwigType_lstr(t,0), ") ptr;\n",
 	     "}\n",
@@ -741,21 +743,21 @@ TCL8::link_variable(char *name, char *iname, SwigType *t) {
     DelWrapper(get);
     DelWrapper(set);
   }
-  Printv(var_info, tab4,"{ SWIG_prefix \"", iname, "\", (void *) ", isarray ? "" : "&", name, ",", getname, ",", 0);
+  Printv(var_info, tab4,"{ SWIG_prefix \"", iname, "\", (void *) ", isarray ? "" : "&", name, ", (swig_variable_func) ", getname, ",", 0);
 
   if (readonly) {
     static int readonlywrap = 0;
     if (!readonlywrap) {
       Wrapper *ro = NewWrapper();
-      Printf(ro->def, "static char *swig_readonly(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags) {");
+      Printf(ro->def, "static const char *swig_readonly(ClientData clientData, Tcl_Interp *interp, char *name1, char *name2, int flags) {");
       Printv(ro->code, "return \"Variable is read-only\";\n", "}\n", 0);
       Wrapper_print(ro,f_wrappers);
       readonlywrap = 1;
       DelWrapper(ro);
     }
-    Printf(var_info, "swig_readonly},\n");
+    Printf(var_info, "(swig_variable_func) swig_readonly},\n");
   } else {
-    Printv(var_info, setname, "},\n",0);
+    Printv(var_info, "(swig_variable_func) ", setname, "},\n",0);
   }
 }
 
@@ -764,7 +766,7 @@ TCL8::link_variable(char *name, char *iname, SwigType *t) {
  * ----------------------------------------------------------------------------- */
 
 void
-TCL8::declare_const(char *name, char *, SwigType *type, char *value) {
+TCL8::declare_const(char *name, char *iname, SwigType *type, char *value) {
   int OldStatus = Status;
   SwigType *t;
   char      var_name[256];
@@ -789,7 +791,7 @@ TCL8::declare_const(char *name, char *, SwigType *type, char *value) {
     switch(SwigType_type(type)) {
     case T_BOOL: case T_INT: case T_DOUBLE:
       Printf(f_header,"static %s %s = %s;\n", SwigType_str(type,0), var_name, value);
-      link_variable(var_name,name,type);
+      link_variable(var_name,iname,type);
       break;
 
     case T_SHORT:
@@ -806,7 +808,7 @@ TCL8::declare_const(char *name, char *, SwigType *type, char *value) {
       sprintf(var_name,"%s_char",var_name);
       t = NewString("char");
       SwigType_add_pointer(t);
-      link_variable(var_name,name,t);
+      link_variable(var_name,iname,t);
       Delete(t);
       break;
 
@@ -825,25 +827,25 @@ TCL8::declare_const(char *name, char *, SwigType *type, char *value) {
       sprintf(var_name,"%s_char",var_name);
       t = NewSwigType(T_CHAR);
       SwigType_add_pointer(t);
-      link_variable(var_name,name,t);
+      link_variable(var_name,iname,t);
       Delete(t);
       break;
 
     case T_FLOAT:
       Printf(f_header,"static %s %s = (%s) (%s);\n", SwigType_lstr(type,0), var_name, SwigType_lstr(type,0), value);
-      link_variable(var_name,name,type);
+      link_variable(var_name,iname,type);
       break;
 
     case T_CHAR:
       SwigType_add_pointer(type);
       Printf(f_header,"static %s %s = \"%s\";\n", SwigType_lstr(type,0), var_name, value);
-      link_variable(var_name,name,type);
+      link_variable(var_name,iname,type);
       SwigType_del_pointer(type);
       break;
 
     case T_STRING:
       Printf(f_header,"static %s %s = \"%s\";\n", SwigType_lstr(type,0), var_name, value);
-      link_variable(var_name,name,type);
+      link_variable(var_name,iname,type);
       break;
 
     case T_POINTER: case T_ARRAY: case T_REFERENCE:
@@ -860,7 +862,7 @@ TCL8::declare_const(char *name, char *, SwigType *type, char *value) {
       Printf(f_init,"\t SWIG_MakePtr(%s_char, (void *) %s, SWIGTYPE%s);\n",
 	     var_name, var_name, SwigType_manglestr(type));
       sprintf(var_name,"%s_char",var_name);
-      link_variable(var_name,name,t);
+      link_variable(var_name,iname,t);
       Delete(t);
       break;
 
@@ -925,7 +927,7 @@ TCL8::usage_string(char *iname, SwigType *, ParmList *l) {
 
 void
 TCL8::add_native(char *name, char *funcname, SwigType *, ParmList *) {
-  Printf(f_init,"\t Tcl_CreateObjCommand(interp, SWIG_prefix \"%s\", %s, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);\n",name, funcname);
+  Printf(f_init,"\t Tcl_CreateObjCommand(interp, SWIG_prefix \"%s\", (swig_wrapper_func) %s, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);\n",name, funcname);
 }
 
 /* -----------------------------------------------------------------------------
@@ -959,6 +961,7 @@ TCL8::cpp_open_class(char *classname, char *rename, char *ctype, int strip) {
     Delete(class_name);
     Delete(class_type);
     Delete(real_classname);
+    cpp_bases = NewString("");
 
     class_name = rename ? NewString(rename) : NewString(classname);
     class_type = strip  ? NewString("") : NewStringf("%s ",ctype);
@@ -976,6 +979,9 @@ TCL8::cpp_close_class() {
     t = NewStringf("%s%s", class_type, real_classname);
     SwigType_add_pointer(t);
 
+    // Catch all: eg. a class with only static functions and/or variables will not have 'remembered'
+    SwigType_remember(t);
+
     if (have_destructor) {
       Printv(code, "static void swig_delete_", class_name, "(void *obj) {\n", 0);
       if (CPlusPlus) {
@@ -992,7 +998,10 @@ TCL8::cpp_close_class() {
     Printf(attributes, "    {0,0,0}\n};\n");
     Printv(code,attributes,0);
 
-    Printv(code, "static swig_class _wrap_class_", class_name, " = { \"", class_name,
+    /* Dump bases */
+    Printv(code,"static swig_class *swig_",real_classname,"_bases[] = {", cpp_bases,"0};\n", 0);
+
+    Printv(code, "swig_class _wrap_class_", real_classname, " = { \"", class_name,
 	   "\", &SWIGTYPE", SwigType_manglestr(t), ",",0);
 
     if (have_constructor) {
@@ -1005,10 +1014,10 @@ TCL8::cpp_close_class() {
     } else {
       Printf(code,",0");
     }
-    Printv(code, ", swig_", real_classname, "_methods, swig_", real_classname, "_attributes };\n", 0);
+    Printv(code, ", swig_", real_classname, "_methods, swig_", real_classname, "_attributes, swig_", real_classname,"_bases };\n", 0);
     Printf(f_wrappers,"%s",code);
 
-    Printv(cmd_info, tab4, "{ SWIG_prefix \"", class_name, "\", SwigObjectCmd, &_wrap_class_", class_name, "},\n", 0);
+    Printv(cmd_info, tab4, "{ SWIG_prefix \"", class_name, "\", (swig_wrapper_func) SwigObjectCmd, &_wrap_class_", real_classname, "},\n", 0);
   }
   Delete(code);
 }
@@ -1019,7 +1028,7 @@ void TCL8::cpp_member_func(char *name, char *iname, SwigType *t, ParmList *l) {
   String  *rname;
 
   this->Language::cpp_member_func(name,iname,t,l);
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     realname = iname ? iname : name;
     /* Add stubs for this member to our class handler function */
 
@@ -1038,7 +1047,7 @@ void TCL8::cpp_variable(char *name, char *iname, SwigType *t) {
 
   this->Language::cpp_variable(name, iname, t);
 
-  if (shadow && !is_multiple_definition()) {
+  if (shadow) {
     realname = iname ? iname : name;
     Printv(attributes, tab4, "{ \"-", realname, "\",", 0);
 
@@ -1070,3 +1079,21 @@ TCL8::cpp_destructor(char *name, char *newname) {
   this->Language::cpp_destructor(name,newname);
   have_destructor = 1;
 }
+
+void
+TCL8::cpp_inherit(char **bases, int mode) {
+  this->Language::cpp_inherit(bases,mode);
+  if (shadow) {
+    int i = 0;
+    while (bases[i]) {
+      Printv(f_wrappers,"extern swig_class _wrap_class_",bases[i],";\n",0);
+      Printf(cpp_bases,"&_wrap_class_%s,", bases[i]);
+      i++;
+    }
+  }
+}
+
+void
+TCL8::import_start(char *) {
+  /* Does nothing */
+}
