@@ -10,9 +10,8 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_expr_c[] = "$Header: /cvsroot/swig/SWIG/Source/Preprocessor/expr.c,v 1.11 2004/09/14 20:31:12 wsfulton Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Preprocessor/expr.c,v 1.6 2000/05/11 07:52:57 ttn Exp $";
 
-#include "swig.h"
 #include "preprocessor.h"
 
 static SwigScanner *scan = 0;
@@ -20,7 +19,6 @@ static SwigScanner *scan = 0;
 typedef struct {
   int       op;
   long      value;
-  String   *svalue;
 } exprval;
 
 #define  EXPR_TOP      1
@@ -61,157 +59,104 @@ static void init_precedence() {
   expr_init = 1;
 }
 
-#define UNARY_OP(token) (((token) == SWIG_TOKEN_NOT) || \
-			 ((token) == SWIG_TOKEN_LNOT) || \
-			 ((token) == EXPR_UMINUS))
-
 /* Reduce a single operator on the stack */
-/* return 0 on failure, 1 on success */
-static int reduce_op() {
-  long op_token = stack[sp-1].value;
-  assert(sp > 0);
-  assert(stack[sp-1].op == EXPR_OP);
-  /* do some basic checking first: */
-  if (stack[sp].op != EXPR_VALUE) {
-    errmsg = "Right-hand side is not value";
-    return 0;
+static void reduce_op() {
+  if (stack[sp-1].op != EXPR_OP) {
+    errmsg = "Missing operator";
+    sp = 0;
+    return;
   }
-  if (UNARY_OP(op_token)) {
-    if (stack[sp].svalue) {
-      errmsg = "Syntax error: attempt to apply unary operator to string";
-      return 0;
-    }
-  } else {
-    /* binary operator: */
-    if (sp == 1) {
-      /* top of stack: don't attempt to use sp-2! */
-      errmsg = "Missing left-hand side for binary operator";
-      return 0;
-    }
-    if (stack[sp].op != EXPR_VALUE) {
-      errmsg = "Left-hand side of binary operator is not a value";
-      return 0;
-    }
-    if ((!stack[sp-2].svalue) != (!stack[sp].svalue)) {
-      errmsg = "Can't mix strings and integers in expression";
-      return 0;
-    }
-  }
-  if (stack[sp].svalue) {
-    /* A binary string expression */
-    switch(stack[sp-1].value) {
-    case SWIG_TOKEN_EQUALTO:
-      stack[sp-2].value = (Strcmp(stack[sp-2].svalue,stack[sp].svalue) == 0);
-      Delete(stack[sp-2].svalue);
-      Delete(stack[sp].svalue);
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_NOTEQUAL:
-      stack[sp-2].value = (Strcmp(stack[sp-2].svalue,stack[sp].svalue) != 0);
-      Delete(stack[sp-2].svalue);
-      Delete(stack[sp].svalue);
-      sp -= 2;
-      break;
-    default:
-      errmsg = "Syntax error: bad binary operator for strings";
-      return 0;
-      break;
-    }
-  } else {
-    switch(op_token) {
-    case SWIG_TOKEN_STAR:
-      stack[sp-2].value = stack[sp-2].value * stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_EQUALTO:
-      stack[sp-2].value = stack[sp-2].value == stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_NOTEQUAL:
-      stack[sp-2].value = stack[sp-2].value != stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_PLUS:
-      stack[sp-2].value = stack[sp-2].value + stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_MINUS:
-      stack[sp-2].value = stack[sp-2].value - stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_AND:
-      stack[sp-2].value = stack[sp-2].value & stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_LAND:
-      stack[sp-2].value = stack[sp-2].value && stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_OR:
-      stack[sp-2].value = stack[sp-2].value | stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_LOR:
-      stack[sp-2].value = stack[sp-2].value || stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_XOR:
-      stack[sp-2].value = stack[sp-2].value ^ stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_LESSTHAN:
-      stack[sp-2].value = stack[sp-2].value < stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_GREATERTHAN:
-      stack[sp-2].value = stack[sp-2].value > stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_LTEQUAL:
-      stack[sp-2].value = stack[sp-2].value <= stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_GTEQUAL:
-      stack[sp-2].value = stack[sp-2].value >= stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_NOT:
-      stack[sp-1].value = ~stack[sp].value;
-      sp--;
-      break;
-    case SWIG_TOKEN_LNOT:
-      stack[sp-1].value = !stack[sp].value;
-      sp--;
-      break;
-    case EXPR_UMINUS:
-      stack[sp-1].value = -stack[sp].value;
-      sp--;
-      break;
-    case SWIG_TOKEN_SLASH:
-      stack[sp-2].value = stack[sp-2].value / stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_PERCENT:
-      stack[sp-2].value = stack[sp-2].value % stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_LSHIFT:
-      stack[sp-2].value = stack[sp-2].value << stack[sp].value;
-      sp -= 2;
-      break;
-    case SWIG_TOKEN_RSHIFT:
-      stack[sp-2].value = stack[sp-2].value >> stack[sp].value;
-      sp -= 2;
-      break;
-    default:
-      errmsg = "Syntax error: bad operator";
-      return 0;
-      break;
-    }
+  switch(stack[sp-1].value) {
+  case SWIG_TOKEN_STAR:
+    stack[sp-2].value = stack[sp-2].value * stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_EQUALTO:
+    stack[sp-2].value = stack[sp-2].value == stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_NOTEQUAL:
+    stack[sp-2].value = stack[sp-2].value != stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_PLUS:
+    stack[sp-2].value = stack[sp-2].value + stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_MINUS:
+    stack[sp-2].value = stack[sp-2].value - stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_AND:
+    stack[sp-2].value = stack[sp-2].value & stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_LAND:
+    stack[sp-2].value = stack[sp-2].value && stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_OR:
+    stack[sp-2].value = stack[sp-2].value | stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_LOR:
+    stack[sp-2].value = stack[sp-2].value || stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_XOR:
+    stack[sp-2].value = stack[sp-2].value ^ stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_LESSTHAN:
+    stack[sp-2].value = stack[sp-2].value < stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_GREATERTHAN:
+    stack[sp-2].value = stack[sp-2].value > stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_LTEQUAL:
+    stack[sp-2].value = stack[sp-2].value <= stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_GTEQUAL:
+    stack[sp-2].value = stack[sp-2].value >= stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_NOT:
+    stack[sp-1].value = ~stack[sp].value;
+    sp--;
+    break;
+  case SWIG_TOKEN_LNOT:
+    stack[sp-1].value = !stack[sp].value;
+    sp--;
+    break;
+  case EXPR_UMINUS:
+    stack[sp-1].value = -stack[sp].value;
+    sp--;
+    break;
+  case SWIG_TOKEN_SLASH:
+    stack[sp-2].value = stack[sp-2].value / stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_PERCENT:
+    stack[sp-2].value = stack[sp-2].value % stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_LSHIFT:
+    stack[sp-2].value = stack[sp-2].value << stack[sp].value;
+    sp -= 2;
+    break;
+  case SWIG_TOKEN_RSHIFT:
+    stack[sp-2].value = stack[sp-2].value >> stack[sp].value;
+    sp -= 2;
+    break;
+  default:
+    errmsg = "Syntax error";
+    sp = 0;
+    break;
   }
   stack[sp].op = EXPR_VALUE;
-  stack[sp].svalue = 0;		/* ensure it's not a string! */
-  return 1;
 }
 
 /* -----------------------------------------------------------------------------
@@ -226,19 +171,6 @@ Preprocessor_expr_init (void) {
   if (!scan) scan = NewSwigScanner();
 }
 
-/* -----------------------------------------------------------------------------
- * Tokenizer 
- * ----------------------------------------------------------------------------- */
-
-static int expr_token(SwigScanner *s) {
-  int t;
-  while(1) {
-    t = SwigScanner_token(s);
-    if (!((t == SWIG_TOKEN_BACKSLASH) || (t == SWIG_TOKEN_ENDLINE))) break;
-  }
-  return t;
-}
-
 /* -----------------------------------------------------------------------------
  * Preprocessor_expr()
  *
@@ -255,7 +187,6 @@ Preprocessor_expr(DOH *s, int *error) {
   assert(scan);
 
   Seek(s,0,SEEK_SET);
-  /* Printf(stdout,"evaluating : '%s'\n", s); */
   *error = 0;
   SwigScanner_clear(scan);
   SwigScanner_push(scan,s);
@@ -269,7 +200,7 @@ Preprocessor_expr(DOH *s, int *error) {
     switch(stack[sp].op) {
     case EXPR_TOP:
       /* An expression.   Can be a number or another expression enclosed in parens */
-      token = expr_token(scan);
+      token = SwigScanner_token(scan);
       if (!token) {
 	errmsg = "Expected an expression";
 	*error = 1;
@@ -277,10 +208,7 @@ Preprocessor_expr(DOH *s, int *error) {
       }
       if ((token == SWIG_TOKEN_INT) || (token == SWIG_TOKEN_UINT) || (token == SWIG_TOKEN_LONG) || (token == SWIG_TOKEN_ULONG)) {
 	/* A number.  Reduce EXPR_TOP to an EXPR_VALUE */
-	char *c = Char(SwigScanner_text(scan));
-	stack[sp].value = (long) strtol(c,0,0);
-	stack[sp].svalue = 0;
-	/*	  stack[sp].value = (long) atol(Char(SwigScanner_text(scan))); */
+	stack[sp].value = (long) atol(Char(SwigScanner_text(scan)));
 	stack[sp].op = EXPR_VALUE;
       } else if (token == SWIG_TOKEN_PLUS) { }
        else if ((token == SWIG_TOKEN_MINUS) || (token == SWIG_TOKEN_LNOT) || (token==SWIG_TOKEN_NOT)) {
@@ -288,30 +216,21 @@ Preprocessor_expr(DOH *s, int *error) {
 	stack[sp].value = token;
 	stack[sp++].op = EXPR_OP;
 	stack[sp].op = EXPR_TOP;
-	stack[sp].svalue = 0;
       } else if ((token == SWIG_TOKEN_LPAREN)) {
 	stack[sp++].op = EXPR_GROUP;
 	stack[sp].op = EXPR_TOP;
 	stack[sp].value = 0;
-	stack[sp].svalue = 0;
-      } else if (token == SWIG_TOKEN_ENDLINE) { 
-      } else if ((token == SWIG_TOKEN_STRING)) {
-	stack[sp].svalue = NewString(SwigScanner_text(scan));
-	stack[sp].op = EXPR_VALUE;
-      } else if ((token == SWIG_TOKEN_ID)) {
-	stack[sp].value = 0;
-	stack[sp].svalue = 0;
-	stack[sp].op = EXPR_VALUE;
-      } else goto syntax_error;
+      } else if (token == SWIG_TOKEN_ENDLINE) { }
+      else goto syntax_error;
       break;
     case EXPR_VALUE:
       /* A value is on the stack.   We may reduce or evaluate depending on what the next token is */
-      token = expr_token(scan);
+      token = SwigScanner_token(scan);
       if (!token) {
 	/* End of input. Might have to reduce if an operator is on stack */
 	while (sp > 0) {
 	  if (stack[sp-1].op == EXPR_OP) {
-	    if (!reduce_op()) goto reduce_error;
+	    reduce_op();
 	  } else if (stack[sp-1].op == EXPR_GROUP) {
 	    errmsg = "Missing \')\'";
 	    *error = 1;
@@ -349,7 +268,7 @@ Preprocessor_expr(DOH *s, int *error) {
 	  stack[sp].op = EXPR_TOP;
 	  stack[sp].value = 0;
 	} else {
-	  if (stack[sp-1].op != EXPR_OP) goto syntax_error_expected_operator;
+	  if (stack[sp-1].op != EXPR_OP) goto syntax_error;
 	  op = stack[sp-1].value;         /* Previous operator */
 
 	  /* Now, depending on the precedence relationship between the last operator and the current
@@ -357,7 +276,8 @@ Preprocessor_expr(DOH *s, int *error) {
 
 	  if (prec[op] <= prec[token]) {
 	    /* Reduce the previous operator */
-	    if (!reduce_op()) goto reduce_error;
+	    reduce_op();
+	    if (stack[sp].op != EXPR_VALUE) goto syntax_error;
 	  }
 	  sp++;
 	  stack[sp].op = EXPR_OP;
@@ -371,41 +291,27 @@ Preprocessor_expr(DOH *s, int *error) {
 	if (sp ==  0) goto extra_rparen;
 
 	/* Might have to reduce operators first */
-	while ((sp > 0) && (stack[sp-1].op == EXPR_OP)) {
-	  if (!reduce_op()) goto reduce_error;
-	}
+	while ((sp > 0) && (stack[sp-1].op == EXPR_OP)) reduce_op();
 	if ((sp == 0) || (stack[sp-1].op != EXPR_GROUP)) goto extra_rparen;
 	stack[sp-1].op = EXPR_VALUE;
 	stack[sp-1].value = stack[sp].value;
 	sp--;
 	break;
       default:
-	goto syntax_error_expected_operator;
+	goto syntax_error;
 	break;
       }
       break;
-
     default:
       fprintf(stderr,"Internal error in expression evaluator.\n");
       abort();
     }
   }
-
  syntax_error:
   errmsg = "Syntax error";
   *error = 1;
   return 0;
 
- syntax_error_expected_operator:
-  errmsg = "Syntax error: expected operator";
-  *error = 1;
-  return 0;
-
- reduce_error:
-  /*  errmsg has been set by reduce_op */
-  *error = 1;
-  return 0;
-
  extra_rparen:
   errmsg = "Extra \')\'";
   *error = 1;
