@@ -8,7 +8,7 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvs/projects/SWIG/Source/Preprocessor/preprocessor.h,v 1.5 2000/05/11 07:49:25 ttn Exp $
+ * $Header: /cvs/projects/SWIG/Source/Preprocessor/preprocessor.h,v 1.5.4.2 2001/09/21 03:19:50 beazley Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef _PREPROCESSOR_H
@@ -28,6 +28,8 @@ extern void  Preprocessor_undef(DOHString_or_char *name);
 extern void  Preprocessor_init();
 extern DOH  *Preprocessor_parse(DOH *s);
 extern void  Preprocessor_include_all(int);
+extern void  Preprocessor_import_all(int);
+extern int   Preprocessor_errors(void);
 
 #ifdef __cplusplus
 }
