@@ -17,7 +17,7 @@ extern "C" {
 #include "swig.h"
 }
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/SWIG1.1/Attic/emit.cxx,v 1.29 2000/08/15 18:29:22 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/SWIG1.1/Attic/emit.cxx,v 1.29.4.3 2001/08/30 19:51:35 beazley Exp $";
 
 /* -----------------------------------------------------------------------------
  * emit_args()
@@ -47,25 +47,27 @@ int emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
   while (p != 0) {
     lname  = Getlname(p);
     pt     = Gettype(p);
-    pname  = Getname(p);
-    pvalue = Getvalue(p);
-
-    tm = Swig_typemap_lookup((char*)"arginit",pt,pname,(char*)"",lname,f);
-    if (tm) {
-      Printv(f->code,tm,"\n",0);
+    if (SwigType_type(pt) != T_VOID) {
+      pname  = Getname(p);
+      pvalue = Getvalue(p);
+      
+      tm = Swig_typemap_lookup((char*)"arginit",pt,pname,(char*)"",lname,f);
+      if (tm) {
+	Printv(f->code,tm,"\n",0);
+      }
+      /* Check for ignore or default typemaps */
+      tm = Swig_typemap_lookup((char*)"default",pt,pname,(char*)"",lname,f);
+      if (tm) {
+	Printv(f->code,tm,"\n",0);
+      }
+      tm = Swig_typemap_lookup((char*)"ignore",pt,pname,(char*)"",lname,f);
+      if (tm) {
+	Printv(f->code,tm,"\n",0);
+	Setignore(p,1);
+      }
+      i++;
     }
-    /* Check for ignore or default typemaps */
-    tm = Swig_typemap_lookup((char*)"default",pt,pname,(char*)"",lname,f);
-    if (tm) {
-      Printv(f->code,tm,"\n",0);
-    }
-    tm = Swig_typemap_lookup((char*)"ignore",pt,pname,(char*)"",lname,f);
-    if (tm) {
-      Printv(f->code,tm,"\n",0);
-      Setignore(p,1);
-    }
-    i++;
-    p = Getnext(p);
+    p = nextSibling(p);
   }
   return(i);
 }
@@ -86,7 +88,11 @@ static DOH *fcall = 0;
 
 void emit_set_action(DOHString_or_char *decl) {
   if (fcall) Delete (fcall);
-  fcall = NewString(decl);
+  if (decl) {
+    fcall = NewString(decl);
+  } else {
+    fcall = 0;
+  }
 }
 
 void emit_func_call(char *decl, SwigType *t, ParmList *l, Wrapper *f) {
@@ -194,7 +200,7 @@ int check_numopt(ParmList *p) {
   int  i = 0;
   int  state = 0;
 
-  for (;p; p = Getnext(p),i++) {
+  for (;p; p = nextSibling(p),i++) {
     SwigType *pt = Gettype(p);
     String   *pn = Getname(p);
     if (Getvalue(p)) {
