@@ -11,18 +11,16 @@
  *
  * See the file LICENSE for information on usage and redistribution.	
  *
- * $Header: /cvs/projects/SWIG/Source/SWIG1.1/Attic/internal.h,v 1.7 2000/08/14 22:09:55 beazley Exp $
+ * $Header: /cvs/projects/SWIG/Source/SWIG1.1/Attic/internal.h,v 1.7.4.2 2001/08/13 15:53:06 beazley Exp $
  * ----------------------------------------------------------------------------- */
 
 #include "swig11.h"
 
 extern    Language      *lang;
 extern    int           ForceExtern;
-extern    int           WrapExtern;
+extern    int           ImportMode;
 extern    void         *CCode;
 extern    int           GenerateDefault;
-extern    char         *objc_construct;
-extern    char         *objc_destruct;
 
 // Structure for holding typemap parameters
 
