@@ -12,14 +12,13 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/SWIG1.1/Attic/main.cxx,v 1.49.2.2 2001/07/03 20:06:17 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/SWIG1.1/Attic/main.cxx,v 1.49.2.2.2.14 2001/09/21 03:20:45 beazley Exp $";
 
 #include "internal.h"
+#ifndef MACSWIG
 #include "swigconfig.h"
+#endif
 
-#include <time.h>
-#include <stdlib.h>
-#include <stdio.h>
 #include <ctype.h>
 extern "C" {
 #include "preprocessor.h"
@@ -45,19 +44,18 @@ extern "C" {
     int       Status;
     Language  *lang;                            // Language method
     int        CPlusPlus = 0;
-    int        ObjC = 0;
-    int        ObjCClass = 0;
     int        AddMethods = 0;                  // AddMethods flag
     int        NewObject = 0;                   // NewObject flag
     int        Inline = 0;                      // Inline mode
     int        ForceExtern = 0;                 // Force extern mode
-    int        WrapExtern = 0;
-    int        GenerateDefault = 0;            // Generate default constructors
+    int        ImportMode = 0;
+    int        GenerateDefault = 1;             // Generate default constructors
     char      *Config = 0;
     int        NoInclude = 0;
 
     int        error_count = 0;                 // Error count
     int        Verbose = 0;
+extern String  *ModuleName;
 
 class SwigException {};
 
@@ -69,10 +67,12 @@ static char *usage = (char*)"\
      -Dsymbol        - Define a symbol (for conditional compilation)\n\
      -I<dir>         - Look for SWIG files in <dir>\n\
      -includeall     - Follow all #include statements\n\
+     -importall      - Follow all #include statements as imports\n\
      -l<ifile>       - Include SWIG library file.\n\
-     -make_default   - Create default constructors/destructors\n\
+     -make_default   - Create default constructors/destructors (the default)\n\
+     -no_default     - Do not generate constructors/destructors\n\
+     -module         - Set module name\n\
      -o outfile      - Set name of the output file.\n\
-     -objc           - Enable Objective C processing\n\
      -swiglib        - Report location of SWIG library and exit\n\
      -v              - Run in verbose mode\n\
      -version        - Print SWIG version number\n\
@@ -119,10 +119,6 @@ char  filename[256];
 char  output_dir[512];
 char  fn_runtime[256];
 
-#ifdef MACSWIG
-FILE  *swig_log;
-#endif
-
 char *SwigLib;
 static int     freeze = 0;
 
@@ -148,16 +144,11 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   int     includecount = 0;
   extern  int check_suffix(char *);
   extern  void scanner_file(DOHFile *);
-  extern void parser_init(void);
   DOH    *libfiles = 0;
 
   /* Initialize the SWIG core */
   Swig_init();
-
-#ifdef MACSWIG
-  try {
-#endif
-
+  
   // Initialize the preprocessor
   Preprocessor_init();
 
@@ -190,13 +181,13 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 
   SwigLib = Swig_copy_string(LibDir);        // Make a copy of the real library location
 
-  sprintf(temp,"%s/config", LibDir);
+  sprintf(temp,"%s%sconfig", LibDir, SWIG_FILE_DELIMETER);
   Swig_add_directory((DOH *) temp);
-  Swig_add_directory((DOH *) "./swig_lib/config");
+  Swig_add_directory((DOH *) "." SWIG_FILE_DELIMETER "swig_lib" SWIG_FILE_DELIMETER "config");
   Swig_add_directory((DOH *) LibDir);
-  Swig_add_directory((DOH *) "./swig_lib");
+  Swig_add_directory((DOH *) "." SWIG_FILE_DELIMETER "swig_lib");
   sprintf(InitName,"init_wrap");
-
+  
   libfiles = NewList();
 
   // Get options
@@ -221,10 +212,8 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	      Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-c++") == 0) {
 	      CPlusPlus=1;
+	      Preprocessor_define((DOH *) "__cplusplus 1", 0);
 	      Swig_mark_arg(i);
-          } else if (strcmp(argv[i],"-objc") == 0) {
-	      ObjC = 1;
-              Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-c") == 0) {
 	      NoInclude=1;
 	      Preprocessor_define((DOH *) "SWIG_NOINCLUDE 1", 0);
@@ -232,6 +221,9 @@ int SWIG_main(int argc, char *argv[], Language *l) {
           } else if (strcmp(argv[i],"-make_default") == 0) {
 	    GenerateDefault = 1;
 	    Swig_mark_arg(i);
+          } else if (strcmp(argv[i],"-no_default") == 0) {
+	    GenerateDefault = 0;
+	    Swig_mark_arg(i);
           } else if (strcmp(argv[i],"-swiglib") == 0) {
 	    printf("%s\n", LibDir);
 	    SWIG_exit (EXIT_SUCCESS);
@@ -266,9 +258,20 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	  } else if (strcmp(argv[i],"-includeall") == 0) {
 	    Preprocessor_include_all(1);
 	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-importall") == 0) {
+	    Preprocessor_import_all(1);
+	    Swig_mark_arg(i);
 	  } else if (strcmp(argv[i],"-tm_debug") == 0) {
 	    tm_debug = 1;
 	    Swig_mark_arg(i);
+	  } else if (strcmp(argv[i],"-module") == 0) {
+	    Swig_mark_arg(i);
+	    if (argv[i+1]) {
+	      ModuleName = NewString(argv[i+1]);	    
+	      Swig_mark_arg(i+1);
+	    } else {
+	      Swig_arg_error();
+	    }
 	  } else if (strcmp(argv[i],"-help") == 0) {
 	    fputs(usage,stderr);
 	    Swig_mark_arg(i);
@@ -294,7 +297,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
   // Add language dependent directory to the search path
   {
     DOH *rl = NewString("");
-    Printf(rl,"%s/%s", SwigLib,LibDir);
+    Printf(rl,"%s%s%s", SwigLib, SWIG_FILE_DELIMETER, LibDir);
     Swig_add_directory(rl);
   }
 
@@ -352,7 +355,10 @@ int SWIG_main(int argc, char *argv[], Language *l) {
     }
 
     if (!outfile_name) {
-      sprintf(fn_runtime,"%s_wrap.c",infilename);
+      if (CPlusPlus)
+        sprintf(fn_runtime,"%s_wrap.cxx",infilename);
+      else
+        sprintf(fn_runtime,"%s_wrap.c",infilename);
       strcpy(infile,infilename);
       strcpy(output_dir,"");
     } else {
@@ -360,8 +366,9 @@ int SWIG_main(int argc, char *argv[], Language *l) {
       // Try to identify the output directory
       char *cc = outfile_name;
       char *lastc = outfile_name;
+      const char *delim = SWIG_FILE_DELIMETER;
       while (*cc) {
-	if (*cc == '/') lastc = cc+1;
+	if (*cc == *delim) lastc = cc+1;
 	cc++;
       }
       cc = outfile_name;
@@ -375,7 +382,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
       // Patch up the input filename
       cc = infilename + strlen(infilename);
       while (cc != infilename) {
-	if (*cc == '/') {
+	if (*cc == *delim) {
 	  cc++;
 	  break;
 	}
@@ -401,6 +408,7 @@ int SWIG_main(int argc, char *argv[], Language *l) {
 	SWIG_exit (EXIT_FAILURE);
       }
       fclose(df);
+      Printf(fs,"%%include \"swig.swg\"\n");
       if (lang_config) {
 	Printf(fs,"\n%%include \"%s\"\n", lang_config);
       }
@@ -410,6 +418,9 @@ int SWIG_main(int argc, char *argv[], Language *l) {
       }
       Seek(fs,0,SEEK_SET);
       cpps = Preprocessor_parse(fs);
+      if (Preprocessor_errors()) {
+	SWIG_exit(EXIT_FAILURE);
+      }
       if (cpp_only) {
 	Printf(stdout,"%s", cpps);
 	while (freeze);
@@ -428,25 +439,12 @@ int SWIG_main(int argc, char *argv[], Language *l) {
     f_wrappers = NewString("");
     f_init = NewString("");
 
-    // Set up the typemap for handling new return strings
-    {
-      if (CPlusPlus)
-	Swig_typemap_register((char*)"newfree",(char*)"p.char",(char*)"",(char*)"delete [] $source;\n",0);
-      else
-	Swig_typemap_register((char*)"newfree",(char*)"p.char",(char*)"",(char*)"free($source);\n",0);
-    }
-
-    // If in Objective-C mode.  Load in a configuration file
-
-    if (ObjC) {
-      // Add the 'id' object type as a void *
-      /*      DataType *t = new DataType(T_VOID);
-      t->is_pointer = 1;
-      t->implicit_ptr = 0;
-      t->typedef_add("id");
-      delete t;
-      */
-    }
+    /* Register files with the file handler */
+    Swig_register_filebyname("header",f_header);
+    Swig_register_filebyname("wrapper", f_wrappers);
+    Swig_register_filebyname("init",f_init);
+    Swig_register_filebyname("runtime", NewFileFromFile(f_runtime));
+    Swig_register_filebyname("null", NewString(""));
 
     // Pass control over to the specific language interpreter
 
@@ -454,7 +452,6 @@ int SWIG_main(int argc, char *argv[], Language *l) {
       fprintf (stdout, "Starting language-specific parse...\n");
       fflush (stdout);
     }
-    parser_init();
     lang->parse();
     if (Verbose) {
       fprintf (stdout, "Finished language-specific parse.\n");
@@ -490,25 +487,3 @@ void SWIG_exit(int exit_code) {
   exit (exit_code);
 }
 
-
-// --------------------------------------------------------------------------
-// swig_pragma(char *name, char *value)
-//
-// Handle pragma directives.  Not many supported right now
-// --------------------------------------------------------------------------
-
-void swig_pragma(char *name, char *value) {
-
-  if (strcmp(name,"make_default") == 0) {
-    GenerateDefault = 1;
-  }
-  if (strcmp(name,"no_default") == 0) {
-    GenerateDefault = 0;
-  }
-  if (strcmp(name,"objc_new") == 0) {
-    objc_construct = Swig_copy_string(value);
-  }
-  if (strcmp(name,"objc_delete") == 0) {
-    objc_destruct = Swig_copy_string(value);
-  }
-}
