@@ -13,10 +13,18 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/cwrap.c,v 1.11.2.1 2001/05/25 14:48:57 cheetah Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/cwrap.c,v 1.11.2.1.2.5 2001/09/20 21:16:15 beazley Exp $";
 
 #include "swig.h"
 
+static Parm *nonvoid_parms(Parm *p) {
+  if (p) {
+    SwigType *t = Gettype(p);
+    if (SwigType_type(t) == T_VOID) return 0;
+  }
+  return p;
+}
+
 /* -----------------------------------------------------------------------------
  * Swig_parm_name()
  *
@@ -176,12 +184,14 @@ int Swig_cargs(Wrapper *w, ParmList *p) {
   while (p != 0) {
     lname  = Swig_cparm_name(p,i);
     pt     = Gettype(p);
-    pname  = Getname(p);
-    pvalue = Getvalue(p);
-    local  = Swig_clocal(pt,lname,pvalue);
-    Wrapper_add_localv(w,lname,local,0);
-    i++;
-    p = Getnext(p);
+    if (SwigType_type(pt) != T_VOID) {
+      pname  = Getname(p);
+      pvalue = Getvalue(p);
+      local  = Swig_clocal(pt,lname,pvalue);
+      Wrapper_add_localv(w,lname,local,0);
+      i++;
+    }
+    p = nextSibling(p);
   }
   return(i);
 }
@@ -259,7 +269,14 @@ void Swig_cppresult(Wrapper *w, SwigType *t, String_or_char *name, String_or_cha
   case T_VOID:
     break;
   case T_USER:
-    Printf(fcall, "%s = new %s(", name, SwigType_str(t,0));
+    {
+      SwigType *temp = Copy(t);
+      while (SwigType_isconst(temp)) {
+	SwigType_pop(temp);
+      }
+      Printf(fcall, "%s = new %s(", name, SwigType_str(temp,0));
+      Delete(temp);
+    }
     break;
   case T_REFERENCE:
     Printf(fcall, "%s = ", SwigType_str(t,"_result_ref"));
@@ -319,10 +336,12 @@ Swig_cfunction_call(String_or_char *name, ParmList *parms) {
   while (p) {
     String *pname;
     pt = Gettype(p);
-    pname = Swig_cparm_name(p,i);
-    Printf(func,"%s", Swig_clocal_deref(pt, pname));
-    i++;
-    p = Getnext(p);
+    if (SwigType_type(pt) != T_VOID) {
+      pname = Swig_cparm_name(p,i);
+      Printf(func,"%s", Swig_clocal_deref(pt, pname));
+      i++;
+    }
+    p = nextSibling(p);
     if (p) 
       Printf(func,",");
   }
@@ -350,14 +369,16 @@ Swig_cmethod_call(String_or_char *name, ParmList *parms) {
   if (!p) return Swig_temp_result(func);
   Printf(func,"%s->%s(", Swig_cparm_name(p,0), name);
   i++;
-  p = Getnext(p);
+  p = nextSibling(p);
   while (p) {
     String *pname;
     pt = Gettype(p);
-    pname = Swig_cparm_name(p,i);
-    Printf(func,"%s", Swig_clocal_deref(pt, pname));
-    i++;
-    p = Getnext(p);
+    if (SwigType_type(pt) != T_VOID) {
+      pname = Swig_cparm_name(p,i);
+      Printf(func,"%s", Swig_clocal_deref(pt, pname));
+      i++;
+    }
+    p = nextSibling(p);
     if (p) 
       Printf(func,",");
   }
@@ -404,10 +425,12 @@ Swig_cppconstructor_call(String_or_char *name, ParmList *parms) {
   while (p) {
     String *pname;
     pt = Gettype(p);
-    pname = Swig_cparm_name(p,i);
-    Printf(func,"%s", Swig_clocal_deref(pt, pname));
-    i++;
-    p = Getnext(p);
+    if (SwigType_type(pt) != T_VOID) {
+      pname = Swig_cparm_name(p,i);
+      Printf(func,"%s", Swig_clocal_deref(pt, pname));
+      i++;
+    }
+    p = nextSibling(p);
     if (p) 
       Printf(func,",");
   }
@@ -509,7 +532,7 @@ static void fix_parm_names(ParmList *p) {
       Setname(p,temp);
     }
     i++;
-    p = Getnext(p);
+    p = nextSibling(p);
   }
 }
 
@@ -528,6 +551,7 @@ Swig_cfunction_wrapper(String_or_char *funcname,
 {
   Wrapper *w;
   ParmList *l;
+  String   *tmp;
 
   w = NewWrapper();
 
@@ -536,7 +560,11 @@ Swig_cfunction_wrapper(String_or_char *funcname,
 
   l = CopyParmList(parms);
   fix_parm_names(l);
-  Printf(w->def,"%s %s(%s) {", SwigType_str(rtype,0), funcname, ParmList_str(l));
+  tmp = NewStringf("%s(%s)", funcname, ParmList_str(l));
+  Printf(w->def,"%s {", SwigType_str(rtype,tmp));
+  Delete(tmp);
+
+  /*  Printf(w->def,"%s %s(%s) {", SwigType_str(rtype,0), funcname, ParmList_str(l)); */
   if (code) {
     Printv(w->code, code, "\n", 0);
   }
@@ -566,23 +594,27 @@ Swig_cmethod_wrapper(String_or_char *classname,
   ParmList *l;
   Parm     *p;
   SwigType *t;
-
+  String *tmp;
   w = NewWrapper();
 
   /* Set the name of the function */
   Wrapper_Setname(w,Swig_name_member(classname, methodname));
 
-  l = CopyParmList(parms);
+  l = CopyParmList(nonvoid_parms(parms));
   t = NewString(classname);
   SwigType_add_pointer(t);
   p = NewParm(t,"self");
-  Setnext(p,l);
+  set_nextSibling(p,l);
   Delete(t);
 
   l = p;
   fix_parm_names(l);
 
-  Printf(w->def,"%s %s(%s) {", SwigType_str(rtype,0), Swig_name_member(classname, methodname), ParmList_str(l));
+  tmp = NewStringf("%s(%s)", Swig_name_member(classname,methodname), ParmList_str(l));
+  Printf(w->def,"%s {", SwigType_str(rtype,tmp));
+  Delete(tmp);
+
+  /*  Printf(w->def,"%s %s(%s) {", SwigType_str(rtype,0), Swig_name_member(classname, methodname), ParmList_str(l)); */
 
   if (!code) {
     /* No code supplied.  Write a function manually */
@@ -591,10 +623,10 @@ Swig_cmethod_wrapper(String_or_char *classname,
     }
     
     Printf(w->code,"self->%s(", methodname);
-    p = Getnext(l);
+    p = nextSibling(l);
     while (p) {
       Printf(w->code,"%s", Getname(p));
-      p = Getnext(p);
+      p = nextSibling(p);
       if (p) 
 	Printf(w->code,",");
     }
@@ -631,7 +663,7 @@ Swig_cconstructor_wrapper(String_or_char *classname,
   /* Set the name of the function */
   Wrapper_Setname(w,Swig_name_construct(classname));
 
-  l = CopyParmList(parms);
+  l = CopyParmList(nonvoid_parms(parms));
   t = NewString(classname);
   SwigType_add_pointer(t);
 
@@ -675,7 +707,7 @@ Swig_cppconstructor_wrapper(String_or_char *classname,
   /* Set the name of the function */
   Wrapper_Setname(w,Swig_name_construct(classname));
 
-  l = CopyParmList(parms);
+  l = CopyParmList(nonvoid_parms(parms));
   t = NewString(classname);
   SwigType_add_pointer(t);
 
@@ -692,7 +724,7 @@ Swig_cppconstructor_wrapper(String_or_char *classname,
       Printf(w->code,"(");
       while (p) {
 	Printf(w->code,"%s", Getname(p));
-	p = Getnext(p);
+	p = nextSibling(p);
 	if (p)
 	  Printf(w->code,",");
       }
@@ -833,11 +865,10 @@ Swig_cmemberset_wrapper(String_or_char *classname,
 
   lt = Swig_clocal_type(type);
   ty = Swig_wrapped_var_type(type);
-  p = NewParm(ty,"value");
+  p = NewParm(ty,membername);
 
-  Setnext(l,p);
+  set_nextSibling(l,p);
   
-  /*  Printf(w->def,"%s %s(%s) {", SwigType_str(lt,0), Wrapper_Getname(w), ParmList_str(l)); */
   Printf(w->def,"void %s(%s) {", Wrapper_Getname(w), ParmList_str(l));
 
   if (!code) {
@@ -848,7 +879,6 @@ Swig_cmemberset_wrapper(String_or_char *classname,
     Printv(w->code, code, "\n", 0);
   }
   Printf(w->code,"}\n");
-  /*  Wrapper_Settype(w,lt); */
   Wrapper_Settype(w,"void");
   Wrapper_Setparms(w,l);
   Delete(l);
@@ -876,6 +906,7 @@ Swig_cmemberget_wrapper(String_or_char *classname,
   SwigType *t;
   SwigType *lt;
   SwigType *ty;
+  String   *tmp;
 
   w = NewWrapper();
 
@@ -889,7 +920,9 @@ Swig_cmemberget_wrapper(String_or_char *classname,
   Delete(t);
 
   lt = Swig_clocal_type(type);
-  Printf(w->def,"%s %s(%s) {", SwigType_str(lt,0), Wrapper_Getname(w), ParmList_str(l));
+  tmp = NewStringf("%s(%s)", Wrapper_Getname(w), ParmList_str(l));
+  Printf(w->def,"%s {", SwigType_str(lt,tmp));
+  Delete(tmp);
 
   if (!code) {
     /* No code supplied.  Write a function manually */
@@ -935,7 +968,6 @@ Swig_cvarset_wrapper(String_or_char *varname,
   p = NewParm(ty,"value");
   l = p;
   
-/*  Printf(w->def,"%s %s(%s) {", SwigType_str(lt,0), Wrapper_Getname(w), ParmList_str(l));*/
   Printf(w->def,"void %s(%s) {", Wrapper_Getname(w), ParmList_str(l));
 
   if (!code) {
@@ -972,6 +1004,7 @@ Swig_cvarget_wrapper(String_or_char *varname,
   ParmList *l = 0;
   SwigType *lt;
   SwigType *ty;
+  String *tmp;
 
   w = NewWrapper();
 
@@ -980,7 +1013,9 @@ Swig_cvarget_wrapper(String_or_char *varname,
 
   lt = Swig_clocal_type(type);
 
-  Printf(w->def,"%s %s(%s) {", SwigType_str(lt,0), Wrapper_Getname(w), ParmList_str(l));
+  tmp = NewStringf("%s(%s)", Wrapper_Getname(w), ParmList_str(l));
+  Printf(w->def,"%s {", SwigType_str(lt,tmp));
+  Delete(tmp);
 
   if (!code) {
     /* No code supplied.  Write a function manually */
