@@ -11,7 +11,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_include_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/include.c,v 1.20 2004/09/17 05:55:43 marcelomatus Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/include.c,v 1.10.4.1 2001/07/30 01:01:37 beazley Exp $";
 
 #include "swig.h"
 
@@ -19,19 +19,9 @@ char cvsroot_include_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/include.c,v
 
 static List      *directories = 0;        /* List of include directories */
 static String    *lastpath = 0;           /* Last file that was included */
+static int        bytes_read = 0;         /* Bytes read */
 static String    *swiglib = 0;            /* Location of SWIG library */
 static String    *lang_config = 0;        /* Language configuration file */
-static int       dopush = 1;              /* Whether to push directories */
-
-/* This functions determine whether to push/pop dirs in the preprocessor */
-void Swig_set_push_dir(int push) {
-  dopush = push;
-}
-
-int Swig_get_push_dir(void) {
-  return dopush;
-}
-
 
 /* This function sets the name of the configuration file */
 
@@ -70,48 +60,13 @@ Swig_swiglib_get() {
 
 void 
 Swig_add_directory(const String_or_char *dirname) {
-  String *dir = 0;  
-  if (!directories) directories = NewList();
-  assert(directories);
-  dir = NewString((char *) dirname);
-  assert(dir);
-  Setattr(dir,"sysdir","1");
-  Append(directories, dir);
-}
-
-
-
-/* -----------------------------------------------------------------------------
- * Swig_push_directory()
- *
- * Inserts a directory at the front of the SWIG search path.  This is used by
- * the preprocessor to grab files in the same directory as other included files.
- * ----------------------------------------------------------------------------- */
-
-void 
-Swig_push_directory(const String_or_char *dirname) {
-  if (!Swig_get_push_dir()) return;
   if (!directories) directories = NewList();
   assert(directories);
   if (!DohIsString(dirname)) {
     dirname = NewString((char *) dirname);
     assert(dirname);
   }
-  Insert(directories, 0, dirname);
-}
-
-/* -----------------------------------------------------------------------------
- * Swig_pop_directory()
- *
- * Pops a directory off the front of the SWIG search path.  This is used by
- * the preprocessor.
- * ----------------------------------------------------------------------------- */
-
-void 
-Swig_pop_directory() {
-  if (!Swig_get_push_dir()) return;
-  if (!directories) return;
-  Delitem(directories,0);
+  Append(directories, dirname);
 }
 
 /* -----------------------------------------------------------------------------
@@ -132,14 +87,13 @@ Swig_last_file() {
  * Returns a list of the current search paths.
  * ----------------------------------------------------------------------------- */
 
-static List *
-Swig_search_path_any(int syspath) {
+List *
+Swig_search_path() {
   String *filename;
   String *dirname;
-  List   *slist, *llist;
+  List   *slist;
   int i;
 
-  llist = 0;
   slist = NewList();
   assert(slist);
   filename = NewString("");
@@ -149,50 +103,25 @@ Swig_search_path_any(int syspath) {
 #else
   Printf(filename,".%s", SWIG_FILE_DELIMETER);
 #endif
-  if (syspath) {
-    llist = NewList();
-    assert(slist);
-    Append(llist,filename);
-  } else {
-    Append(slist,filename);
-  }  
-  
+  Append(slist,filename);
   for (i = 0; i < Len(directories); i++) {
     dirname =  Getitem(directories,i);
     filename = NewString("");
     assert(filename);
     Printf(filename, "%s%s", dirname, SWIG_FILE_DELIMETER);
-    if (syspath && !Getattr(dirname,"sysdir")) {
-      Append(llist,filename);
-    } else {
-      Append(slist,filename);
-    }
+    Append(slist,filename);
   }
-  if (syspath) {
-    for (i = 0; i < Len(llist); i++) {
-      Append(slist,Getitem(llist,i));
-    }
-    Delete(llist);
-  }
-
   return slist;
 }  
 
-List *
-Swig_search_path() {
-  return Swig_search_path_any(0);
-}
-
-
-
 /* -----------------------------------------------------------------------------
  * Swig_open()
  *
  * Looks for a file and open it.  Returns an open  FILE * on success.
  * ----------------------------------------------------------------------------- */
 
-static FILE *
-Swig_open_any(const String_or_char *name, int sysfile) {
+FILE *
+Swig_open(const String_or_char *name) {
   FILE        *f;
   String   *filename;
   List     *spath = 0;
@@ -207,7 +136,7 @@ Swig_open_any(const String_or_char *name, int sysfile) {
   assert(filename);
   f = fopen(Char(filename),"r");
   if (!f) {
-      spath = Swig_search_path_any(sysfile);
+      spath = Swig_search_path();
       for (i = 0; i < Len(spath); i++) {
 	  Clear(filename);
 	  Printf(filename,"%s%s", Getitem(spath,i), cname);
@@ -217,10 +146,6 @@ Swig_open_any(const String_or_char *name, int sysfile) {
       Delete(spath);
   }
   if (f) {
-#if defined(_WIN32) /* Note not on Cygwin else filename is displayed with double '/' */
-    Replaceall(filename,"\\\\","\\"); /* remove double '\' in case any already present */
-    Replaceall(filename,"\\","\\\\");
-#endif
     Delete(lastpath);
     lastpath = Copy(filename);
   }
@@ -228,13 +153,6 @@ Swig_open_any(const String_or_char *name, int sysfile) {
   return f;
 }
 
-FILE *
-Swig_open(const String_or_char *name) {
-  return Swig_open_any(name, 0);
-}
-
-
-
 /* -----------------------------------------------------------------------------
  * Swig_read_file()
  * 
@@ -260,32 +178,29 @@ Swig_read_file(FILE *f) {
  * Opens a file and returns it as a string.
  * ----------------------------------------------------------------------------- */
 
-static String *
-Swig_include_any(const String_or_char *name, int sysfile) {
+static int readbytes = 0;
+String *
+Swig_include(const String_or_char *name) {
   FILE         *f;
   String    *str;
 
-  f = Swig_open_any(name, sysfile);
+  f = Swig_open(name);
   if (!f) return 0;
   str = Swig_read_file(f);
+  bytes_read = bytes_read + Len(str);
   fclose(f);
   Seek(str,0,SEEK_SET);
   Setfile(str,lastpath);
   Setline(str,1);
+  readbytes += Len(str);
   return str;
 }
 
-String *
-Swig_include(const String_or_char *name) {
-  return Swig_include_any(name, 0);
+int
+Swig_bytes_read() {
+  return readbytes;
 }
 
-String *
-Swig_include_sys(const String_or_char *name) {
-  return Swig_include_any(name, 1);
-}
-
-
 /* -----------------------------------------------------------------------------
  * Swig_insert_file()
  *
@@ -334,82 +249,4 @@ Swig_filebyname(const String_or_char *filename) {
   return Getattr(named_files,filename);
 }
 
-/* -----------------------------------------------------------------------------
- * Swig_file_suffix()
- *
- * Returns the suffix of a file
- * ----------------------------------------------------------------------------- */
 
-char *
-Swig_file_suffix(const String_or_char *filename) {
-  char *d;
-  char *c = Char(filename);
-  if (strlen(c)) {
-    d = c + Len(filename) - 1;
-    while (d != c) {
-      if (*d == '.') return d;
-      d--;
-    }
-    return c+Len(filename);  
-  }
-  return c;
-}
-
-/* -----------------------------------------------------------------------------
- * Swig_file_basename()
- *
- * Returns the filename with no suffix attached.
- * ----------------------------------------------------------------------------- */
-
-char *
-Swig_file_basename(const String_or_char *filename)
-{
-  static char tmp[1024];
-  char *c;
-  strcpy(tmp,Char(filename));
-  c = Swig_file_suffix(tmp);
-  *c = 0;
-  return tmp;
-}
-
-/* -----------------------------------------------------------------------------
- * Swig_file_filename()
- *
- * Return the file with any leading path stripped off
- * ----------------------------------------------------------------------------- */
-char *
-Swig_file_filename(const String_or_char *filename)
-{
-  static char tmp[1024];
-  const char *delim = SWIG_FILE_DELIMETER;
-  char *c;
-
-  strcpy(tmp,Char(filename));
-  if ((c=strrchr(tmp,*delim))) return c+1;
-  else return tmp;
-}
-
-/* -----------------------------------------------------------------------------
- * Swig_file_dirname()
- *
- * Return the name of the directory associated with a file
- * ----------------------------------------------------------------------------- */
-char *
-Swig_file_dirname(const String_or_char *filename)
-{
-  static char tmp[1024];
-  const char *delim = SWIG_FILE_DELIMETER;
-  char *c;
-  strcpy(tmp,Char(filename));
-  if (!strstr(tmp,delim)) {
-    return "";
-  }
-  c = tmp + strlen(tmp) -1;
-  while (*c != *delim) c--;
-  *(++c) = 0;
-  return tmp;
-}
-      
-    
-    
-  
