@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_scanner_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/scanner.c,v 1.13 2003/10/31 17:48:02 beazley Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/scanner.c,v 1.9.4.1 2001/07/30 08:32:50 mkoeppe Exp $";
 
 #include "swig.h"
 #include <ctype.h>
@@ -223,7 +223,7 @@ retract(SwigScanner *s, int n) {
 static int
 look(SwigScanner *s) {
     int      state;
-    int      c = 0;
+    char     c = 0;
 
     state = 0;
     Clear(s->text);
@@ -598,46 +598,22 @@ look(SwigScanner *s) {
 	    if ((c = nextchar(s)) == 0) return SWIG_TOKEN_LONG;
 	    if ((c == 'u') || (c == 'U')) {
 		return SWIG_TOKEN_ULONG;
-	    } else if ((c == 'l') || (c == 'L')) {
-	      state = 870;
 	    } else {
 		retract(s,1);
 		return SWIG_TOKEN_LONG;
 	    } 
-	    break;
-
-	    /* A long long integer */
-
-	case 870:
-	  if ((c = nextchar(s)) == 0) return SWIG_TOKEN_LONGLONG;
-	  if ((c == 'u') || (c == 'U')) {
-	    return SWIG_TOKEN_ULONGLONG;
-	  } else {
-	    retract(s,1);
-	    return SWIG_TOKEN_LONGLONG;
-	  }
 
 	    /* An unsigned number */
 	case 88:
 
 	    if ((c = nextchar(s)) == 0) return SWIG_TOKEN_UINT;
 	    if ((c == 'l') || (c == 'L')) {
-	      state = 880;
+		return SWIG_TOKEN_ULONG;
 	    } else {
-	      retract(s,1);
-	      return SWIG_TOKEN_UINT;
+		retract(s,1);
+		return SWIG_TOKEN_UINT;
 	    } 
-	    break;
       
-	    /* Possibly an unsigned long long or unsigned long */
-	case 880:
-	  if ((c = nextchar(s)) == 0) return SWIG_TOKEN_ULONG;
-	  if ((c == 'l') || (c == 'L')) return SWIG_TOKEN_ULONGLONG;
-	  else {
-	    retract(s,1);
-	    return SWIG_TOKEN_ULONG;
-	  }
-
 	    /* A character constant */
 	case 9:
 	    if ((c = nextchar(s)) == 0) {
