@@ -9,7 +9,7 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.	
  *
- * $Header: /cvs/projects/SWIG/Source/Swig/swig.h,v 1.51.2.3 2001/06/14 11:00:21 mkoeppe Exp $
+ * $Header: /cvs/projects/SWIG/Source/Swig/swig.h,v 1.51.2.3.2.14 2001/09/19 13:26:49 beazley Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef _SWIGCORE_H
@@ -29,31 +29,46 @@ typedef  DOH     Hash;
 typedef  DOH     List;
 typedef  DOH     String_or_char;
 typedef  DOH     File;
+typedef  DOH     Parm;
+typedef  DOH     ParmList;
+typedef  DOH     Node;
+typedef  DOH     Symtab;
 
 /* --- Legacy DataType interface.  These type codes are provided solely 
-   for backwards compatibility with older modules --- */
+       for backwards compatibility with older modules --- */
 
-#define   T_INT        1
-#define   T_SHORT      2
-#define   T_LONG       3
-#define   T_UINT       4
+/* --- The ordering of type values is used to determine type-promotion 
+   in the parser.  Do not change */
+
+/* Numeric types */
+
+#define   T_BOOL       1
+#define   T_SCHAR      2
+#define   T_UCHAR      3
+#define   T_SHORT      4
 #define   T_USHORT     5
-#define   T_ULONG      6
-#define   T_UCHAR      7
-#define   T_SCHAR      8
-#define   T_BOOL       9
-#define   T_DOUBLE     10
-#define   T_FLOAT      11
-#define   T_CHAR       12
-#define   T_USER       13
-#define   T_VOID       14
-#define   T_LONGLONG   15
-#define   T_ULONGLONG  16
-#define   T_STRING     20
-#define   T_POINTER    21
-#define   T_REFERENCE  22
-#define   T_ARRAY      23
-#define   T_FUNCTION   24
+#define   T_ENUM       6
+#define   T_INT        7
+#define   T_UINT       8
+#define   T_LONG       9
+#define   T_ULONG      10
+#define   T_LONGLONG   11
+#define   T_ULONGLONG  12
+#define   T_FLOAT      20
+#define   T_DOUBLE     21
+#define   T_NUMERIC    22
+
+/* non-numeric */
+
+#define   T_CHAR       30
+#define   T_USER       31
+#define   T_VOID       32
+#define   T_STRING     33
+#define   T_POINTER    34
+#define   T_REFERENCE  35
+#define   T_ARRAY      36
+#define   T_FUNCTION   37
+#define   T_MPOINTER   38
 #define   T_SYMBOL     98
 #define   T_ERROR      99
 
@@ -67,13 +82,26 @@ extern String  *Swig_read_file(FILE *f);
 extern String  *Swig_include(const String_or_char *name);
 extern int      Swig_insert_file(const String_or_char *name, File *outfile);
 extern int      Swig_bytes_read();
+extern void     Swig_set_config_file(const String_or_char *filename);
+extern String  *Swig_get_config_file(void);
+extern void     Swig_swiglib_set(const String_or_char *);
+extern String  *Swig_swiglib_get();
+extern void     Swig_register_filebyname(const String_or_char *filename, File *outfile);
+extern File    *Swig_filebyname(const String_or_char *filename);
 
-#define  SWIG_FILE_DELIMETER   "/"
+#define OUTFILE(x)   Swig_filebyname(x)
+
+#ifdef MACSWIG
+#  define SWIG_FILE_DELIMETER  ":"
+#else
+#  define  SWIG_FILE_DELIMETER "/"
+#endif
 
 /* --- Command line parsing --- */
 
 extern void     Swig_init_args(int argc, char **argv);
 extern void     Swig_mark_arg(int n);
+extern int      Swig_check_marked(int n);
 extern void     Swig_check_options();
 extern void     Swig_arg_error();
 
@@ -143,6 +171,7 @@ extern void         SwigScanner_idstart(SwigScanner *, char *idchar);
 #define   SWIG_TOKEN_AT           45
 #define   SWIG_TOKEN_DOLLAR       46
 #define   SWIG_TOKEN_CODEBLOCK    47
+#define   SWIG_TOKEN_RSTRING      48
 #define   SWIG_TOKEN_ILLEGAL      98
 #define   SWIG_TOKEN_LAST         99 
 
@@ -151,11 +180,12 @@ typedef DOH SwigType;
 
 extern SwigType   *NewSwigType(int typecode);
 extern void        SwigType_add_pointer(SwigType *t);
+extern void        SwigType_add_memberpointer(SwigType *t, String_or_char *qual);
 extern void        SwigType_del_pointer(SwigType *t);
 extern void        SwigType_add_array(SwigType *t, String_or_char *size);
 extern void        SwigType_add_reference(SwigType *t);
 extern void        SwigType_add_qualifier(SwigType *t, String_or_char *qual);
-extern void        SwigType_add_function(SwigType *t, List *parms);
+extern void        SwigType_add_function(SwigType *t, ParmList *parms);
 extern List       *SwigType_split(SwigType *t);
 extern String     *SwigType_pop(SwigType *t);
 extern void        SwigType_push(SwigType *t, SwigType *s);
@@ -168,12 +198,15 @@ extern String     *SwigType_lcaststr(SwigType *s, const String_or_char *id);
 extern String     *SwigType_manglestr(SwigType *t);
 extern SwigType   *SwigType_ltype(SwigType *t);
 extern int         SwigType_ispointer(SwigType *t);
+extern int         SwigType_ismemberpointer(SwigType *t);
 extern int         SwigType_isreference(SwigType *t);
 extern int         SwigType_isarray(SwigType *t);
 extern int         SwigType_isfunction(SwigType *t);
 extern int         SwigType_isqualifier(SwigType *t);
 extern int         SwigType_isconst(SwigType *t);
 
+extern SwigType   *SwigType_strip_qualifiers(SwigType *t);
+
 extern String     *SwigType_base(SwigType *t);
 extern String     *SwigType_prefix(SwigType *t);
 extern void        SwigType_setbase(SwigType *t, String_or_char *name);
@@ -183,53 +216,89 @@ extern void        SwigType_inherit(String *subclass, String *baseclass);
 extern void        SwigType_new_scope();
 extern void        SwigType_reset_scopes();
 extern void        SwigType_set_scope_name(String_or_char *name);
-extern void        SwigType_merge_scope(Hash *scope, String_or_char *prefix);
+extern void        SwigType_merge_scope(Hash *scope);
 extern Hash       *SwigType_pop_scope();
 extern SwigType   *SwigType_typedef_resolve(SwigType *t);
 extern SwigType   *SwigType_typedef_resolve_all(SwigType *t);
+extern SwigType   *SwigType_typedef_qualified(SwigType *t);
 extern int         SwigType_istypedef(SwigType *t);
 extern int         SwigType_cmp(String_or_char *pat, SwigType *t);
 extern int         SwigType_array_ndim(SwigType *t);
 extern String     *SwigType_array_getdim(SwigType *t, int n);
 extern void        SwigType_array_setdim(SwigType *t, int n, String_or_char *rep);
+extern SwigType   *SwigType_array_type(SwigType *t);
 extern String     *SwigType_default(SwigType *t);
 extern int         SwigType_type(SwigType *t);
 extern void        SwigType_remember(SwigType *t);
 extern void        SwigType_emit_type_table(File *f_headers, File *f_table);
 
+/* --- Symbol table module --- */
+
+extern void    Swig_symbol_init();
+extern void    Swig_symbol_setscopename(String_or_char *name);
+extern String *Swig_symbol_getscopename();
+extern String *Swig_symbol_qualifiedscopename(Symtab *symtab);
+extern Symtab *Swig_symbol_newscope();
+extern Symtab *Swig_symbol_setscope(Symtab *);
+extern Symtab *Swig_symbol_current();
+extern Symtab *Swig_symbol_popscope();
+extern Node   *Swig_symbol_add(String_or_char *symname, Node *node);
+extern Node   *Swig_symbol_add_tag(String_or_char *symname, Node *node);
+extern Node   *Swig_symbol_lookup(String_or_char *symname);
+extern Node   *Swig_symbol_lookup_local(String_or_char *symname);
+extern Node   *Swig_symbol_lookup_tag(String_or_char *symname);
+extern String *Swig_symbol_qualified(Node *node);
+extern Node   *Swig_symbol_isoverloaded(Node *node);
+extern void    Swig_symbol_remove(Node *node);
+
 /* --- Parameters and Parameter Lists --- */
 
 /* Parameters are really just hidden behind a DOH object.  The following
    interface will probably be simplified even further. */
 
-typedef DOH Parm;
-
 extern Parm       *NewParm(SwigType *type, String_or_char *n);
 extern Parm       *CopyParm(Parm *p);
-
-typedef DOH   ParmList;
-
 extern ParmList   *CopyParmList(ParmList *);
 extern int         ParmList_len(ParmList *);
 extern int         ParmList_numarg(ParmList *);
+extern int         ParmList_numrequired(ParmList *);
 extern String     *ParmList_str(ParmList *);
 extern String     *ParmList_protostr(ParmList *);
 
 /* --- Parse tree support --- */
 
-typedef struct {
-   char *name;
-   int  (*action)(DOH *obj, void *clientdata);
-} SwigRule;
+/* DOM-like node access */
 
+#define  nodeType(x)               Getattr(x,"nodeType")
+#define  parentNode(x)             Getattr(x,"parentNode")
+#define  previousSibling(x)        Getattr(x,"previousSibling")
+#define  nextSibling(x)            Getattr(x,"nextSibling")
+#define  firstChild(x)             Getattr(x,"firstChild")
+#define  lastChild(x)              Getattr(x,"lastChild")
+
+/* Special accessors for symbol tables */
+
+#define  nextSymbol(x)             Getattr(x,"$symnext")
+#define  prevSymbol(x)             Getattr(x,"$symprev")
+#define  nodeSymtab(x)             Getattr(x,"$symtab")
+#define  nodeSymbol(x)             Getattr(x,"$symname")
+
+/* Macros to set up the DOM tree (mostly used by the parser) */
+
+#define  set_nodeType(x,v)         Setattr(x,"nodeType",v)
+#define  set_parentNode(x,v)       Setattr(x,"parentNode",v)
+#define  set_previousSibling(x,v)  Setattr(x,"previousSibling",v)
+#define  set_nextSibling(x,v)      Setattr(x,"nextSibling",v)
+#define  set_firstChild(x,v)       Setattr(x,"firstChild",v)
+#define  set_lastChild(x,v)        Setattr(x,"lastChild",v)
+
+extern void appendChild(Node *node, Node *child);
+
+/* Debugging of parse trees */
+extern void Swig_debug_emit(int);
 extern void Swig_dump_tags(DOH *obj, DOH *root);
-extern void Swig_add_rule(String_or_char *, int (*action)(DOH *, void *));
-extern void Swig_add_rules(SwigRule ruleset[]);
-extern void Swig_clear_rules();
-extern int  Swig_emit(DOH *obj, void *clientdata);
-extern void Swig_cut_node(DOH *obj);
-extern DOH *Swig_next(DOH *obj);
-extern DOH *Swig_prev(DOH *obj);
+extern void Swig_dump_tree(DOH *obj);
+extern void Swig_dump_rules();
 
 /* -- Wrapper function Object */
 
@@ -278,10 +347,14 @@ extern DOH       *Swig_map_match(Hash *ruleset, Hash *parms, int *nmatch);
 /* --- Misc --- */
 extern char      *Swig_copy_string(const char *c);
 extern void       Swig_banner(File *f);
+extern void       Swig_section(File *f, const String_or_char *s);
 extern DOH       *Swig_temp_result(DOH *x);
 extern String    *Swig_string_escape(String *s);
 extern String    *Swig_string_mangle(String *s);
 extern void       Swig_init();
+extern void       Swig_warn(const char *filename, int line, const char *msg);
+
+#define WARNING(msg) Swig_warn(__FILE__,__LINE__,msg)
 
 /* --- C Wrappers --- */
 extern String    *Swig_clocal(SwigType *t, String_or_char *name, String_or_char *value);
@@ -349,15 +422,15 @@ extern Wrapper   *Swig_cvarget_wrapper(String_or_char *varname,
 /* --- Legacy Typemap API (somewhat simplified) --- */
 
 extern void   Swig_typemap_init();
-extern void   Swig_typemap_register(char *op, SwigType *type, String_or_char *name, String_or_char *code, ParmList *locals);
-extern void   Swig_typemap_copy(char *op, SwigType *stype, String_or_char *sname,
+extern void   Swig_typemap_register(const String_or_char *op, SwigType *type, String_or_char *name, String_or_char *code, ParmList *locals);
+extern void   Swig_typemap_copy(const String_or_char *op, SwigType *stype, String_or_char *sname,
 				SwigType *ttype, String_or_char *tname);
-extern void   Swig_typemap_clear(char *op, SwigType *type, String_or_char *name);
+extern void   Swig_typemap_clear(const String_or_char *op, SwigType *type, String_or_char *name);
 extern void   Swig_typemap_apply(SwigType *tm_type, String_or_char *tmname, SwigType *type, String_or_char *pname);
 extern void   Swig_typemap_clear_apply(SwigType *type, String_or_char *pname);
 extern void   Swig_typemap_debug();
-extern Hash  *Swig_typemap_search(char *op, SwigType *type, String_or_char *pname);
-extern char  *Swig_typemap_lookup(char *op, SwigType *type, String_or_char *pname, String_or_char *source, String_or_char *target, Wrapper *f);
+extern Hash  *Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *pname);
+extern char  *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_char *pname, String_or_char *source, String_or_char *target, Wrapper *f);
 extern void   Swig_typemap_new_scope();
 extern Hash  *Swig_typemap_pop_scope();
 
@@ -373,15 +446,23 @@ extern void   Swig_except_clear();
 #define Getvalue(x)        Getattr(x,"value")
 #define Getlname(x)        Getattr(x,"lname")
 #define Getignore(x)       GetInt(x,"ignore")
+#define Getparms(x)        Getattr(x,"parms")
+#define Getdecl(x)         Getattr(x,"decl")
+#define Getstorage(x)      Getattr(x,"storage")
+#define Getsymname(x)      Getattr(x,"$symname")
+#define Geterror(x)        Getattr(x,"error")
 
-#define Settype(x,v)       Setattr(x,"type",x)
+#define Settype(x,v)       Setattr(x,"type",v)
 #define Setname(x,v)       Setattr(x,"name",v)
 #define Setlname(x,v)      Setattr(x,"lname",v)
 #define Setvalue(x,v)      Setattr(x,"value", v)
 #define Setignore(x,v)     SetInt(x,"ignore",v)
-
-#define Getnext(x)         Getattr(x,"next")
-#define Setnext(x,n)       Setattr(x,"next",n)
+#define Setparms(x,v)      Setattr(x,"parms", v)
+#define Setdecl(x,d)       Setattr(x,"decl",d)
+#define Setstorage(x,s)    Setattr(x,"storage",s)
+#define Seterror(x,e)      Setattr(x,"error",e)
+#define Setsymname(x,s)    Setattr(x,"$symname",s)
+#define Getnext(x)         Getattr(x,"nextSibling")
 
 #endif
 
