@@ -9,7 +9,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/typemap.c,v 1.6.2.1 2001/06/15 13:28:50 mkoeppe Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/typemap.c,v 1.6.2.1.2.3 2001/08/23 21:18:58 beazley Exp $";
 
 #include "swig.h"
 
@@ -71,7 +71,7 @@ Swig_typemap_pop_scope() {
  * ----------------------------------------------------------------------------- */
 
 void
-Swig_typemap_register(char *op, SwigType *type, String_or_char *pname, String_or_char *code, ParmList *locals) {
+Swig_typemap_register(const String_or_char *op, SwigType *type, String_or_char *pname, String_or_char *code, ParmList *locals) {
   char *key;
   Hash *tm;
   Hash *tm1;
@@ -148,7 +148,7 @@ Swig_typemap_get(SwigType *type, String_or_char *name, int scope) {
  * ----------------------------------------------------------------------------- */
 
 void
-Swig_typemap_copy(char *op, SwigType *stype, String_or_char *sname, 
+Swig_typemap_copy(const String_or_char *op, SwigType *stype, String_or_char *sname, 
 		  SwigType *ttype, String_or_char *tname) {
 
   Hash *tm=0, *tm1;
@@ -173,7 +173,7 @@ Swig_typemap_copy(char *op, SwigType *stype, String_or_char *sname,
  * ----------------------------------------------------------------------------- */
 
 void
-Swig_typemap_clear(char *op, SwigType *type, String_or_char *name) {
+Swig_typemap_clear(const String_or_char *op, SwigType *type, String_or_char *name) {
   Hash *tm;
 
   tm = Swig_typemap_get(type,name,tm_scope);
@@ -269,7 +269,7 @@ static SwigType *strip_arrays(SwigType *type) {
  * ----------------------------------------------------------------------------- */
 
 Hash *
-Swig_typemap_search(char *op, SwigType *type, String_or_char *name) {
+Swig_typemap_search(const String_or_char *op, SwigType *type, String_or_char *name) {
   Hash *result = 0, *tm, *tm1, *tma;
   SwigType *noarrays = 0;
   SwigType *primitive = 0;
@@ -370,31 +370,43 @@ static void typemap_locals(SwigType *t, String_or_char *pname, DOHString *s, Par
     if (pn) {
       if (Len(pn) > 0) {
 	DOHString *str;
-	SwigType  *tt;
 
 	str = NewString("");
 	/* If the user gave us $type as the name of the local variable, we'll use
 	   the passed datatype instead */
 
-	if (Cmp(SwigType_base(pt),"$type")==0 || Cmp(SwigType_base(pt),"$basetype")==0) {
-	  tt = t;
-	} else {
-	  tt = pt;
-	}
 	Printf(str,"%s",pn);
 	/* Substitute parameter names */
 	Replace(str,"$arg",pname, DOH_REPLACE_ANY);
-        if (Cmp(SwigType_base(pt),"$basetype")==0) {
-          /* use $basetype */
-	  new_name = Wrapper_new_localv(f,str,SwigType_base(tt),str,0);
-        } else {
-          new_name = Wrapper_new_localv(f,str, SwigType_str(tt,str), 0);
+        if (Cmp(SwigType_base(pt),"$basetype")==0) /* use $basetype */
+	  new_name = Wrapper_new_localv(f,str,SwigType_base(t),str,0);
+	else if (Cmp(SwigType_base(pt), "$type") == 0
+		 || Cmp(SwigType_base(pt), "$ltype") == 0) 
+	  new_name = Wrapper_new_localv(f,str,SwigType_str(Swig_clocal_type(t), 0),str,0);
+	else if ((Cmp(SwigType_base(pt), "$*type") == 0
+		  || Cmp(SwigType_base(pt), "$*ltype") == 0)
+		 && SwigType_ispointer(t)) {
+	  SwigType *star_type = Copy(t);
+	  SwigType_del_pointer(star_type);
+	  new_name = Wrapper_new_localv(f,str,SwigType_str(Swig_clocal_type(star_type), 0),
+					str, 0);
+	  Delete(star_type);
 	}
+	else if (Cmp(SwigType_base(pt), "$&type") == 0
+		 || Cmp(SwigType_base(pt), "$&ltype") == 0) {
+	  SwigType *amp_type = Copy(t);
+	  SwigType_add_pointer(amp_type);
+	  new_name = Wrapper_new_localv(f,str,SwigType_str(Swig_clocal_type(amp_type), 0),
+					str, 0);
+	  Delete(amp_type);
+	}
+	else 
+          new_name = Wrapper_new_localv(f,str, SwigType_str(pt,str), 0);
 	/* Substitute  */
 	Replace(s,pn,new_name,DOH_REPLACE_ID);
       }
     }
-    p = Getnext(p);
+    p = nextSibling(p);
   }
 }
 
@@ -404,7 +416,7 @@ static void typemap_locals(SwigType *t, String_or_char *pname, DOHString *s, Par
  * Perform a typemap lookup (ala SWIG1.1)
  * ----------------------------------------------------------------------------- */
 
-char *Swig_typemap_lookup(char *op, SwigType *type, String_or_char *pname, String_or_char *source,
+char *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_char *pname, String_or_char *source,
 			  String_or_char *target, Wrapper *f) 
 {
   Hash   *tm;
@@ -448,18 +460,64 @@ char *Swig_typemap_lookup(char *op, SwigType *type, String_or_char *pname, Strin
     String *tmname = Getattr(tm,"typemap");
     if (tmname) Replace(s,"$typemap",tmname, DOH_REPLACE_ANY);
   }
-  Replace(s,"$type",SwigType_str(type,0),DOH_REPLACE_ANY);
-  {
-    SwigType *ltype = Swig_clocal_type(type);
-    Replace(s,"$ltype",SwigType_str(ltype,0), DOH_REPLACE_ANY);
-    Delete(ltype);
-  }
   Replace(s,"$parmname",pname, DOH_REPLACE_ANY);
 
-  /* Print base type (without any pointers) */
-  Replace(s,"$basetype",SwigType_base(type), DOH_REPLACE_ANY);
-  Replace(s,"$basemangle",SwigType_manglestr(SwigType_base(type)), DOH_REPLACE_ANY);
-  Replace(s,"$mangle",SwigType_manglestr(type), DOH_REPLACE_ANY);
+  /* Type-related stuff */
+  {
+    SwigType *star_type, *amp_type, *base_type;
+    SwigType *ltype, *star_ltype, *amp_ltype, *base_ltype;
+    String *mangle, *star_mangle, *amp_mangle, *base_mangle;
+    String *descriptor, *star_descriptor, *amp_descriptor;
+
+    /* Given type */
+    Replace(s, "$type", SwigType_str(type, 0), DOH_REPLACE_ANY);
+    ltype = Swig_clocal_type(type);
+    Replace(s, "$ltype", SwigType_str(ltype, 0), DOH_REPLACE_ANY);
+    mangle = SwigType_manglestr(type);
+    Replace(s, "$mangle", mangle, DOH_REPLACE_ANY);
+    descriptor = NewStringf("SWIGTYPE%s", mangle);
+    if (Replace(s, "$descriptor",
+		descriptor, DOH_REPLACE_ANY))
+      SwigType_remember(type);
+    Delete(descriptor);
+    /* One pointer level removed */
+    if (SwigType_ispointer(type)) {
+      star_type = Copy(type);
+      SwigType_del_pointer(star_type);
+      Replace(s, "$*type", SwigType_str(star_type, 0), DOH_REPLACE_ANY);
+      star_ltype = Swig_clocal_type(star_type);
+      Replace(s, "$*ltype", SwigType_str(star_ltype, 0), DOH_REPLACE_ANY);
+      star_mangle = SwigType_manglestr(star_type);
+      Replace(s, "$*mangle", star_mangle, DOH_REPLACE_ANY);
+      star_descriptor = NewStringf("SWIGTYPE%s", star_mangle);
+      if (Replace(s, "$*descriptor",
+		  star_descriptor, DOH_REPLACE_ANY))
+	SwigType_remember(star_type);
+      Delete(star_descriptor);
+    }
+    else {
+      /* TODO: Signal error if one of the $* substitutions is
+	 requested */
+    }
+    /* One pointer level added */
+    amp_type = Copy(type);
+    SwigType_add_pointer(amp_type);
+    Replace(s, "$&type", SwigType_str(amp_type, 0), DOH_REPLACE_ANY);
+    amp_ltype = Swig_clocal_type(amp_type);
+    Replace(s, "$&ltype", SwigType_str(amp_ltype, 0), DOH_REPLACE_ANY);
+    amp_mangle = SwigType_manglestr(amp_type);
+    Replace(s, "$&mangle", amp_mangle, DOH_REPLACE_ANY);
+    amp_descriptor = NewStringf("SWIGTYPE%s", amp_mangle);
+    if (Replace(s, "$&descriptor",
+		amp_descriptor, DOH_REPLACE_ANY))
+      SwigType_remember(amp_type);
+    Delete(amp_descriptor);
+    /* Base type */
+    base_type = SwigType_base(type);
+    Replace(s,"$basetype", base_type, DOH_REPLACE_ANY);
+    base_mangle = SwigType_manglestr(base_type);
+    Replace(s,"$basemangle", base_mangle, DOH_REPLACE_ANY);
+  }
   Swig_temp_result(s);
   return Char(s);
 }
