@@ -14,7 +14,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/wrapfunc.c,v 1.15.4.3 2002/01/02 20:27:26 cheetah Exp $";
+static char cvsroot[] = "$Header: /cvs/projects/SWIG/Source/Swig/wrapfunc.c,v 1.15.4.1 2001/09/20 13:03:53 beazley Exp $";
 
 #include "swig.h"
 #include <ctype.h>
@@ -33,6 +33,9 @@ NewWrapper() {
   w->locals = NewString("");
   w->code = NewString("");
   w->def = NewString("");
+  w->_type = 0;
+  w->_parms = 0;
+  w->_name = 0;
   return w;
 }
 
@@ -48,6 +51,9 @@ DelWrapper(Wrapper *w) {
   Delete(w->locals);
   Delete(w->code);
   Delete(w->def);
+  Delete(w->_type);
+  Delete(w->_parms);
+  Delete(w->_name);
   free(w);
 }
 
@@ -124,7 +130,7 @@ Wrapper_pretty_print(String *str, File *f) {
       c = Getc(str);
       if (c != EOF) {
 	Putc(c,ts);
-	if (c == '/') {         /* C++ comment */
+	if (c == '/') {                     /* C++ comment */
 	  while ((c = Getc(str)) != EOF) {
 	    if (c == '\n') {
 	      Ungetc(c,str);
@@ -132,21 +138,7 @@ Wrapper_pretty_print(String *str, File *f) {
 	    }
 	    Putc(c,ts);
 	  }
-	} else if (c == '*') {  /* C comment */
-          int endstar = 0;
-	  while ((c = Getc(str)) != EOF) {
-	    if (endstar && c == '/') {  /* end of C comment */
-	      Putc(c,ts);
-	      break;
-	    }
-            endstar = (c == '*');
-	    Putc(c,ts);
-            if (c == '\n') { /* multi-line C comment. Could be improved slightly. */
-              for (i = 0; i < level; i++)
-	        Putc(' ',ts);
-            }
-	  }
-        }
+	}
       }
     } else {
       if (!empty || !isspace(c)) {
@@ -301,6 +293,64 @@ Wrapper_new_localv(Wrapper *w, const String_or_char *name, ...) {
   return ret;
 }
 
+/* -----------------------------------------------------------------------------
+ * Wrapper_Gettype()
+ * ----------------------------------------------------------------------------- */
+
+SwigType *
+Wrapper_Gettype(Wrapper *w) {
+  return w->_type;
+}
+
+/* -----------------------------------------------------------------------------
+ * Wrapper_Settype()
+ * ----------------------------------------------------------------------------- */
+
+void
+Wrapper_Settype(Wrapper *w, SwigType *t) {
+  Delete(w->_type);
+  w->_type = NewString(t);
+}
+
+/* -----------------------------------------------------------------------------
+ * Wrapper_Getparms()
+ * ----------------------------------------------------------------------------- */
+
+ParmList *
+Wrapper_Getparms(Wrapper *w) {
+  return w->_parms;
+}
+
+/* -----------------------------------------------------------------------------
+ * Wrapper_Setparms()
+ * ----------------------------------------------------------------------------- */
+
+void
+Wrapper_Setparms(Wrapper *w, ParmList *l) {
+  Delete(w->_parms);
+  w->_parms = l;
+  DohIncref(l);
+}
+
+/* -----------------------------------------------------------------------------
+ * Wrapper_Getname()
+ * ----------------------------------------------------------------------------- */
+
+char *
+Wrapper_Getname(Wrapper *w) {
+  return Char(w->_name);
+}
+
+/* -----------------------------------------------------------------------------
+ * Wrapper_Setname()
+ * ----------------------------------------------------------------------------- */
+
+void
+Wrapper_Setname(Wrapper *w, String_or_char *n) {
+  Delete(w->_name);
+  w->_name = NewString(n);
+}
+
 
 
 
